@@ -24,15 +24,13 @@
  * This is mod_rewrite, contrib software for proftpd 1.2 and above.
  * For more information contact TJ Saunders <tj@castaglia.org>.
  *
- * $Id: mod_rewrite.c,v 1.4 2003/01/02 18:25:13 castaglia Exp $
+ * $Id: mod_rewrite.c,v 1.16 2003/08/06 22:03:32 castaglia Exp $
  */
 
 #include "conf.h"
 #include "privs.h"
 
-#include <sys/ioctl.h>
-
-#define MOD_REWRITE_VERSION "mod_rewrite/0.6.5"
+#define MOD_REWRITE_VERSION "mod_rewrite/0.6.7"
 
 /* Make sure the version of proftpd is as necessary. */
 #if PROFTPD_VERSION_NUMBER < 0x0001020801
@@ -40,13 +38,33 @@
 #endif
 
 #ifdef HAVE_REGEX_H
-#include <regex.h>
+# include <regex.h>
 
 #define REWRITE_FIFO_MAXLEN		256
 #define REWRITE_LOG_MODE		0640
 #define REWRITE_MAX_MATCHES		10
 #define REWRITE_U32_BITS		0xffffffff
 
+/* RewriteCondition operations */
+typedef enum {
+  REWRITE_COND_OP_REGEX = 1,
+  REWRITE_COND_OP_LEX_LT,
+  REWRITE_COND_OP_LEX_GT,
+  REWRITE_COND_OP_LEX_EQ,
+  REWRITE_COND_OP_TEST_DIR,
+  REWRITE_COND_OP_TEST_FILE,
+  REWRITE_COND_OP_TEST_SYMLINK,
+  REWRITE_COND_OP_TEST_SIZE
+} rewrite_cond_op_t;
+
+/* RewriteCondition flags */
+#define REWRITE_COND_FLAG_NOCASE	0x001	/* nocase|NC */
+#define REWRITE_COND_FLAG_ORNEXT	0x002	/* ornext|OR */
+
+/* RewriteRule flags */
+#define REWRITE_RULE_FLAG_NOCASE	0x001	/* nocase|NC */
+#define REWRITE_RULE_FLAG_LAST		0x002	/* last|L */
+
 /* Module structures */
 typedef struct {
   pool *map_pool;
@@ -76,6 +94,8 @@ static char *rewrite_logfile = NULL;
 static int rewrite_logfd = -1;
 static pool *rewrite_pool = NULL;
 static pool *rewrite_cond_pool = NULL;
+
+static unsigned int rewrite_nrules = 0;
 static array_header *rewrite_conds = NULL, *rewrite_regexes = NULL;
 static rewrite_match_t rewrite_cond_matches;
 static rewrite_match_t rewrite_rule_matches;
@@ -101,14 +121,17 @@ static char rewrite_vars[REWRITE_MAX_VARS][3] = {
 
 /* Necessary prototypes
  */
+static char *rewrite_argsep(char **);
 static void rewrite_closelog(void);
 static char *rewrite_expand_var(cmd_rec *, const char *, const char *);
 static void rewrite_log(char *format, ...);
 static unsigned char rewrite_match_cond(cmd_rec *, config_rec *);
 static void rewrite_openlog(void);
 static unsigned char rewrite_open_fifo(config_rec *);
+static unsigned int rewrite_parse_cond_flags(pool *, const char *);
 static unsigned char rewrite_parse_map_str(char *, rewrite_map_t *);
 static unsigned char rewrite_parse_map_txt(rewrite_map_txt_t *);
+static unsigned int rewrite_parse_rule_flags(pool *, const char *);
 static int rewrite_read_fifo(int, char *, size_t);
 static regex_t *rewrite_regalloc(void);
 static unsigned char rewrite_regexec(const char *, regex_t *, unsigned char,
@@ -185,13 +208,13 @@ static char *rewrite_expand_var(cmd_rec *cmd, const char *subst_pattern,
     return session.user;
 
   } else if (!strcmp(var, "%a")) {
-    return inet_ascii(cmd->tmp_pool, session.c->remote_ipaddr);
+    return (char *) pr_netaddr_get_ipstr(session.c->remote_addr);
 
   } else if (!strcmp(var, "%h")) {
-    return session.c->remote_name;
+    return (char *) session.c->remote_name;
 
   } else if (!strcmp(var, "%v")) {
-    return main_server->ServerName;
+    return (char *) main_server->ServerName;
 
   } else if (!strcmp(var, "%G")) {
 
@@ -222,17 +245,213 @@ static char *rewrite_expand_var(cmd_rec *cmd, const char *subst_pattern,
   return NULL;
 }
 
+static char *rewrite_argsep(char **arg) {
+  char *res = NULL, *dst = NULL;
+  char quote_mode = 0;
+
+  if (!arg || !*arg || !**arg)
+    return NULL;
+
+  while (**arg && isspace((int) **arg))
+    (*arg)++;
+
+  if (!**arg)
+    return NULL;
+
+  res = dst = *arg;
+
+  if (**arg == '\"') {
+    quote_mode++;
+    (*arg)++;
+  }
+
+  while (**arg && **arg != ',' &&
+      (quote_mode ? (**arg != '\"') : (!isspace((int) **arg)))) {
+
+    if (**arg == '\\' && quote_mode) {
+
+      /* escaped char */
+      if (*((*arg) + 1))
+        *dst = *(++(*arg));
+    }
+
+    *dst++ = **arg;
+    ++(*arg);
+  }
+
+  if (**arg)
+    (*arg)++;
+
+  *dst = '\0';
+  return res;
+}
+
+static unsigned int rewrite_parse_cond_flags(pool *p, const char *flags_str) {
+  char *opt = NULL, *str = NULL, **opts;
+  array_header *opt_list = NULL;
+  unsigned int flags = 0;
+  register unsigned int i = 0;
+
+  opt_list = make_array(p, 0, sizeof(char **));
+
+  /* Make a duplicate of the given string, as the argsep() function consumes
+   * the string.
+   */
+  str = pstrdup(p, flags_str);
+
+  /* Skip past the first [ in the string, and trim the last ]. */
+  str++;
+  str[strlen(str)-1] = '\0';
+
+  while ((opt = rewrite_argsep(&str)) != NULL)
+    *((char **) push_array(opt_list)) = pstrdup(p, opt);
+
+  opts = opt_list->elts;
+  for (i = 0; i < opt_list->nelts; i++) {
+    if (strcmp(opts[i], "nocase") == 0 || strcmp(opts[i], "NC") == 0)
+      flags |= REWRITE_COND_FLAG_NOCASE;
+  
+    else if (strcmp(opts[i], "ornext") == 0 || strcmp(opts[i], "OR") == 0)
+      flags |= REWRITE_COND_FLAG_ORNEXT;
+  }
+
+  return flags;
+}
+
+static unsigned int rewrite_parse_rule_flags(pool *p, const char *flags_str) {
+  char *opt = NULL, *str = NULL, **opts;
+  array_header *opt_list = NULL;
+  unsigned int flags = 0;
+  register unsigned int i = 0;
+
+  opt_list = make_array(p, 0, sizeof(char **));
+
+  /* Make a duplicate of the given string, as the argsep() function consumes
+   * the string.
+   */
+  str = pstrdup(p, flags_str);
+
+  /* Skip past the first [ in the string, and trim the last ]. */
+  str++;
+  str[strlen(str)-1] = '\0';
+
+  while ((opt = rewrite_argsep(&str)) != NULL)
+    *((char **) push_array(opt_list)) = pstrdup(p, opt);
+
+  opts = opt_list->elts;
+  for (i = 0; i < opt_list->nelts; i++) {
+    if (strcmp(opts[i], "nocase") == 0 || strcmp(opts[i], "NC") == 0)
+      flags |= REWRITE_RULE_FLAG_NOCASE;
+
+    else if (strcmp(opts[i], "last") == 0 || strcmp(opts[i], "L") == 0)
+      flags |= REWRITE_RULE_FLAG_LAST;
+  }
+
+  return flags;
+}
+
 static unsigned char rewrite_match_cond(cmd_rec *cmd, config_rec *cond) {
   char *cond_str = cond->argv[0];
+  unsigned char negated = *((unsigned char *) cond->argv[2]);
+  rewrite_cond_op_t cond_op = *((rewrite_cond_op_t *) cond->argv[3]);
+
   rewrite_log("rewrite_match_cond(): original cond: '%s'", cond_str);
 
   cond_str = rewrite_subst(cmd, cond->argv[0]);
   rewrite_log("rewrite_match_cond: subst'd cond: '%s'", cond_str);
 
   /* Check the condition */
-  rewrite_cond_matches.match_string = cond->argv[0];
-  return rewrite_regexec(cond_str, (regex_t *) cond->argv[1],
-    *((unsigned char *) cond->argv[2]), &rewrite_cond_matches);
+  switch (cond_op) {
+    case REWRITE_COND_OP_LEX_LT: {
+      int res = strcmp(cond_str, (char *) cond->argv[1]);
+      rewrite_log("rewrite_match_cond(): checking lexical LT cond");
+
+      if (!negated)
+        return (res < 0 ? TRUE : FALSE);
+      else
+        return (res < 0 ? FALSE : TRUE);
+    }
+
+    case REWRITE_COND_OP_LEX_GT: {
+      int res = strcmp(cond_str, (char *) cond->argv[1]);
+      rewrite_log("rewrite_match_cond(): checking lexical GT cond");
+
+      if (!negated)
+        return (res > 0 ? TRUE : FALSE);
+      else
+        return (res > 0 ? FALSE : TRUE);
+    }
+
+    case REWRITE_COND_OP_LEX_EQ: {
+      int res = strcmp(cond_str, (char *) cond->argv[1]);
+      rewrite_log("rewrite_match_cond(): checking lexical EQ cond");
+
+      if (!negated)
+        return (res == 0 ? TRUE : FALSE);
+      else
+        return (res == 0 ? FALSE : TRUE);
+    }
+
+    case REWRITE_COND_OP_REGEX: {
+      rewrite_log("rewrite_match_cond(): checking regex cond");
+
+      memset(&rewrite_cond_matches, '\0', sizeof(rewrite_cond_matches));
+      rewrite_cond_matches.match_string = cond->argv[0];
+      return rewrite_regexec(cond_str, (regex_t *) cond->argv[1], negated,
+        &rewrite_cond_matches);
+    }
+
+    case REWRITE_COND_OP_TEST_DIR: {
+      struct stat st;
+      rewrite_log("rewrite_match_cond(): checking dir test cond");
+
+      pr_fs_clear_cache();
+      if (pr_fsio_lstat(cond_str, &st) >= 0 && S_ISDIR(st.st_mode))
+        return TRUE;
+
+      return FALSE;
+    }
+
+    case REWRITE_COND_OP_TEST_FILE: {
+      struct stat st;
+      rewrite_log("rewrite_match_cond(): checking file test cond");
+
+      pr_fs_clear_cache();
+      if (pr_fsio_lstat(cond_str, &st) >= 0 && S_ISREG(st.st_mode))
+        return TRUE;
+
+      return FALSE;
+    }
+
+    case REWRITE_COND_OP_TEST_SYMLINK: {
+      struct stat st;
+      rewrite_log("rewrite_match_cond(): checking symlink test cond");
+
+      pr_fs_clear_cache();
+      if (pr_fsio_lstat(cond_str, &st) >= 0 && S_ISLNK(st.st_mode))
+        return TRUE;
+
+      return FALSE;
+    }
+
+    case REWRITE_COND_OP_TEST_SIZE: {
+      struct stat st;
+      rewrite_log("rewrite_match_cond(): checking size test cond");
+
+      pr_fs_clear_cache();
+      if (pr_fsio_lstat(cond_str, &st) >= 0 && S_ISREG(st.st_mode) &&
+          st.st_size > 0)
+        return TRUE;
+
+      return FALSE;
+    }
+
+    default:
+      rewrite_log("rewrite_match_cond(): unknown cond op: %d", cond_op);
+      break;
+  }
+
+  return FALSE;
 }
 
 static unsigned char rewrite_parse_map_str(char *str, rewrite_map_t *map) {
@@ -341,6 +560,8 @@ static unsigned char rewrite_parse_map_txt(rewrite_map_txt_t *txtmap) {
     unsigned int key_so = 0, key_eo = 0;
     unsigned int val_so = 0, val_eo = 0;
 
+    pr_signals_handle();
+
     /* Skip leading whitespace. */
     for (pos = 0; pos < linelen && isspace(linebuf[pos]); pos++);
 
@@ -446,11 +667,11 @@ static unsigned char rewrite_regexec(const char *string, regex_t *regbuf,
     have_match = TRUE;
     break;
 
+#if 0
     /* If negated, be done */
-/*
     if (negated)
       break;
-*/
+#endif
   }
 
   /* Invert the return value if necessary. */
@@ -490,7 +711,7 @@ static char *rewrite_subst_backrefs(cmd_rec *cmd, char *pattern,
   register unsigned int i = 0;
   char *new_pattern = NULL;
 
-  for (i = 1; i < REWRITE_MAX_MATCHES; i++) {
+  for (i = 0; i < REWRITE_MAX_MATCHES; i++) {
     if (matches->match_groups[i].rm_so != -1) {
       char buf[3] = {'\0'}, tmp;
 
@@ -502,16 +723,17 @@ static char *rewrite_subst_backrefs(cmd_rec *cmd, char *pattern,
         /* Substitute "%N" backreferences for RewriteCondition matches */
         snprintf(buf, sizeof(buf), "%%%u", i);
 
-      /* Make sure there's a backreference for this in the substitution
-       * pattern.  Otherwise, just continue on.
-       */
-      if (!strstr(pattern, buf))
-        continue;
-
       if (!new_pattern)
         new_pattern = pstrdup(cmd->pool, pattern);
 
-      tmp = (matches->match_string)[matches->match_groups[i].rm_eo] = '\0';
+      /* Make sure there's a backreference for this in the substitution
+       * pattern.  Otherwise, just continue on.
+       */
+      if (!strstr(new_pattern, buf))
+        continue;
+
+      tmp = (matches->match_string)[matches->match_groups[i].rm_eo];
+      (matches->match_string)[matches->match_groups[i].rm_eo] = '\0';
 
       rewrite_log("rewrite_subst_backrefs(): replacing backref '%s' with '%s'",
         buf, &(matches->match_string)[matches->match_groups[i].rm_so]);
@@ -609,6 +831,10 @@ static char *rewrite_subst_maps_fifo(cmd_rec *cmd, config_rec *c,
   char *value = NULL, *fifo_lockname = NULL;
   const char *fifo = (char *) c->argv[2];
 
+#ifndef HAVE_FLOCK
+  struct flock lock;
+#endif /* HAVE_FLOCK */
+
   /* The FIFO file descriptor should already be open. */
   fifo_fd = *((int *) c->argv[3]);
 
@@ -632,8 +858,28 @@ static char *rewrite_subst_maps_fifo(cmd_rec *cmd, config_rec *c,
   }
 
   /* Obtain a write lock on the lock file, if configured */
-  if (fifo_lockfd != -1)
-    flock(fifo_lockfd, LOCK_EX);
+  if (fifo_lockfd != -1) {
+#ifdef HAVE_FLOCK
+    if (flock(fifo_lockfd, LOCK_EX) < 0)
+      rewrite_log("rewrite_subst_maps_fifo(): error obtaining lock: %s",
+        strerror(errno));
+#else
+    lock.l_type = F_WRLCK;
+    lock.l_whence = 0;
+    lock.l_start = lock.l_len = 0;
+
+    while (fcntl(fifo_lockfd, F_SETLKW, &lock) < 0) {
+      if (errno == EINTR) {
+        pr_signals_handle();
+        continue;
+      }
+
+      rewrite_log("rewrite_subst_maps_fifo(): error obtaining lock: %s",
+        strerror(errno));
+      break;
+    }
+#endif /* HAVE_FLOCK */
+  }
 
   /* Write the lookup key to the FIFO.  There is no need to restrict the size
    * of data written to the FIFO to the PIPE_BUF length; the advisory lock on
@@ -659,7 +905,27 @@ static char *rewrite_subst_maps_fifo(cmd_rec *cmd, config_rec *c,
       "FIFO '%s': %s", map->map_lookup_key, fifo, strerror(errno));
 
     if (fifo_lockfd != -1) {
-      flock(fifo_lockfd, LOCK_UN);
+#ifdef HAVE_FLOCK
+      if (flock(fifo_lockfd, LOCK_UN) < 0)
+        rewrite_log("rewrite_subst_maps_fifo(): error releasing lock: %s",
+          strerror(errno));
+#else
+      lock.l_type = F_UNLCK;
+      lock.l_whence = 0;
+      lock.l_start = lock.l_len = 0;
+
+      while (fcntl(fifo_lockfd, F_SETLKW, &lock) < 0) {
+        if (errno == EINTR) {
+          pr_signals_handle();
+          continue;
+        }
+
+        rewrite_log("rewrite_subst_maps_fifo(): error releasing lock: %s",
+          strerror(errno));
+        break;
+      }
+#endif /* HAVE_FLOCK */
+
       close(fifo_lockfd);
     }
 
@@ -710,7 +976,27 @@ static char *rewrite_subst_maps_fifo(cmd_rec *cmd, config_rec *c,
   fsync(fifo_fd);
 
   if (fifo_lockfd != -1) {
-    flock(fifo_lockfd, LOCK_UN);
+#ifdef HAVE_FLOCK
+    if (flock(fifo_lockfd, LOCK_UN) < 0)
+      rewrite_log("rewrite_subst_maps_fifo(): error releasing lock: %s",
+        strerror(errno));
+#else
+    lock.l_type = F_UNLCK;
+    lock.l_whence = 0;
+    lock.l_start = lock.l_len = 0;
+
+    while (fcntl(fifo_lockfd, F_SETLKW, &lock) < 0) {
+      if (errno == EINTR) {
+        pr_signals_handle();
+        continue;
+      }
+
+      rewrite_log("rewrite_subst_maps_fifo(): error releasing lock: %s",
+        strerror(errno));
+      break;
+    }
+#endif /* HAVE_FLOCK */
+
     close(fifo_lockfd);
   }
 
@@ -1203,16 +1489,19 @@ static void rewrite_log(char *format, ...) {
 /* Configuration directive handlers
  */
 
-/* usage: RewriteCondition condition pattern */
+/* usage: RewriteCondition condition pattern [flags] */
 MODRET set_rewritecondition(cmd_rec *cmd) {
   config_rec *c = NULL;
   pool *cond_pool = NULL;
-  regex_t *preg = NULL;
+  void *cond_data = NULL;
+  unsigned int cond_flags = 0;
   char *var = NULL;
   unsigned char negated = FALSE;
-  int res = -1;
+  rewrite_cond_op_t cond_op = 0;
+  int regex_flags = REG_EXTENDED, res = -1;
 
-  CHECK_ARGS(cmd, 2);
+  if (cmd->argc-1 < 2 || cmd->argc-1 > 3)
+    CONF_ERROR(cmd, "bad number of parameters");
   CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON|CONF_DIR);
 
   /* The following variables are not allowed in RewriteConditions:
@@ -1221,22 +1510,21 @@ MODRET set_rewritecondition(cmd_rec *cmd) {
   if (strstr(cmd->argv[2], "%P") || strstr(cmd->argv[2], "%t"))
     CONF_ERROR(cmd, "illegal RewriteCondition variable used");
 
-  preg = rewrite_regalloc();
+  CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON|CONF_DIR);
 
-  /* Check for a leading '!' negation prefix to the regex pattern */
-  if (*cmd->argv[2] == '!') {
-    cmd->argv[2]++;
-    negated = TRUE;
-  }
+  /* Make sure that, if present, the flags parameter is correctly formatted. */
+  if (cmd->argc-1 == 3) {
+    if (cmd->argv[3][0] != '[' || cmd->argv[3][strlen(cmd->argv[3])-1] != ']')
+      CONF_ERROR(cmd, pstrcat(cmd->tmp_pool,
+        ": badly formatted flags parameter: '", cmd->argv[3], "'", NULL));
 
-  if ((res = regcomp(preg, cmd->argv[2], REG_EXTENDED)) != 0) {
-    char errstr[200] = {'\0'};
+    /* We need to parse the flags parameter here, to see if any flags which
+     * affect the compilation of the regex (e.g. NC) are present.
+     */
+    cond_flags = rewrite_parse_cond_flags(cmd->tmp_pool, cmd->argv[3]);
 
-    regerror(res, preg, errstr, sizeof(errstr));
-    regfree(preg);
-
-    CONF_ERROR(cmd, pstrcat(cmd->tmp_pool, "unable to compile '",
-      cmd->argv[2], "' regex: ", errstr, NULL));
+    if (cond_flags & REWRITE_COND_FLAG_NOCASE)
+      regex_flags |= REG_ICASE;
   }
 
   if (!rewrite_conds) {
@@ -1247,6 +1535,54 @@ MODRET set_rewritecondition(cmd_rec *cmd) {
     rewrite_conds = make_array(rewrite_cond_pool, 0, sizeof(config_rec *));
   }
 
+  /* Check for a leading '!' negation prefix to the regex pattern */
+  if (*cmd->argv[2] == '!') {
+    cmd->argv[2]++;
+    negated = TRUE;
+  }
+
+  /* Check the next character in the given pattern.  It may be a lexical
+   * or a file test pattern...
+   */
+  if (*cmd->argv[2] == '>') {
+    cond_op = REWRITE_COND_OP_LEX_LT;
+    cond_data = pstrdup(rewrite_pool, ++cmd->argv[2]);
+
+  } else if (*cmd->argv[2] == '<') {
+    cond_op = REWRITE_COND_OP_LEX_GT;
+    cond_data = pstrdup(rewrite_pool, ++cmd->argv[2]);
+
+  } else if (*cmd->argv[2] == '=') {
+    cond_op = REWRITE_COND_OP_LEX_EQ;
+    cond_data = pstrdup(rewrite_pool, ++cmd->argv[2]);
+
+  } else if (strcmp(cmd->argv[2], "-d") == 0) {
+    cond_op = REWRITE_COND_OP_TEST_DIR;
+
+  } else if (strcmp(cmd->argv[2], "-f") == 0) {
+    cond_op = REWRITE_COND_OP_TEST_FILE;
+
+  } else if (strcmp(cmd->argv[2], "-l") == 0) {
+    cond_op = REWRITE_COND_OP_TEST_SYMLINK;
+
+  } else if (strcmp(cmd->argv[2], "-s") == 0) {
+    cond_op = REWRITE_COND_OP_TEST_SIZE;
+
+  } else {
+    cond_op = REWRITE_COND_OP_REGEX;
+    cond_data = rewrite_regalloc();
+
+    if ((res = regcomp(cond_data, cmd->argv[2], regex_flags)) != 0) {
+      char errstr[200] = {'\0'};
+
+      regerror(res, cond_data, errstr, sizeof(errstr));
+      regfree(cond_data);
+
+      CONF_ERROR(cmd, pstrcat(cmd->tmp_pool, "unable to compile '",
+        cmd->argv[2], "' regex: ", errstr, NULL));
+    }
+  }
+
   /* Make sure the variables used, if any, are valid. */
   var = cmd->argv[1];
   while (*var != '\0' && (var = strchr(var, '%')) != NULL &&
@@ -1278,14 +1614,20 @@ MODRET set_rewritecondition(cmd_rec *cmd) {
   c->pool = cond_pool;
   c->name = pstrdup(c->pool, cmd->argv[0]);
   c->config_type = CONF_PARAM;
-  c->argc = 3;
-  c->argv = pcalloc(c->pool, 4 * sizeof(void *));
+  c->argc = 5;
+  c->argv = pcalloc(c->pool, (c->argc+1) * sizeof(void *));
   c->argv[0] = pstrdup(c->pool, cmd->argv[1]); 
-  c->argv[1] = (void *) preg;
+  c->argv[1] = (void *) cond_data;
 
   c->argv[2] = palloc(c->pool, sizeof(unsigned char));
   *((unsigned char *) c->argv[2]) = negated;
 
+  c->argv[3] = pcalloc(c->pool, sizeof(rewrite_cond_op_t));
+  *((rewrite_cond_op_t *) c->argv[3]) = cond_op;
+
+  c->argv[4] = pcalloc(c->pool, sizeof(unsigned int));
+  *((unsigned int *) c->argv[4]) = cond_flags;
+
   /* Add this config_rec to an array, to be added to the RewriteRule when
    * (if?) it appears.
    */
@@ -1447,18 +1789,36 @@ MODRET set_rewritemap(cmd_rec *cmd) {
   return HANDLED(cmd);
 }
 
-/* usage: RewriteRule pattern substitution */
+/* usage: RewriteRule pattern substitution [flags] */
 MODRET set_rewriterule(cmd_rec *cmd) {
   config_rec *c = NULL;
   regex_t *preg = NULL;
-  char *tmp = NULL;
+  unsigned int rule_flags = 0;
   unsigned char negated = FALSE;
-  int res = -1;
+  int regex_flags = REG_EXTENDED, res = -1;
+  char *tmp = NULL;
   register unsigned int i = 0;
 
-  CHECK_ARGS(cmd, 2);
+  if (cmd->argc-1 < 2 || cmd->argc-1 > 3)
+    CONF_ERROR(cmd, "bad number of parameters");
+
   CHECK_CONF(cmd, CONF_ROOT|CONF_VIRTUAL|CONF_GLOBAL|CONF_ANON|CONF_DIR);
 
+  /* Make sure that, if present, the flags parameter is correctly formatted. */
+  if (cmd->argc-1 == 3) {
+    if (cmd->argv[3][0] != '[' || cmd->argv[3][strlen(cmd->argv[3])-1] != ']')
+      CONF_ERROR(cmd, pstrcat(cmd->tmp_pool,
+        ": badly formatted flags parameter: '", cmd->argv[3], "'", NULL));
+
+    /* We need to parse the flags parameter here, to see if any flags which
+     * affect the compilation of the regex (e.g. NC) are present.
+     */
+    rule_flags = rewrite_parse_rule_flags(cmd->tmp_pool, cmd->argv[3]);
+
+    if (rule_flags & REWRITE_RULE_FLAG_NOCASE)
+      regex_flags |= REG_ICASE;
+  }
+
   preg = rewrite_regalloc();
 
   /* Check for a leading '!' prefix, signifying regex negation */
@@ -1467,7 +1827,7 @@ MODRET set_rewriterule(cmd_rec *cmd) {
     cmd->argv[1]++;
   }
 
-  if ((res = regcomp(preg, cmd->argv[1], REG_EXTENDED)) != 0) {
+  if ((res = regcomp(preg, cmd->argv[1], regex_flags)) != 0) {
     char errstr[200] = {'\0'};
 
     regerror(res, preg, errstr, sizeof(errstr));
@@ -1477,14 +1837,14 @@ MODRET set_rewriterule(cmd_rec *cmd) {
       cmd->argv[1], "' regex: ", errstr, NULL));
   }
 
-  /* NOTE: scan through the substitution parameter, checking any given
+  /* Note: scan through the substitution parameter, checking any given
    * variables, and noting if any invalid sequences are found
    */
   tmp = NULL;
 
-  c = add_config_param(cmd->argv[0], 4, preg, NULL, NULL, NULL);
+  c = add_config_param(cmd->argv[0], 6, preg, NULL, NULL, NULL, NULL, NULL);
 
-  /* NOTE: how to handle the substitution expression? Later? */
+  /* Note: how to handle the substitution expression? Later? */
   c->argv[1] = pstrdup(c->pool, cmd->argv[2]);
 
   c->argv[2] = palloc(c->pool, sizeof(unsigned char));
@@ -1517,8 +1877,16 @@ MODRET set_rewriterule(cmd_rec *cmd) {
   } else
     c->argv[3] = NULL;
 
-  c->flags |= CF_MERGEDOWN_MULTI;
+  c->argv[4] = pcalloc(c->pool, sizeof(unsigned int));
+  *((unsigned int *) c->argv[4]) = rule_flags;
 
+  /* The very last slot is to be filled by a unique ID (just a counter
+   * value).
+   */
+  c->argv[5] = pcalloc(c->pool, sizeof(unsigned int));
+  *((unsigned int *) c->argv[5]) = rewrite_nrules++;
+
+  c->flags |= CF_MERGEDOWN_MULTI;
   return HANDLED(cmd);
 }
 
@@ -1527,6 +1895,7 @@ MODRET set_rewriterule(cmd_rec *cmd) {
 
 MODRET rewrite_fixup(cmd_rec *cmd) {
   config_rec *c = NULL;
+  array_header *seen_rules = NULL;
 
   /* Is RewriteEngine on? */
   if (!rewrite_engine)
@@ -1540,6 +1909,11 @@ MODRET rewrite_fixup(cmd_rec *cmd) {
     return DECLINED(cmd);
   }
 
+  /* Create an array that will contain the IDs of the RewriteRules we've
+   * already processed.
+   */
+  seen_rules = make_array(cmd->tmp_pool, 0, sizeof(unsigned int));
+
   /* Find all RewriteRules in effect. */
   c = find_config(CURRENT_CONF, CONF_PARAM, "RewriteRule", FALSE);
 
@@ -1547,9 +1921,31 @@ MODRET rewrite_fixup(cmd_rec *cmd) {
     unsigned char exec_rule = FALSE;
     rewrite_log("rewrite_fixup(): found RewriteRule");
 
-    /* First, make sure the given RewriteRule regex matches the command
-     * argument.
-     */
+    /* If we've already seen this Rule, skip on to the next Rule. */
+    if (seen_rules->nelts > 0) {
+      register unsigned int i = 0;
+      unsigned char saw_rule = FALSE;
+      unsigned int id = *((unsigned int *) c->argv[5]), *ids = seen_rules->elts;
+
+      for (i = 0; i < seen_rules->nelts; i++) {
+        if (ids[i] == id) {
+          saw_rule = TRUE;
+          break;
+        }
+      }
+
+      if (saw_rule) {
+        rewrite_log("rewrite_fixup(): already saw this RewriteRule, skipping");
+        c = find_config_next(c, c->next, CONF_PARAM, "RewriteRule", FALSE);
+        continue;
+      }
+    }
+
+    /* Add this Rule's ID to the list of seen Rules. */
+    *((unsigned int *) push_array(seen_rules)) = *((unsigned int *) c->argv[5]);
+
+    /* Make sure the given RewriteRule regex matches the command argument. */
+    memset(&rewrite_rule_matches, '\0', sizeof(rewrite_rule_matches));
     rewrite_rule_matches.match_string = cmd->arg;
     if (!rewrite_regexec(cmd->arg, (regex_t *) c->argv[0],
         *((unsigned char *) c->argv[2]), &rewrite_rule_matches)) {
@@ -1567,26 +1963,39 @@ MODRET rewrite_fixup(cmd_rec *cmd) {
         register unsigned int i = 0;
         config_rec **conds = (config_rec **) c->argv[3];
 
-        rewrite_log("rewrite_fixup(): examing RewriteRule conditions");
+        rewrite_log("rewrite_fixup(): examining RewriteRule conditions");
         exec_rule = TRUE;
 
         for (i = 0; conds[i] != NULL; i++) {
+          unsigned int cond_flags = *((unsigned int *) conds[i]->argv[4]);
+
           if (!rewrite_match_cond(cmd, conds[i])) {
-            rewrite_log("rewrite_fixup(): condition not met, skipping this "
-              "Rule");
-            exec_rule = FALSE;
-            break;
+
+            /* If this condition is not to be OR'd with the next condition,
+             * or there is no next condition, fail the Rule.
+             */
+            if (!(cond_flags & REWRITE_COND_FLAG_ORNEXT) ||
+                conds[i+1] == NULL) {
+              exec_rule = FALSE;
+              rewrite_log("rewrite_fixup(): condition not met, skipping this "
+                "Rule");
+              break;
+            }
 
           } else
             rewrite_log("rewrite_fixup(): condition met");
         }
-      }
+
+      } else
+        /* There are no conditions. */
+        exec_rule = TRUE;
     } 
 
     if (exec_rule) {
       char *new_arg = NULL;
-      rewrite_log("rewrite_fixup(): executing RewriteRule");
+      unsigned int rule_flags = *((unsigned int *) c->argv[4]);
 
+      rewrite_log("rewrite_fixup(): executing RewriteRule");
       new_arg = rewrite_subst(cmd, (char *) c->argv[1]);
 
       if (strlen(new_arg) > 0) {
@@ -1595,6 +2004,13 @@ MODRET rewrite_fixup(cmd_rec *cmd) {
 
       } else
         rewrite_log("rewrite_fixup(): error processing RewriteRule");
+
+      /* If this Rule is marked as "last", break out of the loop. */
+      if (rule_flags & REWRITE_RULE_FLAG_LAST) {
+        rewrite_log("rewrite_fixup(): Rule marked as 'last', done processing "
+          "Rules");
+        break;
+      }
     }
 
     c = find_config_next(c, c->next, CONF_PARAM, "RewriteRule", FALSE);
@@ -1639,11 +2055,13 @@ static int rewrite_sess_init(void) {
     FALSE);
 
   /* Is RewriteEngine on? */
-  if (engine && (rewrite_engine = *engine) != TRUE) {
+  if (!engine || *engine == FALSE) {
     rewrite_engine = FALSE;
     return 0;
   }
 
+  rewrite_engine = TRUE;
+
   /* Open the RewriteLog, if present. */
   rewrite_openlog();
 
@@ -1683,7 +2101,7 @@ static int rewrite_init(void) {
 
   /* Allocate a pool for this module's use. */
   if (!rewrite_pool)
-    rewrite_pool = make_named_sub_pool(permanent_pool, "mod_rewrite pool");
+    rewrite_pool = make_sub_pool(permanent_pool);
 
   /* Add a rehash handler. */
   pr_rehash_register_handler(NULL, rewrite_rehash_cb);
@@ -1736,4 +2154,4 @@ module rewrite_module = {
   rewrite_sess_init
 };
 
-#endif
+#endif /* HAVE_REGEX_H */
