@@ -1,6 +1,5 @@
 /*
- * ProFTPD: mod_mysql -- Support for connecting to MySQL databases.
- * Time-stamp: <1999-10-04 03:21:21 root>
+ * ProFTPD: mod_sql_mysql -- Support for connecting to MySQL databases.
  * Copyright (c) 2001 Andrew Houghton
  *  
  * This program is free software; you can redistribute it and/or modify
@@ -21,6 +20,8 @@
  * holders give permission to link this program with OpenSSL, and distribute
  * the resulting executable, without including the source code for OpenSSL in
  * the source distribution.
+ *
+ * $Id: mod_sql_mysql.c,v 1.27 2003/10/01 14:21:31 castaglia Exp $
  */
 
 /*
@@ -129,11 +130,11 @@
  * Internal define used for debug and logging.  All backends are encouraged
  * to use the same format.
  */
-#define _MOD_VERSION "mod_sql_mysql/4.03"
+#define _MOD_VERSION "mod_sql_mysql/4.04"
 
 #define _MYSQL_PORT "3306"
 
-#include <mysql/mysql.h>
+#include <mysql.h>
 #include "conf.h"
 #include "../contrib/mod_sql.h"
 
@@ -191,8 +192,8 @@ typedef struct conn_entry_struct conn_entry_t;
 
 #define DEF_CONN_POOL_SIZE 10
 
-array_header *conn_cache;
-pool *conn_pool;
+pool *conn_pool = NULL;
+array_header *conn_cache = NULL;
 
 /*
  *  _sql_get_connection: walks the connection cache looking for the named
@@ -281,7 +282,7 @@ static int _sql_timer_callback(CALLBACK_FRAME)
       sql_log(DEBUG_INFO, "timer expired for connection '%s'", entry->name);
       cmd = _sql_make_cmd( conn_pool, 2, entry->name, "1" );
       cmd_close( cmd );
-      _sql_free_cmd( cmd );
+      SQL_FREE_CMD(cmd);
       entry->timer = 0;
     }
   }
@@ -305,7 +306,7 @@ static void _sql_shutdown(void)
     if (entry->connections > 0) {
       cmd = _sql_make_cmd( conn_pool, 2, entry->name, "1" );
       cmd_close( cmd );
-      _sql_free_cmd( cmd );
+      SQL_FREE_CMD(cmd);
     }
   }
 
@@ -321,8 +322,7 @@ static void _sql_shutdown(void)
  *  If a backend database does not provide error information, please use
  *  the PR_ERR_SQL_UNKNOWN macro.
  */
-static modret_t *_build_error( cmd_rec *cmd, db_conn_t *conn )
-{
+static modret_t *_build_error(cmd_rec *cmd, db_conn_t *conn) {
   char num[20] = {'\0'};
 
   if (!conn)
@@ -330,7 +330,7 @@ static modret_t *_build_error( cmd_rec *cmd, db_conn_t *conn )
 
   snprintf(num, 20, "%u", mysql_errno(conn->mysql) );
 
-  return ERROR_MSG(cmd, num, mysql_error(conn->mysql));
+  return ERROR_MSG(cmd, num, (char *) mysql_error(conn->mysql));
 }
 
 /*
@@ -339,8 +339,7 @@ static modret_t *_build_error( cmd_rec *cmd, db_conn_t *conn )
  *  that data.  This is MySQL specific; other backends may choose 
  *  to do things differently.
  */
-static modret_t *_build_data( cmd_rec *cmd, db_conn_t *conn )
-{
+static modret_t *_build_data(cmd_rec *cmd, db_conn_t *conn) {
   modret_t *mr = NULL;
   MYSQL *mysql = NULL;
   MYSQL_RES *result = NULL;
@@ -409,8 +408,7 @@ static modret_t *_build_data( cmd_rec *cmd, db_conn_t *conn )
  *  compatibility; a connection should not be closed unless the count
  *  reaches 0, and ideally will not need to be re-opened for counts > 1.
  */
-MODRET cmd_open(cmd_rec *cmd)
-{
+MODRET cmd_open(cmd_rec *cmd) {
   conn_entry_t *entry = NULL;
   db_conn_t *conn = NULL;
 
@@ -446,9 +444,9 @@ MODRET cmd_open(cmd_rec *cmd)
     return HANDLED(cmd);
   }
 
-  /* make sure we have a new conn struct */
+  /* Make sure we have a new conn struct */
   conn->mysql = mysql_init(NULL);
-  
+
   if (!conn->mysql) {
     log_pri(PR_LOG_ERR, _MOD_VERSION ": failed to allocate memory for "
 	    "MYSQL structure.  Shutting down.");
@@ -457,13 +455,18 @@ MODRET cmd_open(cmd_rec *cmd)
     end_login(1);
   }
 
-  if (!mysql_real_connect(conn->mysql, conn->host, conn->user,
-			  conn->pass, conn->db, 
-			  (int) strtol(conn->port, (char **) NULL, 10), 
-			  NULL, 0)) {
-    /* if it didn't work, return an error */
+  /* Make sure the MySQL config files are read in.  This will read in
+   * options from group "client" in the MySQL .cnf files.
+   */
+  mysql_options(conn->mysql, MYSQL_READ_DEFAULT_GROUP, "client");
+
+  if (!mysql_real_connect(conn->mysql, conn->host, conn->user, conn->pass,
+      conn->db, (int) strtol(conn->port, (char **) NULL, 10), NULL,
+      CLIENT_INTERACTIVE)) {
+
+    /* If it didn't work, return an error. */
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_open");
-    return _build_error( cmd, conn );
+    return _build_error(cmd, conn);
   }
 
   /* bump connections */
@@ -486,7 +489,7 @@ MODRET cmd_open(cmd_rec *cmd)
   /* return HANDLED */
   sql_log(DEBUG_INFO, "connection '%s' opened", entry->name);
 
-  sql_log(DEBUG_INFO, ": connection '%s' count is now %d", entry->name,
+  sql_log(DEBUG_INFO, "connection '%s' count is now %d", entry->name,
     entry->connections);
 
   sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_open");
@@ -755,7 +758,7 @@ MODRET cmd_select(cmd_rec *cmd)
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_select");
     return PR_ERR_SQL_UNDEF(cmd);
   }
-  
+ 
   conn = (db_conn_t *) entry->data;
 
   cmr = cmd_open(cmd);
@@ -803,7 +806,7 @@ MODRET cmd_select(cmd_rec *cmd)
 
     close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
     cmd_close(close_cmd);
-    _sql_free_cmd( close_cmd );
+    SQL_FREE_CMD(close_cmd);
 
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_select");
     return dmr;
@@ -818,7 +821,7 @@ MODRET cmd_select(cmd_rec *cmd)
 
     close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
     cmd_close(close_cmd);
-    _sql_free_cmd( close_cmd );
+    SQL_FREE_CMD(close_cmd);
 
     return dmr;
   }    
@@ -826,7 +829,7 @@ MODRET cmd_select(cmd_rec *cmd)
   /* close the connection, return the data. */
   close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
   cmd_close(close_cmd);
-  _sql_free_cmd( close_cmd );
+  SQL_FREE_CMD(close_cmd);
 
   sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_select");
   return dmr;
@@ -917,7 +920,7 @@ MODRET cmd_insert(cmd_rec *cmd)
 
     close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
     cmd_close(close_cmd);
-    _sql_free_cmd( close_cmd );
+    SQL_FREE_CMD(close_cmd);
 
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_insert");
     return dmr;
@@ -926,7 +929,7 @@ MODRET cmd_insert(cmd_rec *cmd)
   /* close the connection and return HANDLED. */
   close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
   cmd_close(close_cmd);
-  _sql_free_cmd( close_cmd );
+  SQL_FREE_CMD(close_cmd);
 
   sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_insert");
   return HANDLED(cmd);
@@ -1016,7 +1019,7 @@ MODRET cmd_update(cmd_rec *cmd)
 
     close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
     cmd_close(close_cmd);
-    _sql_free_cmd( close_cmd );
+    SQL_FREE_CMD(close_cmd);
 
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_update");
     return dmr;
@@ -1025,7 +1028,7 @@ MODRET cmd_update(cmd_rec *cmd)
   /* close the connection, return HANDLED.  */
   close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
   cmd_close(close_cmd);
-  _sql_free_cmd( close_cmd );
+  SQL_FREE_CMD(close_cmd);
 
   sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_update");
   return HANDLED(cmd);
@@ -1132,7 +1135,7 @@ MODRET cmd_query(cmd_rec *cmd)
     
     close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
     cmd_close(close_cmd);
-    _sql_free_cmd( close_cmd );
+    SQL_FREE_CMD(close_cmd);
     
     sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_query");
     return dmr;
@@ -1154,7 +1157,7 @@ MODRET cmd_query(cmd_rec *cmd)
   /* close the connection, return the data. */
   close_cmd = _sql_make_cmd( cmd->tmp_pool, 1, entry->name );
   cmd_close(close_cmd);
-  _sql_free_cmd( close_cmd );
+  SQL_FREE_CMD(close_cmd);
 
   sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_query");
   return dmr;
@@ -1182,8 +1185,9 @@ MODRET cmd_query(cmd_rec *cmd)
  *  the database documentation and figure it out) to do the conversion
  *  themselves in this function.
  *
- *  At the very least, a backend MUST simply copy the data from argv[0]
- *  into the data field of the modret.
+ *  A backend MUST supply a working escapestring implementation.  Simply
+ *  copying the data from argv[0] into the data field of the modret allows
+ *  for possible SQL injection attacks when this backend is used.
  */
 MODRET cmd_escapestring(cmd_rec * cmd)
 {
@@ -1211,18 +1215,17 @@ MODRET cmd_escapestring(cmd_rec * cmd)
   conn = (db_conn_t *) entry->data;
 
   unescaped = cmd->argv[1];
-  escaped = (char *) pcalloc( cmd->tmp_pool, sizeof(char) * 
-			      (strlen(unescaped) * 2) + 1);
+  escaped = (char *) pcalloc(cmd->tmp_pool, sizeof(char) *
+    (strlen(unescaped) * 2) + 1);
 
   /* Note: the mysql_real_escape_string() function appeared in the C API
    * as of MySQL 3.23.14; this macro allows functioning with older mysql
    * installations.
    */
 #if MYSQL_VERSION_ID >= 32314
-  mysql_real_escape_string( conn->mysql, escaped, unescaped, 
-			    strlen(unescaped) );
+  mysql_real_escape_string(conn->mysql, escaped, unescaped, strlen(unescaped));
 #else
-  mysql_escape_string( escaped, unescaped, strlen(unescaped) );
+  mysql_escape_string(escaped, unescaped, strlen(unescaped));
 #endif
 
   sql_log(DEBUG_FUNC, "%s", "exiting \tmysql cmd_escapestring");
@@ -1243,7 +1246,7 @@ MODRET cmd_escapestring(cmd_rec * cmd)
  *  HANDLED(cmd)                        -- passwords match
  *  ERROR_INT(cmd, PR_AUTH_NOPWD)       -- missing password
  *  ERROR_INT(cmd, PR_AUTH_BADPWD)      -- passwords don't match
- *  ERROR_INT(cmd, PR_AUTH_DISABLEPWD) -- password is disabled
+ *  ERROR_INT(cmd, PR_AUTH_DISABLEPWD)  -- password is disabled
  *  ERROR_INT(cmd, PR_AUTH_AGEPWD)      -- password is aged
  *  ERROR(cmd)                          -- unknown error
  *
@@ -1286,7 +1289,11 @@ MODRET cmd_checkauth(cmd_rec * cmd)
   c_clear = cmd->argv[1];
   c_hash = cmd->argv[2];
 
-  make_scrambled_password( scrambled, c_clear );
+#if MYSQL_VERSION_ID >= 40100
+  make_scrambled_password(scrambled, c_clear, 1, NULL);
+#else
+  make_scrambled_password(scrambled, c_clear);
+#endif
 
   success = !strcmp(scrambled, c_hash); 
 
@@ -1359,9 +1366,13 @@ cmdtable sql_cmdtable[] = {
  *  the exit handler.
  */
 static int sql_mysql_init(void) {
-  conn_pool  = make_sub_pool(session.pool);
-  conn_cache = make_array(session.pool, DEF_CONN_POOL_SIZE,
-			  sizeof(conn_entry_t));
+
+  if (!conn_pool)
+    conn_pool = make_sub_pool(session.pool);
+
+  if (!conn_cache)
+    conn_cache = make_array(make_sub_pool(session.pool), DEF_CONN_POOL_SIZE,
+      sizeof(conn_entry_t));
 
   pr_exit_register_handler( _sql_shutdown );
 
