@@ -25,7 +25,7 @@
  */
 
 /* Generic configuration and standard header file includes.
- * $Id: conf.h,v 1.31 2003/02/25 18:56:39 castaglia Exp $
+ * $Id: conf.h,v 1.42 2003/09/11 06:36:11 castaglia Exp $
  */
 
 #ifndef PR_CONF_H
@@ -40,12 +40,6 @@
 
 #include "default_paths.h"
 
-/* This is no longer used, autoconf provides us with a top level
- * config.h
- */
-/* #include "platform.h" */
-#include "options.h"
-
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
@@ -137,6 +131,14 @@ char *strchr(),*strrchr();
 # include <netinet/in.h>
 #endif
 
+#ifdef HAVE_NETINET_IN_SYSTM_H
+# include <netinet/in_systm.h>
+#endif
+
+#ifdef HAVE_NETINET_IP_H
+# include <netinet/ip.h>
+#endif
+
 #ifdef HAVE_NETINET_TCP_H
 # include <netinet/tcp.h>
 #endif
@@ -179,6 +181,9 @@ char *strchr(),*strrchr();
 #endif
 
 #ifdef HAVE_SYS_IOCTL_H
+# ifdef SOLARIS2
+#  define BSD_COMP 1
+# endif
 # include <sys/ioctl.h>
 #endif
 
@@ -215,6 +220,12 @@ char *strchr(),*strrchr();
 # endif /* HAVE_SYS_TERMIO_H */
 #endif /* HAVE_SYS_TERMIOS_H */
 
+/* The tunable options header needs to be included after all the system headers,
+ * so that limits are picked up properly.
+ */
+
+#include "options.h"
+
 /* Solaris 2.5 seems to already have a typedef for 'timer_t', so
  * #define timer_t to something else as a workaround.
  */
@@ -271,11 +282,6 @@ off_t lseek(int, off_t, int);
 extern int optopt;
 #endif
 
-/* GNU HURD (and maybe others) do not define MAXPATHLEN */
-#ifndef MAXPATHLEN
-#define MAXPATHLEN      1024
-#endif
-
 /* Necessary for alloca to work */
 #if !defined(__alloca) && !defined(__GNU_LIBRARY__)
 # ifdef __GNUC__
@@ -299,7 +305,7 @@ extern char *alloca();
 
 #endif
 
-#if defined(_FILE_OFFSET_BITS) || defined(_LARGE_FILES) || SIZEOF_OFF_T == 8
+#if defined(HAVE_LLU) && SIZEOF_OFF_T == 8
 # define PR_LU	"llu"
 #else
 # define PR_LU	"lu"
@@ -316,9 +322,44 @@ extern char *alloca();
  * not for the support library.
  */
 
-/* Generic typedefs */
+#ifndef INET_ADDRSTRLEN
+# define INET_ADDRSTRLEN        16
+#endif /* INET_ADDRSTRLEN */
 
-typedef struct in_addr p_in_addr_t;
+#ifndef INET6_ADDRSTRLEN
+# define INET6_ADDRSTRLEN       46
+#endif /* INET6_ADDRSTRLEN */
+
+typedef struct {
+  int na_family;
+
+  /* Note: this assumes that DNS names have a maximum size of
+   * 256 characters.
+   */
+  char na_dnsstr[256];
+  int na_have_dnsstr;
+
+#ifdef USE_IPV6
+  char na_ipstr[INET6_ADDRSTRLEN];
+#else
+  char na_ipstr[INET_ADDRSTRLEN];
+#endif /* USE_IPV6 */
+  int na_have_ipstr;
+
+  /* Note: at some point, this union might/should be replaced with
+   * struct sockaddr_storage.  Why?  The sockaddr_storage struct is
+   * better defined to be aligned on OS/arch boundaries, for more efficient
+   * allocation/access.
+   */
+
+  union {
+    struct sockaddr_in v4;
+#ifdef USE_IPV6
+    struct sockaddr_in6 v6;
+#endif /* USE_IPV6 */
+  } na_addr;
+
+} pr_netaddr_t;
 
 #include "pool.h"
 #include "regexp.h"
@@ -331,6 +372,7 @@ typedef struct in_addr p_in_addr_t;
 #include "response.h"
 #include "timers.h"
 #include "inet.h"
+#include "netaddr.h"
 #include "bindings.h"
 #include "ident.h"
 #include "feat.h"
@@ -351,6 +393,15 @@ typedef struct in_addr p_in_addr_t;
 #  define setgrent()	setgroupent(1)
 # endif /* HAVE_SETGROUPENT */
 
+/* Define a buffer size to use for responses, making sure it is big enough
+ * to handle large path names (e.g. for MKD responses).
+ */
+#define PR_RESPONSE_BUFFER_SIZE	 (PR_TUNABLE_BUFFER_SIZE + PR_TUNABLE_PATH_MAX)
+
 #endif /* __PROFTPD_SUPPORT_LIBRARY */
 
+#ifdef WITH_DMALLOC
+# include <dmalloc.h>
+#endif /* WITH_DMALLOC */
+
 #endif /* PR_CONF_H */
