@@ -26,7 +26,7 @@
 
 /* ProFTPD module definitions.
  *
- * $Id: modules.h,v 1.27 2003/01/17 16:46:23 castaglia Exp $
+ * $Id: modules.h,v 1.32 2003/09/08 00:31:32 castaglia Exp $
  */
 
 #ifndef PR_MODULES_H
@@ -118,29 +118,24 @@ typedef struct {
 /* classes for command table */
 
 #define CL_NONE				0x0
-#define CL_AUTH				(1 << 0)
-/* USER, PASS */
-#define CL_INFO				(1 << 1)
-/* Informational commands (PWD, SYST, etc) */
-#define CL_DIRS				(1 << 2)
-/* Directory commands (LIST, NLST, CWD, etc) */
-#define CL_READ				(1 << 3)
-/* File reading commands (RETR) */
-#define CL_WRITE			(1 << 4)
-/* Writing commands (STOR, MKD, etc) */
-#define CL_MISC				(1 << 5)
-/* Miscellaneous (RNFR/RNTO, SITE, etc) */
+#define CL_AUTH				(1 << 0) /* USER, PASS */
+#define CL_INFO				(1 << 1) /* Informational commands (PWD, SYST, etc) */
+#define CL_DIRS				(1 << 2) /* Directory commands (LIST, NLST, CWD, etc) */
+#define CL_READ				(1 << 3) /* File reading commands (RETR) */
+#define CL_WRITE			(1 << 4) /* Writing commands (STOR, MKD, etc) */
+#define CL_MISC				(1 << 5) /* Miscellaneous (RNFR/RNTO, SITE, etc) */
+
 #define CL_ALL				(CL_AUTH|CL_INFO|CL_DIRS|CL_READ| \
 					CL_WRITE|CL_MISC)
 
-/* command types for command table */
-
+/* Command handler types for command table */
 #define PRE_CMD				1
 #define CMD				2
 #define POST_CMD			3
 #define POST_CMD_ERR			4
 #define LOG_CMD				5
 #define LOG_CMD_ERR			6
+#define HOOK				7
 
 typedef struct {
 
@@ -184,20 +179,9 @@ struct module_struc {
   int (*module_init_session_cb)(void);	/* Session initialization */
 
   /* Internal use; high number == higher priority. */
-  int auth_priority, priority;
+  int priority;
 };
 
-/* module hash types */
-typedef struct authsym {
-  struct authsym *next, *prev;
-
-  /* pointer to the auth symbol */
-  char *name;
-
-  module *module;
-  authtable *table;
-} pr_authsym_t;
-
 /* These are stored in modules.c */
 
 extern conftable *m_conftable;			/* Master conftable */
@@ -213,24 +197,26 @@ unsigned char module_exists(const char *);
 void list_modules(void);
 
 /* Module daemon-startup initialization */
+int module_daemon_startup(void);
 int module_preparse_init(void);
 int module_postparse_init(void);
 int module_session_init(void);
+void module_remove_daemon_startups(void);
 void module_remove_postparse_inits(void);
+void pr_register_daemon_startup(int (*)(void));
 void pr_register_postparse_init(int (*)(void));
 
 modret_t *call_module(module *, modret_t *(*)(cmd_rec *), cmd_rec *);
-modret_t *call_module_cmd(module *, modret_t *(*)(cmd_rec *), cmd_rec *);
-modret_t *call_module_auth(module *, modret_t *(*)(cmd_rec *), cmd_rec *);
 
 /* Symbol table hash ("stash") support. */
 typedef enum {
   PR_SYM_CONF = 1,
   PR_SYM_CMD,
-  PR_SYM_AUTH
+  PR_SYM_AUTH,
+  PR_SYM_HOOK
 } pr_stash_type_t;
 
-int pr_init_stash(void);
+int init_stash(void);
 int pr_stash_add_symbol(pr_stash_type_t, void *);
 void *pr_stash_get_symbol(pr_stash_type_t, const char *, void *, int *);
 int pr_stash_remove_symbol(pr_stash_type_t, const char *, module *);
