@@ -24,12 +24,24 @@
  * the source code for OpenSSL in the source distribution.
  */
 
-/* $Id: privs.h,v 1.17 2003/02/05 21:14:26 castaglia Exp $
+/* $Id: privs.h,v 1.21 2003/09/19 20:21:31 castaglia Exp $
  */
 
 #ifndef PR_PRIVS_H
 #define PR_PRIVS_H
 
+/* Definition of root user/group IDs (non-Unix platforms may have these as
+ * different from 0/0).
+ */
+
+#ifdef __CYGWIN__
+# define PR_ROOT_UID    18
+# define PR_ROOT_GID    544
+#else
+# define PR_ROOT_UID    0
+# define PR_ROOT_GID    0
+#endif /* __CYGWIN__ */
+
 /* Macros for manipulating saved, real and effective uid for easy
  * switching from/to root.
  *
@@ -46,18 +58,36 @@
  */
 
 #ifdef __hpux
-#define setreuid(x,y) setresuid(x,y,0)
-#endif
+# define setreuid(x,y) setresuid(x,y,0)
+#endif /* __hpux */
 
-#if !defined(HAVE_SETEUID)
+#ifdef PR_DEVEL_COREDUMP
+/* Unix kernels can be notoriously picky about dumping the core for
+ * processes that have fiddled with their effective/actual UID and GID.
+ * So, to make it possible for people to have their proftpd processes
+ * actually be able to coredump, these PRIVS macros, which switch
+ * privileges, are effectively disabled.
+ *
+ * Hence it is not a Good Idea to run a proftpd built with PR_DEVEL_COREDUMP
+ * defined in production.
+ */
+
+# define PRIVS_SETUP(u, g)
+# define PRIVS_ROOT
+# define PRIVS_USER
+# define PRIVS_RELINQUISH
+# define PRIVS_REVOKE
+
+#else
+
+# if !defined(HAVE_SETEUID)
 
 /* Use setreuid() to perform uid swapping.
  */
 
-#define PRIVS_SETUP(u, g) \
-  { \
+#  define PRIVS_SETUP(u, g) { \
     log_debug(DEBUG9, "SETUP PRIVS at %s:%d", __FILE__, __LINE__); \
-    if (getuid()) { \
+    if (getuid() != PR_ROOT_UID) { \
       session.ouid = session.uid = getuid(); \
       session.gid = getgid(); \
       if (setgid(session.gid)) \
@@ -73,30 +103,31 @@
       if (setgid(session.gid)) \
         log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setgid(): %s", \
           strerror(errno)); \
-      if (setreuid(0, session.uid)) \
+      if (setreuid(PR_ROOT_UID, session.uid)) \
         log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setreuid(): %s", \
           strerror(errno)); \
     } \
   }
 
-#define PRIVS_ROOT \
-  { \
+#  define PRIVS_ROOT { \
     log_debug(DEBUG9, "ROOT PRIVS at %s:%d", __FILE__, __LINE__); \
     if (!session.disable_id_switching) { \
-      if (setreuid(session.uid, 0)) \
+      if (setregid(session.gid, PR_ROOT_GID)) \
+        log_pri(PR_LOG_ERR, "PRIVS_ROOT: unable to setregid(): %s", \
+          strerror(errno)); \
+      if (setreuid(session.uid, PR_ROOT_UID)) \
         log_pri(PR_LOG_ERR, "PRIVS_ROOT: unable to setreuid(): %s", \
           strerror(errno)); \
     } else \
       log_debug(DEBUG9, "ROOT PRIVS: ID switching disabled"); \
   }
 
-#define PRIVS_USER \
-  { \
+#  define PRIVS_USER { \
     log_debug(DEBUG9, "USER PRIVS %d at %s:%d", (int) session.login_uid, \
       __FILE__, __LINE__); \
     if (!session.disable_id_switching) { \
-      if (setreuid(session.uid,0)) \
-        log_pri(PR_LOG_ERR, "PRIVS_USER: unable to setreuid(session.uid, 0): %s", \
+      if (setreuid(session.uid, PR_ROOT_UID)) \
+        log_pri(PR_LOG_ERR, "PRIVS_USER: unable to setreuid(session.uid, PR_ROOT_UID): %s", \
           strerror(errno)); \
       if (setreuid(session.uid, session.login_uid)) \
         log_pri(PR_LOG_ERR, "PRIVS_USER: unable to setreuid(session.uid, " \
@@ -105,14 +136,21 @@
       log_debug(DEBUG9, "ROOT PRIVS: ID switching disabled"); \
   }
 
-#define PRIVS_RELINQUISH  \
-  { \
+#  define PRIVS_RELINQUISH  { \
     log_debug(DEBUG9, "RELINQUISH PRIVS at %s:%d", __FILE__, __LINE__); \
     if (!session.disable_id_switching) { \
-      if (geteuid() != 0) { \
-        if (setreuid(session.uid, 0)) \
+      if (getegid() != PR_ROOT_GID) { \
+        if (setregid(session.gid, PR_ROOT_GID)) \
           log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to " \
-            "setreuid(session.uid, 0): %s", strerror(errno)); \
+            "setregid(session.gid, PR_ROOT_GID): %s", strerror(errno)); \
+      } \
+      if (setregid(session.gid, session.gid)) \
+        log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to setregid(session.jid, " \
+          "session.gid): %s", strerror(errno)); \
+      if (geteuid() != PR_ROOT_UID) { \
+        if (setreuid(session.uid, PR_ROOT_UID)) \
+          log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to " \
+            "setreuid(session.uid, PR_ROOT_UID): %s", strerror(errno)); \
       } \
       if (setreuid(session.uid, session.uid)) \
         log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to setreuid(session.uid, " \
@@ -121,11 +159,10 @@
       log_debug(DEBUG9, "ROOT PRIVS: ID switching disabled"); \
   }
 
-#define PRIVS_REVOKE \
-  { \
+#  define PRIVS_REVOKE { \
     log_debug(DEBUG9, "REVOKE PRIVS at %s:%d", __FILE__, __LINE__); \
-    if (setreuid(0, 0)) \
-      log_pri(PR_LOG_ERR, "PRIVS_REVOKE: unable to setreuid(0, 0): %s", \
+    if (setreuid(PR_ROOT_UID, PR_ROOT_UID)) \
+      log_pri(PR_LOG_ERR, "PRIVS_REVOKE: unable to setreuid(PR_ROOT_UID, PR_ROOT_UID): %s", \
         strerror(errno)); \
     if (setgid(session.gid)) \
       log_pri(PR_LOG_ERR, "PRIVS_REVOKE: unable to setgid(): %s", \
@@ -135,7 +172,7 @@
         strerror(errno)); \
   }
 
-#else /* HAVE_SETEUID */
+# else /* HAVE_SETEUID */
 
 /* Set the saved uid/gid using setuid/seteuid().  setreuid() is
  * no longer used as it is considered obsolete on many systems.
@@ -147,10 +184,9 @@
  *   real/eff/saved group : <group>
  */
 
-#define PRIVS_SETUP(u, g) \
-  { \
+#  define PRIVS_SETUP(u, g) { \
     log_debug(DEBUG9, "SETUP PRIVS at %s:%d", __FILE__, __LINE__); \
-    if (getuid()) { \
+    if (getuid() != PR_ROOT_UID) { \
       session.ouid = session.uid = getuid(); \
       session.gid = getgid(); \
       if (setgid(session.gid)) \
@@ -166,7 +202,7 @@
       session.ouid = getuid(); \
       session.uid = (u); \
       session.gid = (g); \
-      if (setuid(0)) \
+      if (setuid(PR_ROOT_UID)) \
         log_pri(PR_LOG_ERR, "PRIVS_SETUP: unable to setuid(): %s", \
           strerror(errno)); \
       if (setgid((g))) \
@@ -180,27 +216,30 @@
 
 /* Switch back to root privs.
  */
-#define PRIVS_ROOT \
+#  define PRIVS_ROOT \
   if (!session.disable_id_switching) { \
     log_debug(DEBUG9, "ROOT PRIVS at %s:%d", __FILE__, __LINE__); \
-    if (seteuid(0)) \
+    if (seteuid(PR_ROOT_UID)) \
       log_pri(PR_LOG_ERR, "PRIVS_ROOT: unable to seteuid(): %s", \
         strerror(errno)); \
+    if (setegid(PR_ROOT_GID)) \
+      log_pri(PR_LOG_ERR, "PRIVS_ROOT: unable to setegid(): %s", \
+        strerror(errno)); \
   } else \
     log_debug(DEBUG9, "ROOT PRIVS: ID switching disabled");
 
 /* Switch to the privs of the login user.
  */
-#define PRIVS_USER \
+#  define PRIVS_USER \
   if (!session.disable_id_switching) { \
-    if (session.login_uid == 0) { \
+    if (session.login_uid == PR_ROOT_UID) { \
       log_debug(DEBUG1, "Use of PRIVS_USER before session.login_uid set " \
         "in %s %d", __FILE__, __LINE__); \
     } else { \
       log_debug(DEBUG9, "USER PRIVS %d at %s:%d", (int) session.login_uid, \
         __FILE__, __LINE__); \
-      if (seteuid(0)) \
-        log_pri(PR_LOG_ERR, "PRIVS_USER: unable to seteuid(0): %s", \
+      if (seteuid(PR_ROOT_UID)) \
+        log_pri(PR_LOG_ERR, "PRIVS_USER: unable to seteuid(PR_ROOT_UID): %s", \
           strerror(errno)); \
       if (seteuid(session.login_uid)) \
         log_pri(PR_LOG_ERR, "PRIVS_USER: unable to seteuid(session.login_uid): " \
@@ -211,14 +250,17 @@
 
 /* Relinquish privs granted by PRIVS_ROOT or PRIVS_USER.
  */
-#define PRIVS_RELINQUISH \
+#  define PRIVS_RELINQUISH \
   if (!session.disable_id_switching) { \
-    if (geteuid() != 0) { \
-      if (seteuid(0)) \
-        log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to seteuid(0): %s", \
+    if (geteuid() != PR_ROOT_UID) { \
+      if (seteuid(PR_ROOT_UID)) \
+        log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to seteuid(PR_ROOT_UID): %s", \
           strerror(errno)); \
     } \
     log_debug(DEBUG9, "RELINQUISH PRIVS at %s:%d", __FILE__, __LINE__); \
+    if (setegid(session.gid)) \
+      log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to setegid(session.gid): %s", \
+        strerror(errno)); \
     if (seteuid(session.uid)) \
       log_pri(PR_LOG_ERR, "PRIVS_RELINQUISH: unable to seteuid(session.uid): %s", \
         strerror(errno)); \
@@ -227,10 +269,9 @@
 
 /* Revoke all privs.
  */
-#define PRIVS_REVOKE \
-  { \
+#  define PRIVS_REVOKE { \
     log_debug(DEBUG9, "REVOKE PRIVS at %s:%d", __FILE__, __LINE__); \
-    if (seteuid(0)) \
+    if (seteuid(PR_ROOT_UID)) \
       log_pri(PR_LOG_ERR, "PRIVS_REVOKE: unable to seteuid(): %s", \
         strerror(errno)); \
     if (setgid(session.gid)) \
@@ -241,6 +282,7 @@
         strerror(errno)); \
   }
 
-#endif /* HAVE_SETEUID */
+# endif /* HAVE_SETEUID */
+#endif /* PR_DEVEL_COREDUMP */
 
 #endif /* PR_PRIVS_H */
