@@ -2,8 +2,8 @@
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
  * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
- * Copyright (c) 2001, 2002 The ProFTPD Project team
- *  
+ * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -25,11 +25,11 @@
  */
 
 /* General options
- * $Id: proftpd.h,v 1.29 2002/08/14 16:17:35 castaglia Exp $
+ * $Id: proftpd.h,v 1.46 2003/08/09 07:22:04 castaglia Exp $
  */
 
-#ifndef __PROFTPD_H
-#define __PROFTPD_H
+#ifndef PR_PROFTPD_H
+#define PR_PROFTPD_H
 
 #ifndef TRUE
 #define TRUE				1
@@ -39,15 +39,11 @@
 #define FALSE				0
 #endif
 
-#define CALLBACK_FRAME			LPARAM p1,LPARAM p2,LPARAM p3,void *data
+#define CALLBACK_FRAME	\
+  unsigned long p1, unsigned long p2, unsigned long p3, void *data
+
 #define ASSERT(x)			assert(x)
 #include <assert.h>
-#define MAX_PATH_LEN			256
-
-typedef unsigned long LPARAM;		/* Longest bitsize compatible with
-                                         * a scalar and largest pointer
-					 * (platform dependant)
-					 */
 
 /* adjust per platform */
 /* unsigned 32 bit integer */
@@ -57,10 +53,6 @@ typedef unsigned short u_int_16;
 /* unsigned 8 bit integer */
 typedef unsigned char u_int_8;
 
-typedef unsigned char UCHAR;
-typedef unsigned int UINT;
-typedef unsigned long ULONG;
-
 typedef int (*callback_t)(CALLBACK_FRAME);
 
 /* Class structure */
@@ -98,15 +90,26 @@ struct config_struc;
 typedef struct {
   pool *pool;
 
-  volatile int flags;			/* Session & State flags */
-  p_in_addr_t data_addr;		/* Remote data address */
-  short data_port;			/* Remote data port */
+  volatile int sf_flags;		/* Session/State flags */
+  volatile int sp_flags;		/* Session/Protection flags */
+
+  pr_netaddr_t data_addr;		/* Remote data address */
+  unsigned short data_port;		/* Remote data port */
 
   unsigned char ident_lookups;		/* Is RFC931 (ident) protocol used? */
   char *ident_user;			/* User identified by ident protocol */
 
-  char cwd[MAX_PATH_LEN];		/* Current working directory */  
-  char vwd[MAX_PATH_LEN];		/* Current virtual working directory */
+  const char *auth_mech;		/* Name of the authentication
+                                         * module/mechanism that successfully
+                                         * authenticated the client
+					 */
+
+  const char *rfc2228_mech;		/* RFC2228 authentication mechanism
+					 * used
+					 */
+
+  char cwd[PR_TUNABLE_PATH_MAX];	/* Current working directory */
+  char vwd[PR_TUNABLE_PATH_MAX];	/* Current virtual working directory */
 
   struct config_struc *dir_config;	/* Closest matching configuration
                                          * for current operation
@@ -116,40 +119,44 @@ typedef struct {
    * privs.h
    */
 
-  int disable_id_switching;		/* disable uid/gid switching */
-  uid_t uid,ouid;                       /* current and original UIDs */
-  gid_t gid;                            /* current gid */
+  int disable_id_switching;		/* Disable UID/GID switching */
+  uid_t uid, ouid;                      /* Current and original UIDs */
+  gid_t gid;                            /* Current GID */
 
   array_header *gids;
   array_header *groups;
 
-  /* fsuid/fsgid are used for automagic chown after creation or upload,
-   * they are initially -1, meaning no chown/chgrp
-   * jss - 7/4/2001
+  /* fsuid/fsgid are used for automagic chown after creation or upload.
+   * They are initially -1, meaning no chown/chgrp.
    */
   uid_t fsuid;				/* Saved file UID */
   gid_t fsgid;				/* Saved file GID */
 
-  char *user,*group;			/* username/groupname after login */
+  char *user,*group;			/* Username/groupname after login */
   uid_t login_uid;                      /* UID after login, but before
-                                         * session.uid is changed */
+                                         * session.uid is changed
+                                         */
   gid_t login_gid;                      /* GID after login, but before
-                                         * session.gid is changed */
+                                         * session.gid is changed
+                                         */
 
-  class_t *class;			/* session class */
+  class_t *class;			/* Session class */
   char *proc_prefix;			/* The "prefix" of our process name */
 
   int wtmp_log;				/* Are we logging to wtmp? */
   struct conn_struc *c;			/* Control connection */
   struct conn_struc *d;			/* Data connection */
 
-  struct IO_Request *d_req;		/* Active data connection request */
-
   int hide_password;			/* Hide password in logs/ps listing */
-  char *anon_root;			/* Anonymous root directory */
+
+  char *chroot_path;			/* Chroot directory */
+
   struct config_struc *anon_config;	/* Anonymous FTP configuration */
   char *anon_user;			/* E-mail address sent to us */
-  
+
+  char *curr_cmd;                       /* Current FTP command */
+  int curr_phase;                       /* Current handler phase */
+
   off_t restart_pos;			/* Restart marked position */
 
   struct {
@@ -161,19 +168,28 @@ typedef struct {
     char *path;				/* As used in transfer */
     char *path_hidden;			/* As used in hidden stor */
 
-    char *bufstart,*buf;
-    int bufsize,buflen;
+    unsigned int bufsize,buflen;
 
     struct timeval start_time;		/* Time current transfer started */
     off_t file_size;			/* Total size of file (if known) */
     off_t total_bytes;			/* Total bytes transfered */
 
-    int (*get_data)(struct IO_File*,char*,int);
-    int (*complete)(struct IO_File*);
-    int (*abort)(struct IO_File*,int err);
+    char *bufstart,*buf;
   } xfer;
 
-  off_t total_bytes;          /* Total bytes transfered for this session */
+  /* Total number of bytes transfered in this session. */
+  off_t total_bytes;
+
+  /* Total number of files uploaded in this session. */
+  unsigned int total_files_in;
+
+  /* Total number of files downloaded in this session. */
+  unsigned int total_files_out;
+
+  /* Total number of files transfered (both uploaded and downloaded) in
+   * this session.
+   */
+  unsigned int total_files_xfer;
 
 } session_t;
 
@@ -195,20 +211,29 @@ extern const char	*pwdfname,*grpfname;
 
 /* Session/State flags */
 
-#define SF_PASSIVE	(1 << 0)	/* Data connection is in passive mode */
-#define SF_ABORT	(1 << 1)	/* Abort in progess */
-#define SF_XFER		(1 << 2)	/* Transfer in progress */
-#define SF_ASCII	(1 << 3)	/* ASCII mode transfer */
-#define SF_ASCII_OVERRIDE (1 << 4)	/* ASCII override this xfer only */
-#define SF_ANON		(1 << 5)	/* Anonymous (chroot) login */
-#define SF_POST_ABORT	(1 << 6)	/* After abort has occured */
-#define SF_PORT		(1 << 7)	/* Port command given */
+#define SF_PASSIVE		0x0001	/* Data connection is in passive mode */
+#define SF_ABORT		0x0002	/* Abort in progess */
+#define SF_XFER			0x0004	/* Transfer in progress */
+#define SF_ASCII		0x0010	/* ASCII mode transfer */
+#define SF_ASCII_OVERRIDE 	0x0020	/* ASCII override this xfer only */
+#define SF_ANON			0x0040	/* Anonymous (chroot) login */
+#define SF_POST_ABORT		0x0100	/* After abort has occured */
+#define SF_PORT			0x0200	/* Port command given */
+#define	SF_EPSV_ALL		0x0400	/* EPSV ALL in effect */
 
 #define SF_ALL		(SF_PASSIVE|SF_ABORT|SF_XFER|SF_ASCII| \
-                        SF_ASCII_OVERRIDE|SF_ANON|SF_POST_ABORT|SF_PORT)
+                        SF_ASCII_OVERRIDE|SF_ANON|SF_POST_ABORT|SF_PORT| \
+                        SF_EPSV_ALL)
+
+/* Session/Protection flags (RFC 2228) */
+
+#define SP_CCC		(1 << 0)	/* Clear command channel */
+#define SP_ENC		(1 << 1)	/* Privacy protected command */
+#define SP_MIC		(1 << 2)	/* Integrity protected command */
+#define SP_CONF		(1 << 3)	/* Confidentiality protected command */
 
 /* Macro to test global abort flag */
-#define XFER_ABORTED	(session.flags & SF_ABORT)
+#define XFER_ABORTED	(session.sf_flags & SF_ABORT)
 
 /* Server Types */
 #define SERVER_INETD		0
@@ -234,82 +259,14 @@ extern const char	*pwdfname,*grpfname;
 #define TIMER_STALLED		4
 #define TIMER_SESSION		5
 
-/* Response structure */
-
-typedef struct response_struc {
-  struct response_struc *next;
-
-  char *num;
-  char *msg;
-} response_t;
-
 /* Misc Prototypes */
 
-/* Utilize gcc's __attribute__ pragma for signalling that it should perform
- * printf-style checking of this function's arguments.
- */
-
-void send_response_raw(const char *, ...)
-#ifdef __GNUC__
-       __attribute__ ((format (printf, 1, 2)));
-#else
-       ;
-#endif
-
-void send_response_async(const char *, const char *, ...)
-#ifdef __GNUC__
-       __attribute__ ((format (printf, 2, 3)));
-#else
-       ;
-#endif
-
-void send_response(const char *, const char *, ...)
-#ifdef __GNUC__
-       __attribute__ ((format (printf, 2, 3)));
-#else
-       ;
-#endif
-
-void send_response_ml_start(const char *, const char *, ...)
-#ifdef __GNUC__
-       __attribute__ ((format (printf, 2, 3)));
-#else
-       ;
-#endif
-
-void send_response_ml(const char *, ...)
-#ifdef __GNUC__
-       __attribute__ ((format (printf, 1, 2)));
-#else
-       ;
-#endif
-
-void send_response_ml_end(const char *, ...)
-#ifdef __GNUC__
-       __attribute__ ((format (printf, 1, 2)));
-#else
-       ;
-#endif
-
-void add_response(const char *, const char *, ...)
-#ifdef __GNUC__
-       __attribute__ ((format (printf, 2, 3)));
-#else
-       ;
-#endif
-
-void add_response_err(const char *, const char *, ...)
-#ifdef __GNUC__
-       __attribute__ ((format (printf, 2, 3)));
-#else
-       ;
-#endif
-
 void end_login(int);
-void pr_handle_signals(void);
-void main_exit(void *, void *, void *, void *);
-void main_set_idle(void);
-void register_rehash(void *, void(*)(void *));
-void set_rlimits(void);
+void pr_signals_handle(void);
+void session_exit(int, void *, int, void *);
+void session_set_idle(void);
+void pr_rehash_register_handler(void *, void(*)(void *));
+void set_daemon_rlimits(void);
+void set_session_rlimits(void);
 
-#endif /* __PROFTPD_H */
+#endif /* PR_PROFTPD_H */
