@@ -1,7 +1,8 @@
 /*
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (C) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 1999, 2000 MacGyver aka Habeeb J. Dihu <macgyver@tos.net>
+ * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,10 +22,12 @@
  * and other respective copyright holders give permission to link this program
  * with OpenSSL, and distribute the resulting executable, without including
  * the source code for OpenSSL in the source distribution.
+ *
+ * $Id*
  */
 
-#ifndef __TIMERS_H
-#define __TIMERS_H
+#ifndef PR_TIMERS_H
+#define PR_TIMERS_H
 
 typedef struct timer_struc {
   struct timer_struc *next,*prev;
@@ -44,6 +47,7 @@ int add_timer(int, int, module *, callback_t);
 int remove_timer(int, module *);
 int reset_timer(int, module *);
 int timer_sleep(int);
-void handle_sig_alarm(void);
+void handle_alarm(void);
+void set_sig_alarm(void);
 
 #endif /* __TIMERS_H */
