@@ -24,9 +24,8 @@
  * the source code for OpenSSL in the source distribution.
  */
 
-/*
- * Flexible logging module for proftpd
- * $Id: mod_log.c,v 1.49 2003/02/24 02:37:18 castaglia Exp $
+/* Flexible logging module for proftpd
+ * $Id: mod_log.c,v 1.57 2003/10/20 07:28:57 castaglia Exp $
  */
 
 #include "conf.h"
@@ -93,6 +92,7 @@ struct logfile_struc {
 #define META_XFER_PATH		22
 #define META_DIR_NAME		23
 #define META_DIR_PATH		24
+#define META_CMD_PARAMS		25
 
 static pool			*log_pool;
 static logformat_t		*formats = NULL;
@@ -111,9 +111,10 @@ static xaset_t			*log_set = NULL;
    %F			- Transfer path (filename for client)
    %f			- Filename
    %h			- Remote client DNS name
+   %J                   - Request (command) arguments (file.txt, etc)
    %L                   - Local server IP address
    %l			- Remote logname (from identd)
-   %m			- Request (command) method (RETR, etc.)
+   %m			- Request (command) method (RETR, etc)
    %P			- Process ID of child serving request
    %p			- Port of server serving request
    %r			- Full request (command)
@@ -141,7 +142,7 @@ static void add_meta(unsigned char **s, unsigned char meta, int args,
     va_list ap;
     va_start(ap, args);
 
-    while(args--) {
+    while (args--) {
       arglen = va_arg(ap, int);
       arg = va_arg(ap, char *);
 
@@ -161,7 +162,7 @@ char *preparse_arg(char **s)
   char *ret = (*s) + 1;
 
   (*s) = (*s) + 1;
-  while(**s && **s != '}')
+  while (**s && **s != '}')
     (*s) = (*s) + 1;
 
   **s = 0;
@@ -235,6 +236,10 @@ void logformat(char *nickname, char *fmts)
           add_meta(&outs, META_REMOTE_HOST, 0);
           break;
 
+        case 'J':
+          add_meta(&outs, META_CMD_PARAMS, 0);
+          break;
+
         case 'l':
           add_meta(&outs, META_IDENT_USER, 0);
           break;
@@ -324,8 +329,7 @@ MODRET set_logformat(cmd_rec *cmd) {
   return HANDLED(cmd);
 }
 
-static int _parse_classes(char *s)
-{
+static int _parse_classes(char *s) {
   int classes = 0;
   char *nextp = NULL;
 
@@ -357,7 +361,7 @@ static int _parse_classes(char *s)
       classes |= CL_MISC;
     else
       log_pri(PR_LOG_NOTICE, "ExtendedLog class '%s' is not defined.", s);
-  } while((s = nextp));
+  } while ((s = nextp));
 
   return classes;
 }
@@ -452,12 +456,10 @@ MODRET set_systemlog(cmd_rec *cmd) {
     syslogfn = dir_canonical_path(cmd->tmp_pool,syslogfn);
 
   pr_signals_block();
-  PRIVS_ROOT
 
   if ((ret = log_opensyslog(syslogfn)) < 0) {
     int xerrno = errno;
 
-    PRIVS_RELINQUISH
     pr_signals_unblock();
 
     if (ret == LOG_WRITEABLE_DIR) {
@@ -474,9 +476,7 @@ MODRET set_systemlog(cmd_rec *cmd) {
     }
   }
 
-  PRIVS_RELINQUISH
   pr_signals_unblock();
-
   return HANDLED(cmd);
 }
 
@@ -525,7 +525,7 @@ static char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f) {
   switch (*m) {
   case META_ARG:
     m++; argp = arg;
-    while(*m != META_ARG_END)
+    while (*m != META_ARG_END)
       *argp++ = (char)*m++;
 
     *argp = 0; argp = arg;
@@ -690,7 +690,7 @@ static char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f) {
 
   case META_REMOTE_IP:
     argp = arg;
-    sstrncpy(argp, inet_ntoa(*session.c->remote_ipaddr), sizeof(arg));
+    sstrncpy(argp, pr_netaddr_get_ipstr(session.c->remote_addr), sizeof(arg));
     m++;
     break;
 
@@ -714,7 +714,7 @@ static char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f) {
 
   case META_LOCAL_IP:
     argp = arg;
-    sstrncpy(argp, inet_ntoa(*session.c->local_ipaddr), sizeof(arg));
+    sstrncpy(argp, pr_netaddr_get_ipstr(session.c->local_addr), sizeof(arg));
     m++;
     break;
 
@@ -741,42 +741,58 @@ static char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f) {
       argp = arg; m++;
 
       if (*m == META_START && *(m+1) == META_ARG) {
-        time_fmt = get_next_meta(p,cmd,&m);
+        time_fmt = get_next_meta(p, cmd, &m);
         internal_fmt = 0;
       }
+
       t = *_get_gmtoff(&timz);
       sign = (timz < 0 ? '-' : '+');
       if (timz < 0)
         timz = -timz;
 
-      strftime(argp,80,time_fmt,&t);
-      if (internal_fmt)
-        snprintf(argp + strlen(argp), sizeof(arg) - strlen(argp),
-                "%c%.2d%.2d]", sign, timz/60, timz%60);
-
+      strftime(argp, 80, time_fmt, &t);
+      if (internal_fmt) {
+        if (strlen(argp) < sizeof(arg))
+          snprintf(argp + strlen(argp), sizeof(arg) - strlen(argp),
+            "%c%.2d%.2d]", sign, timz/60, timz%60);
+        else
+          log_pri(PR_LOG_NOTICE, "notice: %%t expansion yields excessive "
+            "string, ignoring");
+      }
     }
     break;
 
   case META_SECONDS:
     argp = arg;
     if (session.xfer.p) {
-      struct timeval end_time;
 
-      gettimeofday(&end_time,NULL);
-      end_time.tv_sec -= session.xfer.start_time.tv_sec;
-      if (end_time.tv_usec >= session.xfer.start_time.tv_usec)
-        end_time.tv_usec -= session.xfer.start_time.tv_usec;
-      else {
-        end_time.tv_usec = 1000000L - (session.xfer.start_time.tv_usec -
-                           end_time.tv_usec);
-        end_time.tv_sec--;
-      }
+      /* Make sure that session.xfer.start_time actually has values (which
+       * is not always the case).
+       */
+      if (session.xfer.start_time.tv_sec != 0 ||
+          session.xfer.start_time.tv_usec != 0) {
+        struct timeval end_time;
 
-      snprintf(argp, sizeof(arg), "%ld.%03ld", (time_t) end_time.tv_sec,
-               (time_t) (end_time.tv_usec / 1000));
-    } else {
-      sstrncpy(argp,"-",sizeof(arg));
-    }
+        gettimeofday(&end_time,NULL);
+        end_time.tv_sec -= session.xfer.start_time.tv_sec;
+
+        if (end_time.tv_usec >= session.xfer.start_time.tv_usec)
+          end_time.tv_usec -= session.xfer.start_time.tv_usec;
+
+        else {
+          end_time.tv_usec = 1000000L - (session.xfer.start_time.tv_usec -
+            end_time.tv_usec);
+          end_time.tv_sec--;
+        }
+
+        snprintf(argp, sizeof(arg), "%ld.%03ld", (time_t) end_time.tv_sec,
+          (time_t) (end_time.tv_usec / 1000));
+
+      } else
+        sstrncpy(argp, "-", sizeof(arg));
+
+    } else
+      sstrncpy(argp, "-", sizeof(arg));
 
     m++;
     break;
@@ -791,6 +807,12 @@ static char *get_next_meta(pool *p, cmd_rec *cmd, unsigned char **f) {
     m++;
     break;
 
+  case META_CMD_PARAMS:
+    argp = arg;
+    sstrncpy(argp, cmd->arg, sizeof(arg));
+    m++;
+    break;
+
   case META_LOCAL_NAME:
     argp = arg;
 
@@ -954,7 +976,7 @@ static void find_extendedlogs(void) {
   int logclasses = CL_ALL;
   logformat_t *logfmt;
   char *logfmt_s = NULL;
-  logfile_t *logf;
+  logfile_t *extlog = NULL;
 
   /* We _do_ actually want the recursion here.  The reason is that we want
    * to find _all_ ExtendedLog directives in the configuration, including
@@ -996,18 +1018,18 @@ static void find_extendedlogs(void) {
       logfmt = formats;
     }
 
-    logf = (logfile_t *) pcalloc(permanent_pool, sizeof(logfile_t));
+    extlog = (logfile_t *) pcalloc(session.pool, sizeof(logfile_t));
 
-    logf->lf_filename = pstrdup(permanent_pool, logfname);
-    logf->lf_fd = -1;
-    logf->lf_syslog_level = -1;
-    logf->lf_classes = logclasses;
-    logf->lf_format = logfmt;
-    logf->lf_conf = c->parent;
+    extlog->lf_filename = pstrdup(session.pool, logfname);
+    extlog->lf_fd = -1;
+    extlog->lf_syslog_level = -1;
+    extlog->lf_classes = logclasses;
+    extlog->lf_format = logfmt;
+    extlog->lf_conf = c->parent;
     if (!log_set)
-      log_set = xaset_create(permanent_pool, NULL);
+      log_set = xaset_create(session.pool, NULL);
 
-    xaset_insert(log_set, (xasetmember_t *) logf);
+    xaset_insert(log_set, (xasetmember_t *) extlog);
     logs = (logfile_t *) log_set->xas_list;
 
 loop_extendedlogs:
