@@ -25,7 +25,7 @@
 
 /*
  * "SITE" commands module for ProFTPD
- * $Id: mod_site.c,v 1.36 2003/01/05 01:29:38 jwm Exp $
+ * $Id: mod_site.c,v 1.38 2003/08/01 01:05:25 castaglia Exp $
  */
 
 #include "conf.h"
@@ -57,7 +57,7 @@ static char *_get_full_cmd(cmd_rec *cmd) {
   for (i = 0; i < cmd->argc; i++)
     res = pstrcat(cmd->tmp_pool,res,cmd->argv[i]," ",NULL);
 
-  while(res[strlen(res)-1] == ' ')
+  while (res[strlen(res)-1] == ' ')
     res[strlen(res)-1] = '\0';
 
   return res;
@@ -83,7 +83,7 @@ MODRET site_chgrp(cmd_rec *cmd) {
     arg = pstrcat(cmd->tmp_pool, arg, *arg ? " " : "", cmd->argv[i], NULL);
 
 #if defined(HAVE_REGEX_H) && defined(HAVE_REGCOMP)
-  preg = (regex_t *) get_param_ptr(TOPLEVEL_CONF, "PathAllowFilter", FALSE);
+  preg = (regex_t *) get_param_ptr(CURRENT_CONF, "PathAllowFilter", FALSE);
 
   if (preg && regexec(preg, arg, 0, NULL, 0) != 0) {
     log_debug(DEBUG2, "'%s %s' denied by PathAllowFilter", cmd->argv[0],
@@ -92,7 +92,7 @@ MODRET site_chgrp(cmd_rec *cmd) {
     return ERROR(cmd);
   }
 
-  preg = (regex_t *) get_param_ptr(TOPLEVEL_CONF, "PathDenyFilter", FALSE);
+  preg = (regex_t *) get_param_ptr(CURRENT_CONF, "PathDenyFilter", FALSE);
 
   if (preg && regexec(preg, arg, 0, NULL, 0) == 0) {
     log_debug(DEBUG2, "'%s %s' denied by PathDenyFilter", cmd->argv[0],
@@ -153,7 +153,7 @@ MODRET site_chmod(cmd_rec *cmd) {
     arg = pstrcat(cmd->tmp_pool, arg, *arg ? " " : "", cmd->argv[i], NULL);
 
 #if defined(HAVE_REGEX_H) && defined(HAVE_REGCOMP)
-  preg = (regex_t *) get_param_ptr(TOPLEVEL_CONF, "PathAllowFilter", FALSE);
+  preg = (regex_t *) get_param_ptr(CURRENT_CONF, "PathAllowFilter", FALSE);
 
   if (preg && regexec(preg, arg, 0, NULL, 0) != 0) {
     log_debug(DEBUG2, "'%s %s %s' denied by PathAllowFilter", cmd->argv[0],
@@ -162,7 +162,7 @@ MODRET site_chmod(cmd_rec *cmd) {
     return ERROR(cmd);
   }
 
-  preg = (regex_t *) get_param_ptr(TOPLEVEL_CONF, "PathDenyFilter", FALSE);
+  preg = (regex_t *) get_param_ptr(CURRENT_CONF, "PathDenyFilter", FALSE);
 
   if (preg && regexec(preg, arg, 0, NULL, 0) == 0) {
     log_debug(DEBUG2, "'%s %s %s' denied by PathDenyFilter", cmd->argv[0],
@@ -226,7 +226,7 @@ MODRET site_chmod(cmd_rec *cmd) {
       }
 
       cp = what;
-      while(cp) {
+      while (cp) {
         switch (*who) {
         case 'u':
           mask = 0077;
