@@ -26,7 +26,7 @@
 
 /* Data transfer module for ProFTPD
  *
- * $Id: mod_xfer.c,v 1.129 2003/02/25 01:39:35 castaglia Exp $
+ * $Id: mod_xfer.c,v 1.146 2003/07/15 02:57:36 castaglia Exp $
  */
 
 #include "conf.h"
@@ -45,12 +45,6 @@
 extern module auth_module;
 extern pid_t mpid;
 
-/* From the auth module */
-char *auth_map_uid(int);
-char *auth_map_gid(int);
-
-void xfer_abort(pr_netio_stream_t *, int);
-
 /* Variables for this module */
 static pr_fh_t *retr_fh = NULL;
 static pr_fh_t *stor_fh = NULL;
@@ -83,10 +77,17 @@ static unsigned long find_max_nbytes(char *directive) {
   c = find_config(CURRENT_CONF, CONF_PARAM, directive, FALSE);
 
   while (c) {
-    if (c->argc == 3) {
-      if (!strcmp(c->argv[1], "user")) {
 
-        if (pr_user_or_expression((char **) &c->argv[2])) {
+    /* This check is for more than three arguments: one argument is the
+     * classifier (i.e. "user", "group", or "class"), one argument is
+     * the precedence, one is the number of bytes; the remaining arguments
+     * are the individual items in the configured expression.
+     */
+
+    if (c->argc > 3) {
+      if (!strcmp(c->argv[2], "user")) {
+
+        if (pr_user_or_expression((char **) &c->argv[3])) {
           if (*((unsigned int *) c->argv[1]) > ctxt_precedence) {
 
             /* Set the context precedence */
@@ -99,9 +100,9 @@ static unsigned long find_max_nbytes(char *directive) {
           }
         }
 
-      } else if (!strcmp(c->argv[1], "group")) {
+      } else if (!strcmp(c->argv[2], "group")) {
 
-        if (pr_group_or_expression((char **) &c->argv[2])) {
+        if (pr_group_or_expression((char **) &c->argv[3])) {
           if (*((unsigned int *) c->argv[1]) > ctxt_precedence) {
 
             /* Set the context precedence */
@@ -114,9 +115,9 @@ static unsigned long find_max_nbytes(char *directive) {
           }
         }
 
-      } else if (!strcmp(c->argv[1], "class")) {
+      } else if (!strcmp(c->argv[2], "class")) {
 
-        if (pr_class_or_expression((char **) &c->argv[2])) {
+        if (pr_class_or_expression((char **) &c->argv[3])) {
           if (*((unsigned int *) c->argv[1]) > ctxt_precedence) {
 
             /* Set the context precedence */
@@ -309,10 +310,6 @@ static void xfer_rate_lookup(cmd_rec *cmd) {
     have_class_rate = FALSE, have_all_rate = FALSE;
   unsigned int precedence = 0;
 
-  /* Do nothing if values are already cached. */
-  if (have_xfer_rate)
-    return;
-
   /* Make sure the variables are (re)initialized */
   xfer_rate_kbps = xfer_rate_bps = 0.0;
   xfer_rate_freebytes = 0;
@@ -472,7 +469,7 @@ static void xfer_rate_sigmask(unsigned char block) {
     sigaddset(&sig_set, SIGUSR1);
     sigaddset(&sig_set, SIGINT);
     sigaddset(&sig_set, SIGQUIT);
-    sigaddset(&sig_set, SIGALRM);
+    sigaddset(&sig_set, SIGURG);
 #ifdef SIGIO
     sigaddset(&sig_set, SIGIO);
 #endif /* SIGIO */
@@ -481,10 +478,25 @@ static void xfer_rate_sigmask(unsigned char block) {
 #endif /* SIGBUS */
     sigaddset(&sig_set, SIGHUP);
 
-    sigprocmask(SIG_BLOCK, &sig_set, NULL);
+    while (sigprocmask(SIG_BLOCK, &sig_set, NULL) < 0) {
+      if (errno == EINTR) {
+        pr_signals_handle();
+        continue;
+      }
 
-  } else
-    sigprocmask(SIG_UNBLOCK, &sig_set, NULL);
+      break;
+    }
+
+  } else {
+    while (sigprocmask(SIG_UNBLOCK, &sig_set, NULL) < 0) {
+      if (errno == EINTR) {
+        pr_signals_handle();
+        continue;
+      }
+
+      break;
+    }
+  }
 }
 
 /* Returns the difference, in milliseconds, between the given timeval and
@@ -500,6 +512,7 @@ static long xfer_rate_since(struct timeval *then) {
 
 static void xfer_rate_throttle(off_t xferlen) {
   long ideal = 0, elapsed = 0;
+  off_t orig_xferlen = xferlen;
 
   /* Calculate the time interval since the transfer of data started. */
   elapsed = xfer_rate_since(&session.xfer.start_time);
@@ -509,7 +522,7 @@ static void xfer_rate_throttle(off_t xferlen) {
 
     /* Update the scoreboard. */
     pr_scoreboard_update_entry(getpid(),
-      PR_SCORE_XFER_LEN, xferlen,
+      PR_SCORE_XFER_LEN, orig_xferlen,
       PR_SCORE_XFER_ELAPSED, (unsigned long) elapsed,
       NULL);
 
@@ -519,28 +532,24 @@ static void xfer_rate_throttle(off_t xferlen) {
   /* Give credit for any configured freebytes. */
   if (xferlen && xfer_rate_freebytes) {
 
-    if (xfer_rate_freebytes >= xferlen) {
-       /* Decrement the number of freebytes by the total number of bytes
-        * sent.  Since there are still more freebytes to be used, just
-        * return now, after updating the timeval struc.
-        */
-       xfer_rate_freebytes -= xferlen;
+    if (xferlen > xfer_rate_freebytes)
 
-      /* Update the scoreboard. */
+      /* Decrement the number of bytes transferred by the freebytes, so that
+       * any throttling does not take into account the freebytes.
+       */
+      xferlen -= xfer_rate_freebytes;
+
+    else {
+
+      /* The number of bytes transferred is less than the freebytes.  Just
+       * update the scoreboard -- no throttling needed.
+       */
       pr_scoreboard_update_entry(getpid(),
-        PR_SCORE_XFER_LEN, xferlen,
+        PR_SCORE_XFER_LEN, orig_xferlen,
         PR_SCORE_XFER_ELAPSED, (unsigned long) elapsed,
         NULL);
 
-       return;
-
-    } else {
-      xferlen -= xfer_rate_freebytes;
-
-      /* Make sure that, the next time through, the freebytes are not
-       * credited again.
-       */
-      xfer_rate_freebytes = 0;
+      return;
     }
   }
 
@@ -574,7 +583,7 @@ static void xfer_rate_throttle(off_t xferlen) {
 
     /* Update the scoreboard. */
     pr_scoreboard_update_entry(getpid(),
-      PR_SCORE_XFER_LEN, xferlen,
+      PR_SCORE_XFER_LEN, orig_xferlen,
       PR_SCORE_XFER_ELAPSED, (unsigned long) ideal,
       NULL);
 
@@ -582,7 +591,7 @@ static void xfer_rate_throttle(off_t xferlen) {
 
     /* Update the scoreboard. */
     pr_scoreboard_update_entry(getpid(),
-      PR_SCORE_XFER_LEN, xferlen,
+      PR_SCORE_XFER_LEN, orig_xferlen,
       PR_SCORE_XFER_ELAPSED, (unsigned long) elapsed,
       NULL);
   }
@@ -610,7 +619,7 @@ static int _transmit_sendfile(off_t count, off_t *offset,
    */
   if (have_xfer_rate ||
      !(session.xfer.file_size - count) ||
-     (session.sf_flags & (SF_ASCII | SF_ASCII_OVERRIDE))) {
+     (session.sf_flags & (SF_ASCII|SF_ASCII_OVERRIDE))) {
     return 0;
   }
 
@@ -622,8 +631,8 @@ static int _transmit_sendfile(off_t count, off_t *offset,
     switch (errno) {
     case EAGAIN:
     case EINTR:
-      /* Interrupted call, or the other side wasn't ready yet.
-       */
+      /* Interrupted call, or the other side wasn't ready yet. */
+      pr_signals_handle();
       goto retry;
 
     case EPIPE:
@@ -639,8 +648,7 @@ static int _transmit_sendfile(off_t count, off_t *offset,
 #endif /* ENOSYS */
 
     case EINVAL:
-      /* No sendfile support, apparently.  Try it the normal way.
-       */
+      /* No sendfile support, apparently.  Try it the normal way. */
       return 0;
       break;
 
@@ -658,16 +666,43 @@ static int _transmit_sendfile(off_t count, off_t *offset,
 #endif /* HAVE_SENDFILE */
 
 static long _transmit_data(off_t count, off_t offset, char *buf, long bufsize) {
+  long res;
+
+#ifdef TCP_CORK
+  int on = 1;
+  socklen_t len = sizeof(int);
+#endif /* TCP_CORK */
+
 #ifdef HAVE_SENDFILE
   pr_sendfile_t retval;
-
-  if (!_transmit_sendfile(count, &offset, &retval))
-    return _transmit_normal(buf, bufsize);
-  else
-    return (long) retval;
-#else
-  return _transmit_normal(buf, bufsize);
 #endif /* HAVE_SENDFILE */
+
+#ifdef TCP_CORK
+  /* Note: TCP_CORK is a Linuxism, introduced with the 2.4 kernel.  It
+   * has effects similar to BSD's TCP_NOPUSH option.
+   */
+  if (setsockopt(PR_NETIO_FD(session.d->outstrm), SOL_TCP, TCP_CORK, &on,
+      len) < 0)
+    log_pri(PR_LOG_NOTICE, "error setting TCP_CORK: %s", strerror(errno));
+#endif /* TCP_CORK */
+
+#ifdef HAVE_SENDFILE
+  if (!_transmit_sendfile(count, &offset, &retval))
+    res = _transmit_normal(buf, bufsize);
+  else
+    res = (long) retval;
+#else
+  res = _transmit_normal(buf, bufsize);
+#endif /* HAVE_SENDFILE */
+
+#ifdef TCP_CORK
+  on = 0;
+  if (setsockopt(PR_NETIO_FD(session.d->outstrm), SOL_TCP, TCP_CORK, &on,
+      len) < 0)
+    log_pri(PR_LOG_NOTICE, "error setting TCP_CORK: %s", strerror(errno));
+#endif /* TCP_CORK */
+
+  return res;
 }
 
 static void _stor_chown(void) {
@@ -811,14 +846,6 @@ static void xfer_exit_cb(void) {
   }
 }
 
-/* This function clears any cached TransferRate values, as TransferRates
- * may be set on a directory-by-directory basis.
- */
-MODRET xfer_reset_rate(cmd_rec *cmd) {
-  have_xfer_rate = FALSE;
-  return DECLINED(cmd);
-}
-
 /* This is a PRE_CMD handler that checks security, etc, and places the full
  * filename to receive in cmd->private [note that we CANNOT use cmd->tmp_pool
  * for this, as tmp_pool only lasts for the duration of this function.
@@ -922,7 +949,7 @@ MODRET xfer_pre_stor(cmd_rec *cmd) {
 
     maxlen = strlen(dir) + 1 + 5;
 
-    if (maxlen > MAXPATHLEN) {
+    if (maxlen > PR_TUNABLE_PATH_MAX) {
       /* This probably shouldn't happen */
       pr_response_add_err(R_451, "%s: File name too long", dir);
       return ERROR(cmd);
@@ -1139,30 +1166,37 @@ MODRET xfer_stor(cmd_rec *cmd) {
   }
 
 #if defined(HAVE_REGEX_H) && defined(HAVE_REGCOMP)
-  preg = (regex_t *) get_param_ptr(TOPLEVEL_CONF, "PathAllowFilter", FALSE);
+  preg = (regex_t *) get_param_ptr(CURRENT_CONF, "PathAllowFilter", FALSE);
 
-  if (preg && ((ret = regexec(preg,cmd->arg, 0, NULL, 0)) != 0)) {
-    char errmsg[200];
-    regerror(ret, preg, errmsg, sizeof(errmsg));
-    log_debug(DEBUG2, "'%s' denied by PathAllowFilter: %s", cmd->arg, errmsg);
-    pr_response_add_err(R_550, "%s: Forbidden filename", cmd->arg);
-    return ERROR(cmd);
+  if (preg) {
+    if ((ret = regexec(preg,cmd->arg, 0, NULL, 0)) != 0) {
+      log_debug(DEBUG2, "'%s' denied by PathAllowFilter", cmd->arg);
+      pr_response_add_err(R_550, "%s: Forbidden filename", cmd->arg);
+      return ERROR(cmd);
 
-  } else if (preg)
-    log_debug(DEBUG8, "'%s' allowed by PathAllowFilter", cmd->arg);
+    } else {
+      char errmsg[200];
+      regerror(ret, preg, errmsg, sizeof(errmsg));
+      log_debug(DEBUG8, "'%s' allowed by PathAllowFilter (%s)", cmd->arg,
+        errmsg);
+    }
+  }
 
-  preg = (regex_t *) get_param_ptr(TOPLEVEL_CONF, "PathDenyFilter", FALSE);
+  preg = (regex_t *) get_param_ptr(CURRENT_CONF, "PathDenyFilter", FALSE);
 
-  if (preg && ((ret = regexec(preg, cmd->arg, 0, NULL, 0)) == 0)) {
-    char errmsg[200];
-    regerror(ret, preg, errmsg, sizeof(errmsg));
-    log_debug(DEBUG2, "'%s' denied by PathDenyFilter: %s", cmd->arg, errmsg);
-    pr_response_add_err(R_550, "%s: Forbidden filename", cmd->arg);
-    return ERROR(cmd);
-
-  } else if (preg)
-    log_debug(DEBUG8, "'%s' allowed by PathDenyFilter", cmd->arg);
+  if (preg) {
+    if ((ret = regexec(preg, cmd->arg, 0, NULL, 0)) == 0) {
+      log_debug(DEBUG2, "'%s' denied by PathDenyFilter", cmd->arg);
+      pr_response_add_err(R_550, "%s: Forbidden filename", cmd->arg);
+      return ERROR(cmd);
 
+    } else {
+      char errmsg[200];
+      regerror(ret, preg, errmsg, sizeof(errmsg));
+      log_debug(DEBUG8, "'%s' allowed by PathDenyFilter (%s)", cmd->arg,
+        errmsg);
+    }
+  }
 #endif /* REGEX */
 
   if (session.xfer.xfer_type == STOR_HIDDEN)
@@ -1268,9 +1302,9 @@ MODRET xfer_stor(cmd_rec *cmd) {
     return ERROR(cmd);
   }
 
-  bufsize = (main_server->tcp_rwin > 0 ?
-    main_server->tcp_rwin : PR_TUNABLE_BUFFER_SIZE);
-  lbuf = (char*) palloc(cmd->tmp_pool, bufsize);
+  bufsize = (main_server->tcp_rcvbuf_len > 0 ?  main_server->tcp_rcvbuf_len :
+    PR_TUNABLE_XFER_BUFFER_SIZE);
+  lbuf = (char *) palloc(cmd->tmp_pool, bufsize);
 
   while ((len = pr_data_xfer(lbuf, bufsize)) > 0) {
     if (XFER_ABORTED)
@@ -1466,6 +1500,11 @@ MODRET xfer_retr(cmd_rec *cmd) {
 
   if (pr_fsio_stat(dir, &sbuf) < 0) {
     /* Error stat'ing the file. */
+    int xerrno = errno;
+    pr_fsio_close(retr_fh);
+    errno = xerrno;
+
+    retr_fh = NULL;
     pr_response_add_err(R_550, "%s: %s", cmd->arg, strerror(errno));
     return ERROR(cmd);
   }
@@ -1477,8 +1516,8 @@ MODRET xfer_retr(cmd_rec *cmd) {
      */
     if (session.restart_pos > sbuf.st_size) {
       pr_response_add_err(R_554, "%s: invalid REST argument", cmd->arg);
-      pr_fsio_close(stor_fh);
-      stor_fh = NULL;
+      pr_fsio_close(retr_fh);
+      retr_fh = NULL;
       return ERROR(cmd);
     }
 
@@ -1505,6 +1544,7 @@ MODRET xfer_retr(cmd_rec *cmd) {
     cnt_steps = 1;
 
   if (pr_data_open(cmd->arg, NULL, PR_NETIO_IO_WR, sbuf.st_size - respos) < 0) {
+    retr_abort();
     pr_data_abort(0, TRUE);
     return ERROR(cmd);
   }
@@ -1534,8 +1574,8 @@ MODRET xfer_retr(cmd_rec *cmd) {
     return ERROR(cmd);
   }
 
-  bufsize = (main_server->tcp_swin > 0 ?
-             main_server->tcp_swin : PR_TUNABLE_BUFFER_SIZE);
+  bufsize = (main_server->tcp_sndbuf_len > 0 ?  main_server->tcp_sndbuf_len :
+    PR_TUNABLE_XFER_BUFFER_SIZE);
   lbuf = (char *) palloc(cmd->tmp_pool, bufsize);
 
   nbytes_sent = respos;
@@ -1739,6 +1779,10 @@ MODRET xfer_err_cleanup(cmd_rec *cmd) {
     destroy_pool(session.xfer.p);
 
   memset(&session.xfer, '\0', sizeof(session.xfer));
+
+  /* Don't forget to clear any possible REST parameter as well. */
+  session.restart_pos = 0;
+
   return DECLINED(cmd);
 }
 
@@ -1750,6 +1794,9 @@ MODRET xfer_log_stor(cmd_rec *cmd) {
   session.total_files_in++;
   session.total_files_xfer++;
 
+  /* Don't forget to clear any possible REST parameter as well. */
+  session.restart_pos = 0;
+
   return DECLINED(cmd);
 }
 
@@ -1761,6 +1808,9 @@ MODRET xfer_log_retr(cmd_rec *cmd) {
   session.total_files_out++;
   session.total_files_xfer++;
 
+  /* Don't forget to clear any possible REST parameter as well. */
+  session.restart_pos = 0;
+
   return DECLINED(cmd);
 }
 
@@ -1893,7 +1943,11 @@ MODRET set_maxfilesize(cmd_rec *cmd) {
      cmd->config->config_type : cmd->server->config_type ?
      cmd->server->config_type : CONF_ROOT);
 
-  if (cmd->argc-1 != 2 && cmd->argc-1 != 4)
+  if (cmd->argc-1 == 1) {
+    if (strcmp(cmd->argv[1], "*") != 0)
+      CONF_ERROR(cmd, "incorrect number of parameters");
+
+  } else if (cmd->argc-1 != 2 && cmd->argc-1 != 4)
     CONF_ERROR(cmd, "incorrect number of parameters");
 
   CHECK_CONF(cmd, CONF_ROOT|CONF_ANON|CONF_VIRTUAL|CONF_GLOBAL|CONF_DIR|
@@ -1915,6 +1969,9 @@ MODRET set_maxfilesize(cmd_rec *cmd) {
   else if (ctxt & CONF_DIR)
     precedence = 4;
 
+  else if (ctxt & CONF_DYNDIR)
+    precedence = 5;
+
   /* If the directive was used with four arguments, it means the optional
    * classifiers and expression were used.  Make sure the classifier is a valid
    * one.
@@ -1931,9 +1988,10 @@ MODRET set_maxfilesize(cmd_rec *cmd) {
          cmd->argv[3], "'", NULL));
   }
 
-  if (!strcmp(cmd->argv[1], "*")) {
+  if (cmd->argc-1 == 1) {
 
-    /* Do nothing here -- the "*" signifies an unlimited size, which is
+    /* Do nothing here -- the "*" (the only parameter allowed if there is
+     * only a single parameter given) signifies an unlimited size, which is
      * what the server provides by default.
      */
     nbytes = 0UL;
@@ -1956,7 +2014,7 @@ MODRET set_maxfilesize(cmd_rec *cmd) {
     }
   }
 
-  if (cmd->argc-1 == 2) {
+  if (cmd->argc-1 == 1 || cmd->argc-1 == 2) {
     c = add_config_param(cmd->argv[0], 2, NULL, NULL);
     c->argv[0] = pcalloc(c->pool, sizeof(unsigned long));
     *((unsigned long *) c->argv[0]) = nbytes;
@@ -2266,10 +2324,6 @@ static cmdtable xfer_cmdtab[] = {
   { LOG_CMD_ERR, C_APPE,G_NONE,  xfer_err_cleanup,  FALSE,  FALSE },
   { CMD,     C_ABOR,	G_NONE,	 xfer_abor,	TRUE,	TRUE,  CL_MISC  },
   { CMD,     C_REST,	G_NONE,	 xfer_rest,	TRUE,	FALSE, CL_MISC  },
-  { PRE_CMD, C_CDUP,	G_NONE,  xfer_reset_rate, TRUE,	FALSE },
-  { PRE_CMD, C_CWD,	G_NONE,  xfer_reset_rate, TRUE,	FALSE },
-  { PRE_CMD, C_XCUP,	G_NONE,  xfer_reset_rate, TRUE,	FALSE },
-  { PRE_CMD, C_XCWD,	G_NONE,  xfer_reset_rate, TRUE,	FALSE },
   { 0,NULL }
 };
 
