@@ -24,7 +24,7 @@
 
 /* Routines to work with ProFTPD bindings
  *
- * $Id: bindings.c,v 1.7 2003/01/02 18:25:23 castaglia Exp $
+ * $Id: bindings.c,v 1.17 2003/09/29 17:25:19 castaglia Exp $
  */
 
 #include "conf.h"
@@ -65,10 +65,15 @@ static void server_cleanup_cb(void *conn) {
 /* The hashing function for the hash table of bindings.  This algorithm
  * is stolen from Apache's http_vhost.c
  */
-static unsigned int ipbind_hash_addr(p_in_addr_t *addr) {
+static unsigned int ipbind_hash_addr(pr_netaddr_t *addr) {
+  size_t offset = pr_netaddr_get_inaddr_len(addr);
 
-  /* NOTE: use inet_addr() accessor functions in the future */
-  unsigned int key = addr->s_addr;
+  /* The key is the last four bytes of the IP address.
+   * For IPv4, this is the entire address, as always.
+   * For IPv6, this is usually part of the MAC address.
+   */
+  unsigned int key = *(unsigned *) ((char *) pr_netaddr_get_inaddr(addr) +
+    offset - 4);
 
   key ^= (key >> 16);
   return ((key >> 8) ^ key) % PR_BINDINGS_TABLE_SIZE;
@@ -108,7 +113,7 @@ conn_t *pr_ipbind_accept_conn(fd_set *readfds, int *listenfd) {
     if (FD_ISSET(listener->listen_fd, readfds) &&
         listener->mode == CM_LISTEN) {
 
-      if ((fd = inet_accept_nowait(listener->pool, listener)) == -1) {
+      if ((fd = pr_inet_accept_nowait(listener->pool, listener)) == -1) {
 
         /* Handle errors gracefully.  If we're here, then
          * ipbind->ib_server->listen contains either error information, or
@@ -135,7 +140,7 @@ int pr_ipbind_add_binds(server_rec *serv) {
   int res = 0;
   config_rec *c = NULL;
   conn_t *listen_conn = NULL;
-  p_in_addr_t *addr = NULL;
+  pr_netaddr_t *addr = NULL;
 
   /* sanity check */
   if (!serv)
@@ -146,11 +151,11 @@ int pr_ipbind_add_binds(server_rec *serv) {
   while (c) {
     listen_conn = NULL;
 
-    addr = inet_getaddr(serv->pool, c->argv[0]);
-
+    addr = pr_netaddr_get_addr(serv->pool, c->argv[0], NULL);
     if (!addr) {
       log_pri(PR_LOG_NOTICE, "notice: unable to determine IP address of '%s'",
         (char *) c->argv[0]);
+      c = find_config_next(c, c->next, CONF_PARAM, "Bind", FALSE);
       continue;
     }
 
@@ -158,7 +163,7 @@ int pr_ipbind_add_binds(server_rec *serv) {
      * listen socket for this address, and add it to the binding list.
      */
     if (SocketBindTight && serv->ServerPort) {
-      listen_conn = inet_create_connection(serv->pool, server_list, -1, addr,
+      listen_conn = pr_inet_create_connection(serv->pool, server_list, -1, addr,
         serv->ServerPort, FALSE);
 
       PR_CREATE_IPBIND(serv, addr);
@@ -178,7 +183,7 @@ int pr_ipbind_add_binds(server_rec *serv) {
   return 0;
 }
 
-int pr_ipbind_close(p_in_addr_t *addr, unsigned int port,
+int pr_ipbind_close(pr_netaddr_t *addr, unsigned int port,
     unsigned char close_namebinds) {
   int res = 0;
   register unsigned int i = 0;
@@ -191,15 +196,13 @@ int pr_ipbind_close(p_in_addr_t *addr, unsigned int port,
 
     if (ipbind_table[i] == NULL) {
       log_pri(PR_LOG_NOTICE, "notice: no ipbind found for %s:%d",
-        inet_ntoa(*addr), port);
+        pr_netaddr_get_ipstr(addr), port);
       errno = ENOENT;
       return -1;
     }
 
     for (ipbind = ipbind_table[i]; ipbind; ipbind = ipbind->ib_next) {
-
-      /* NOTE: use the inet_addr() accessor functions in the future */
-      if (ipbind->ib_addr.s_addr == addr->s_addr &&
+      if (pr_netaddr_cmp(ipbind->ib_addr, addr) == 0 &&
           (!ipbind->ib_port || ipbind->ib_port == port)) {
         have_ipbind = TRUE;
         break;
@@ -208,7 +211,7 @@ int pr_ipbind_close(p_in_addr_t *addr, unsigned int port,
 
     if (!have_ipbind) {
       log_pri(PR_LOG_NOTICE, "notice: no ipbind found for %s:%d",
-        inet_ntoa(*addr), port);
+        pr_netaddr_get_ipstr(addr), port);
       errno = ENOENT;
       return -1;
     }
@@ -228,9 +231,9 @@ int pr_ipbind_close(p_in_addr_t *addr, unsigned int port,
      * for the master daemon in inetd mode, in which case virtual servers
      * can't be shutdown via ftpdctl, anyway.
      */
-    if (SocketBindTight && ipbind->ib_server->listen != NULL) {
-      inet_close(ipbind->ib_server->pool, ipbind->ib_server->listen);
-      ipbind->ib_server->listen = NULL;
+    if (SocketBindTight && ipbind->ib_listener != NULL) {
+      pr_inet_close(ipbind->ib_server->pool, ipbind->ib_listener);
+      ipbind->ib_listener = ipbind->ib_server->listen = NULL;
     }
 
     /* Mark this ipbind as inactive.  For SocketBindTight sockets, the
@@ -249,7 +252,7 @@ int pr_ipbind_close(p_in_addr_t *addr, unsigned int port,
       for (j = 0; j < ipbind->ib_namebinds->nelts; j++) {
         pr_namebind_t *nb = namebinds[j];
 
-        PR_CLOSE_NAMEBIND(nb->nb_name, nb->nb_server->ipaddr,
+        PR_CLOSE_NAMEBIND(nb->nb_name, nb->nb_server->addr,
           nb->nb_server->ServerPort);
       }
     }
@@ -264,9 +267,9 @@ int pr_ipbind_close(p_in_addr_t *addr, unsigned int port,
       pr_ipbind_t *ipbind = NULL;
       for (ipbind = ipbind_table[i]; ipbind; ipbind = ipbind->ib_next) {
 
-        if (SocketBindTight && ipbind->ib_server->listen != NULL) {
-          inet_close(main_server->pool, ipbind->ib_server->listen);
-          ipbind->ib_server->listen = NULL;
+        if (SocketBindTight && ipbind->ib_listener != NULL) {
+          pr_inet_close(main_server->pool, ipbind->ib_listener);
+          ipbind->ib_listener = ipbind->ib_server->listen = NULL;
         }
 
         /* Note: do not need to check if this ipbind was previously closed,
@@ -283,7 +286,7 @@ int pr_ipbind_close(p_in_addr_t *addr, unsigned int port,
           for (j = 0; j < ipbind->ib_namebinds->nelts; j++) {
             pr_namebind_t *nb = namebinds[j];
 
-            PR_CLOSE_NAMEBIND(nb->nb_name, nb->nb_server->ipaddr,
+            PR_CLOSE_NAMEBIND(nb->nb_name, nb->nb_server->addr,
               nb->nb_server->ServerPort);
           }
         }
@@ -295,14 +298,36 @@ int pr_ipbind_close(p_in_addr_t *addr, unsigned int port,
   return 0;
 }
 
-int pr_ipbind_create(server_rec *server, p_in_addr_t *addr) {
+/* Need a way to close all listening fds in a child process. */
+int pr_ipbind_close_listeners(void) {
+  conn_t *listener = listener_list;
+  register unsigned int i = 0;
+
+  /* sanity checks */
+  if (!listener)
+    return 0;
+
+  for (i = 0, listener = listener_list; i < listener_listlen;
+      i++, listener = listener->next) {
+    pr_signals_handle();
+
+    if (listener->listen_fd != -1) {
+      close(listener->listen_fd);
+      listener->listen_fd = -1;
+    }
+  }
+
+  return 0;
+}
+
+int pr_ipbind_create(server_rec *server, pr_netaddr_t *addr) {
   int res = 0;
   pr_ipbind_t *ipbind = NULL;
   config_rec *c = NULL;
   server_rec *s = NULL;
   register unsigned int i = 0;
 
-  /* sanity checks */
+  /* Sanity checks */
   if (!server || !addr) {
     errno = EINVAL;
     return -1;
@@ -312,14 +337,12 @@ int pr_ipbind_create(server_rec *server, p_in_addr_t *addr) {
 
   /* Make sure the address is not already in use */
   for (ipbind = ipbind_table[i]; ipbind; ipbind = ipbind->ib_next) {
-
-    /* NOTE: use the inet_addr() accessor functions in the future */
-    if (ipbind->ib_addr.s_addr == addr->s_addr &&
+    if (pr_netaddr_cmp(ipbind->ib_addr, addr) == 0 &&
         ipbind->ib_port == server->ServerPort) {
 
       /* An ipbind already exists for this IP address */
-      log_pri(PR_LOG_NOTICE, "notice: '%s' (%s:%d) already bound to '%s'",
-        server->ServerName, inet_ntoa(*addr), server->ServerPort,
+      log_pri(PR_LOG_NOTICE, "notice: '%s' (%s:%u) already bound to '%s'",
+        server->ServerName, pr_netaddr_get_ipstr(addr), server->ServerPort,
         ipbind->ib_server->ServerName);
 
       errno = EADDRINUSE;
@@ -328,12 +351,12 @@ int pr_ipbind_create(server_rec *server, p_in_addr_t *addr) {
   }
 
   if (!binding_pool)
-    /* initialize the working pool, if not present */
+    /* Initialize the working pool, if not present */
     binding_pool = make_sub_pool(permanent_pool);
 
   ipbind = pcalloc(server->pool, sizeof(pr_ipbind_t));
   ipbind->ib_server = server;
-  ipbind->ib_addr = *addr;
+  ipbind->ib_addr = addr;
   ipbind->ib_port = server->ServerPort;
   ipbind->ib_namebinds = NULL;
   ipbind->ib_isdefault = FALSE;
@@ -351,14 +374,14 @@ int pr_ipbind_create(server_rec *server, p_in_addr_t *addr) {
 
   while (c) {
     s = (server_rec *) c->argv[0];
-    PR_CREATE_NAMEBIND(s, c->name, server->ipaddr, server->ServerPort);
+    PR_CREATE_NAMEBIND(s, c->name, server->addr, server->ServerPort);
     c = find_config_next(c, c->next, CONF_NAMED, NULL, FALSE);
   }
 
   return 0;
 }
 
-pr_ipbind_t *pr_ipbind_find(p_in_addr_t *addr, unsigned int port,
+pr_ipbind_t *pr_ipbind_find(pr_netaddr_t *addr, unsigned int port,
     unsigned char skip_inactive) {
   pr_ipbind_t *ipbind = NULL;
   register unsigned int i = ipbind_hash_addr(addr);
@@ -368,8 +391,7 @@ pr_ipbind_t *pr_ipbind_find(p_in_addr_t *addr, unsigned int port,
     if (skip_inactive && !ipbind->ib_isactive)
       continue;
 
-    /* NOTE: use the inet_addr() accessor functions in the future */
-    if (ipbind->ib_addr.s_addr == addr->s_addr &&
+    if (pr_netaddr_cmp(ipbind->ib_addr, addr) == 0 &&
         (!ipbind->ib_port || ipbind->ib_port == port))
       return ipbind;
   }
@@ -378,7 +400,7 @@ pr_ipbind_t *pr_ipbind_find(p_in_addr_t *addr, unsigned int port,
   return NULL;
 }
 
-server_rec *pr_ipbind_get_server(p_in_addr_t *addr, unsigned int port) {
+server_rec *pr_ipbind_get_server(pr_netaddr_t *addr, unsigned int port) {
   pr_ipbind_t *ipbind = NULL;
 
   /* If we've got a binding configured for this exact address, return it
@@ -387,34 +409,16 @@ server_rec *pr_ipbind_get_server(p_in_addr_t *addr, unsigned int port) {
   if ((ipbind = pr_ipbind_find(addr, port, TRUE)) != NULL)
     return ipbind->ib_server;
 
-  /* Not found in binding list, so see if it's the loopback address */
-  if (ipbind_localhost_server) {
-    p_in_addr_t loopback, loopmask, tmp;
-
-#ifdef HAVE_INET_ATON
-    inet_aton(LOOPBACK_NET, &loopback);
-    inet_aton(LOOPBACK_MASK, &loopmask);
-#else
-    loopback.s_addr = inet_addr(LOOPBACK_NET);
-    loopmask.s_addr = inet_addr(LOOPBACK_MASK);
-#endif
-    loopback.s_addr = ntohl(loopback.s_addr);
-    loopmask.s_addr = ntohl(loopmask.s_addr);
-    tmp.s_addr = ntohl(addr->s_addr);
-
-    /* NOTE: use the inet_addr() accessor functions in the future */
-    if ((tmp.s_addr & loopmask.s_addr) == loopback.s_addr &&
-        (!ipbind_localhost_server->ib_port ||
-         port == ipbind_localhost_server->ib_port))
-    {
-      return ipbind_localhost_server->ib_server;
-    }
-  }
-
-  /* Otherwise, use the default server, if set */
+  /* Use the default server, if set */
   if (ipbind_default_server && ipbind_default_server->ib_isactive)
     return ipbind_default_server->ib_server;
 
+  /* Not found in binding list, and no DefaultServer, so see if it's the
+   * loopback address
+   */
+  if (ipbind_localhost_server && pr_netaddr_loopback(addr))
+      return ipbind_localhost_server->ib_server;
+
   return NULL;
 }
 
@@ -444,24 +448,23 @@ int pr_ipbind_listen(fd_set *readfds) {
       if (SocketBindTight && !ipbind->ib_isactive)
         continue;
 
-      if (ipbind->ib_server->listen) {
+      if (ipbind->ib_listener) {
 
-        if (ipbind->ib_server->listen->mode == CM_NONE)
-          inet_listen(ipbind->ib_server->listen->pool,
-            ipbind->ib_server->listen, tcpBackLog);
+        if (ipbind->ib_listener->mode == CM_NONE)
+          pr_inet_listen(ipbind->ib_listener->pool, ipbind->ib_listener,
+            tcpBackLog);
 
-        if (ipbind->ib_server->listen->mode == CM_ACCEPT)
-          inet_resetlisten(ipbind->ib_server->listen->pool,
-            ipbind->ib_server->listen);
+        if (ipbind->ib_listener->mode == CM_ACCEPT)
+          pr_inet_resetlisten(ipbind->ib_listener->pool, ipbind->ib_listener);
 
-        if (ipbind->ib_server->listen->mode == CM_LISTEN) {
-          FD_SET(ipbind->ib_server->listen->listen_fd, readfds);
-          if (ipbind->ib_server->listen->listen_fd > maxfd)
-            maxfd = ipbind->ib_server->listen->listen_fd;
+        if (ipbind->ib_listener->mode == CM_LISTEN) {
+          FD_SET(ipbind->ib_listener->listen_fd, readfds);
+          if (ipbind->ib_listener->listen_fd > maxfd)
+            maxfd = ipbind->ib_listener->listen_fd;
 
           /* Add this to the listener list as well. */
-          ipbind->ib_server->listen->next = listener_list;
-          listener_list = ipbind->ib_server->listen;
+          ipbind->ib_listener->next = listener_list;
+          listener_list = ipbind->ib_listener;
           listener_listlen++;
         }
       }
@@ -471,7 +474,7 @@ int pr_ipbind_listen(fd_set *readfds) {
   return maxfd;
 }
 
-int pr_ipbind_open(p_in_addr_t *addr, unsigned int port, conn_t *listen_conn,
+int pr_ipbind_open(pr_netaddr_t *addr, unsigned int port, conn_t *listen_conn,
     unsigned char isdefault, unsigned char islocalhost,
     unsigned char open_namebinds) {
   int res = 0;
@@ -492,7 +495,7 @@ int pr_ipbind_open(p_in_addr_t *addr, unsigned int port, conn_t *listen_conn,
   if (listen_conn)
     listen_conn->next = NULL;
 
-  ipbind->ib_server->listen = listen_conn;
+  ipbind->ib_listener = ipbind->ib_server->listen = listen_conn;
   ipbind->ib_isdefault = isdefault;
   ipbind->ib_islocalhost = islocalhost;
 
@@ -511,9 +514,7 @@ int pr_ipbind_open(p_in_addr_t *addr, unsigned int port, conn_t *listen_conn,
   if (islocalhost)
     ipbind_localhost_server = ipbind;
 
-
-  /* If requested, look for any namebinds for this ipbind, and open them.
-   */
+  /* If requested, look for any namebinds for this ipbind, and open them. */
   if (open_namebinds && ipbind->ib_namebinds) {
     register unsigned int i = 0;
     pr_namebind_t **namebinds = NULL;
@@ -526,7 +527,7 @@ int pr_ipbind_open(p_in_addr_t *addr, unsigned int port, conn_t *listen_conn,
     for (i = 0; i < ipbind->ib_namebinds->nelts; i++) {
       pr_namebind_t *nb = namebinds[i];
 
-      PR_OPEN_NAMEBIND(nb->nb_name, nb->nb_server->ipaddr,
+      PR_OPEN_NAMEBIND(nb->nb_name, nb->nb_server->addr,
         nb->nb_server->ServerPort);
     }
   }
@@ -537,7 +538,8 @@ int pr_ipbind_open(p_in_addr_t *addr, unsigned int port, conn_t *listen_conn,
   return 0;
 }
 
-int pr_namebind_close(const char *name, p_in_addr_t *addr, unsigned int port) {
+int pr_namebind_close(const char *name, pr_netaddr_t *addr,
+    unsigned int port) {
   pr_namebind_t *namebind = NULL;
 
   /* sanity checks */
@@ -559,8 +561,8 @@ int pr_namebind_close(const char *name, p_in_addr_t *addr, unsigned int port) {
   return 0;
 }
 
-int pr_namebind_create(server_rec *server, const char *name, p_in_addr_t *addr,
-    unsigned int port) {
+int pr_namebind_create(server_rec *server, const char *name,
+    pr_netaddr_t *addr, unsigned int port) {
   pr_ipbind_t *ipbind = NULL;
   pr_namebind_t *namebind = NULL, **namebinds = NULL;
 
@@ -617,16 +619,22 @@ int pr_namebind_create(server_rec *server, const char *name, p_in_addr_t *addr,
   namebind->nb_server->ServerFQDN = (server->ServerFQDN ?
     server->ServerFQDN : main_server->ServerFQDN);
 
-  namebind->nb_server->tcp_rwin = (server->tcp_rwin ? server->tcp_rwin :
-    main_server->tcp_rwin);
-  namebind->nb_server->tcp_swin = (server->tcp_swin ? server->tcp_swin :
-    main_server->tcp_swin);
+  namebind->nb_server->tcp_mss_len = (server->tcp_mss_len ?
+    server->tcp_mss_len : main_server->tcp_mss_len);
+  namebind->nb_server->tcp_rcvbuf_len = (server->tcp_rcvbuf_len ?
+    server->tcp_rcvbuf_len : main_server->tcp_rcvbuf_len);
+  namebind->nb_server->tcp_rcvbuf_override = (server->tcp_rcvbuf_override ?
+    TRUE : main_server->tcp_rcvbuf_override);
+  namebind->nb_server->tcp_sndbuf_len = (server->tcp_sndbuf_len ?
+    server->tcp_sndbuf_len : main_server->tcp_sndbuf_len);
+  namebind->nb_server->tcp_sndbuf_override = (server->tcp_sndbuf_override ?
+    TRUE : main_server->tcp_sndbuf_override);
 
-  namebind->nb_server->ipaddr = (server->ipaddr ? server->ipaddr :
-    main_server->ipaddr);
+  namebind->nb_server->addr = (server->addr ? server->addr :
+    main_server->addr);
   namebind->nb_server->ServerPort = (server->ServerPort ? server->ServerPort :
     main_server->ServerPort);
-  namebind->nb_server->listen = (server->listen ? server->listen :
+  namebind->nb_listener = (server->listen ? server->listen :
     main_server->listen);
 
   /* Add this namebind to the ipbind's list */
@@ -636,7 +644,7 @@ int pr_namebind_create(server_rec *server, const char *name, p_in_addr_t *addr,
   return 0;
 }
 
-pr_namebind_t *pr_namebind_find(const char *name, p_in_addr_t *addr,
+pr_namebind_t *pr_namebind_find(const char *name, pr_netaddr_t *addr,
     unsigned int port, unsigned char skip_inactive) {
   pr_ipbind_t *ipbind = NULL;
   pr_namebind_t *namebind = NULL;
@@ -682,7 +690,7 @@ pr_namebind_t *pr_namebind_find(const char *name, p_in_addr_t *addr,
   return NULL;
 }
 
-server_rec *pr_namebind_get_server(const char *name, p_in_addr_t *addr,
+server_rec *pr_namebind_get_server(const char *name, pr_netaddr_t *addr,
     unsigned int port) {
   pr_namebind_t *namebind = NULL;
 
@@ -693,7 +701,7 @@ server_rec *pr_namebind_get_server(const char *name, p_in_addr_t *addr,
   return namebind->nb_server;
 }
 
-int pr_namebind_open(const char *name, p_in_addr_t *addr, unsigned int port) {
+int pr_namebind_open(const char *name, pr_netaddr_t *addr, unsigned int port) {
   pr_namebind_t *namebind = NULL;
 
   /* sanity checks */
@@ -715,7 +723,7 @@ int pr_namebind_open(const char *name, p_in_addr_t *addr, unsigned int port) {
   return 0;
 }
 
-void pr_free_bindings(void) {
+void free_bindings(void) {
   if (binding_pool) {
     destroy_pool(binding_pool);
     binding_pool = NULL;
@@ -724,22 +732,21 @@ void pr_free_bindings(void) {
   memset(ipbind_table, 0, sizeof(ipbind_table));
 }
 
-static void pr_init_inetd_bindings(void) {
+static void init_inetd_bindings(void) {
   int res = 0;
   server_rec *serv = NULL;
   unsigned char *default_server = NULL, is_default = FALSE;
 
-  main_server->listen = inet_create_connection(main_server->pool, server_list,
-     STDIN_FILENO, NULL, INPORT_ANY, FALSE);
+  main_server->listen = pr_inet_create_connection(main_server->pool,
+    server_list, STDIN_FILENO, NULL, INPORT_ANY, FALSE);
 
-  /* Fill in all the important connection information
-   */
-  if (inet_get_conn_info(main_server->listen, STDIN_FILENO) == -1) {
+  /* Fill in all the important connection information. */
+  if (pr_inet_get_conn_info(main_server->listen, STDIN_FILENO) == -1) {
     log_pri(PR_LOG_ERR, "fatal: %s", strerror(errno));
 
     if (errno == ENOTSOCK)
       log_pri(PR_LOG_ERR, "(Running from command line? "
-                    "Use `ServerType standalone' in config file!)");
+        "Use `ServerType standalone' in config file!)");
     exit(1);
   }
 
@@ -747,13 +754,12 @@ static void pr_init_inetd_bindings(void) {
       FALSE)) != NULL && *default_server == TRUE)
     is_default = TRUE;
 
-  PR_CREATE_IPBIND(main_server, main_server->ipaddr);
-  PR_OPEN_IPBIND(main_server->ipaddr, main_server->ServerPort,
+  PR_CREATE_IPBIND(main_server, main_server->addr);
+  PR_OPEN_IPBIND(main_server->addr, main_server->ServerPort,
     main_server->listen, is_default, TRUE, TRUE);
   PR_ADD_IPBINDS(main_server);
 
-  /* Now attach the faked connection to all virtual servers
-   */
+  /* Now attach the faked connection to all virtual servers. */
   for (serv = main_server->next; serv; serv = serv->next) {
 
     /* Because this server is sharing the connection with the
@@ -766,13 +772,13 @@ static void pr_init_inetd_bindings(void) {
     register_cleanup(serv->listen->pool, &serv->listen, server_cleanup_cb,
       server_cleanup_cb);
 
-    is_default = TRUE;
+    is_default = FALSE;
     if ((default_server = get_param_ptr(serv->conf, "DefaultServer",
-        FALSE)) != NULL && *default_server != TRUE)
-      is_default = FALSE;
+        FALSE)) != NULL && *default_server == TRUE)
+      is_default = TRUE;
 
-    PR_CREATE_IPBIND(serv, serv->ipaddr);
-    PR_OPEN_IPBIND(serv->ipaddr, serv->ServerPort, serv->listen, is_default,
+    PR_CREATE_IPBIND(serv, serv->addr);
+    PR_OPEN_IPBIND(serv->addr, serv->ServerPort, serv->listen, is_default,
       FALSE, TRUE);
     PR_ADD_IPBINDS(serv);
   }
@@ -780,7 +786,7 @@ static void pr_init_inetd_bindings(void) {
   return;
 }
 
-static void pr_init_standalone_bindings(void) {
+static void init_standalone_bindings(void) {
   int res = 0;
   config_rec *c = NULL;
   server_rec *serv = NULL;
@@ -790,21 +796,30 @@ static void pr_init_standalone_bindings(void) {
   if ((c = find_config(main_server->conf, CONF_PARAM, "DefaultAddress",
       FALSE)) != NULL) {
     log_debug(DEBUG0, "setting default server address to %s",
-      inet_ascii(c->pool, (p_in_addr_t *) c->argv[0]));
-    main_server->ipaddr = (p_in_addr_t *) c->argv[0];
+      pr_netaddr_get_ipstr((pr_netaddr_t *) c->argv[0]));
+    main_server->addr = (pr_netaddr_t *) c->argv[0];
   }
 
   /* If a port is set to zero, the address/port is not bound to a socket
    * at all.
    */
+  if (main_server->ServerPort) {
+
+    /* If SocketBindTight is off, then pr_inet_create_connection() will
+     * create and bind to a wildcard socket.  However, should it be an
+     * IPv4 or an IPv6 wildcard socket?  Check the main_server->addr's
+     * family, and use that.
+     */
+    if (!SocketBindTight)
+      pr_inet_set_default_family(NULL,
+        pr_netaddr_get_family(main_server->addr));
 
-  if (main_server->ServerPort)
     main_server->listen =
-      inet_create_connection(main_server->pool, server_list, -1,
-        (SocketBindTight ? main_server->ipaddr : NULL),
+      pr_inet_create_connection(main_server->pool, server_list, -1,
+        (SocketBindTight ? main_server->addr : NULL),
         main_server->ServerPort, FALSE);
 
-  else
+  } else
     main_server->listen = NULL;
 
   if ((default_server = get_param_ptr(main_server->conf, "DefaultServer",
@@ -812,9 +827,8 @@ static void pr_init_standalone_bindings(void) {
     is_default = TRUE;
 
   if (main_server->ServerPort || is_default) {
-
-    PR_CREATE_IPBIND(main_server, main_server->ipaddr);
-    PR_OPEN_IPBIND(main_server->ipaddr, main_server->ServerPort,
+    PR_CREATE_IPBIND(main_server, main_server->addr);
+    PR_OPEN_IPBIND(main_server->addr, main_server->ServerPort,
       main_server->listen, is_default, TRUE, TRUE);
     PR_ADD_IPBINDS(main_server);
   }
@@ -829,19 +843,23 @@ static void pr_init_standalone_bindings(void) {
         is_default = TRUE;
 
       if (serv->ServerPort) {
-        serv->listen = inet_create_connection(serv->pool, server_list, -1,
-          (SocketBindTight ? serv->ipaddr : NULL), serv->ServerPort, FALSE);
+        if (!SocketBindTight)
+          pr_inet_set_default_family(NULL,
+            pr_netaddr_get_family(serv->addr));
 
-        PR_CREATE_IPBIND(serv, serv->ipaddr);
-        PR_OPEN_IPBIND(serv->ipaddr, serv->ServerPort, serv->listen, is_default,
+        serv->listen = pr_inet_create_connection(serv->pool, server_list, -1,
+          (SocketBindTight ? serv->addr : NULL), serv->ServerPort, FALSE);
+
+        PR_CREATE_IPBIND(serv, serv->addr);
+        PR_OPEN_IPBIND(serv->addr, serv->ServerPort, serv->listen, is_default,
           FALSE, TRUE);
         PR_ADD_IPBINDS(serv);
 
       } else if (is_default) {
         serv->listen = NULL;
 
-        PR_CREATE_IPBIND(serv, serv->ipaddr);
-        PR_OPEN_IPBIND(serv->ipaddr, serv->ServerPort, serv->listen, is_default,
+        PR_CREATE_IPBIND(serv, serv->addr);
+        PR_OPEN_IPBIND(serv->addr, serv->ServerPort, serv->listen, is_default,
           FALSE, TRUE);
         PR_ADD_IPBINDS(serv);
 
@@ -865,8 +883,8 @@ static void pr_init_standalone_bindings(void) {
       register_cleanup(serv->listen->pool, &serv->listen, server_cleanup_cb,
         server_cleanup_cb);
 
-      PR_CREATE_IPBIND(serv, serv->ipaddr);
-      PR_OPEN_IPBIND(serv->ipaddr, serv->ServerPort, NULL, is_default, FALSE,
+      PR_CREATE_IPBIND(serv, serv->addr);
+      PR_OPEN_IPBIND(serv->addr, serv->ServerPort, NULL, is_default, FALSE,
         TRUE);
       PR_ADD_IPBINDS(serv);
     }
@@ -876,12 +894,12 @@ static void pr_init_standalone_bindings(void) {
   return;
 }
 
-void pr_init_bindings(void) {
+void init_bindings(void) {
   if (ServerType == SERVER_INETD)
-    pr_init_inetd_bindings();
+    init_inetd_bindings();
 
   else if (ServerType == SERVER_STANDALONE)
-    pr_init_standalone_bindings();
+    init_standalone_bindings();
 
   return;
 }
