@@ -26,7 +26,7 @@
 
 /*
  * Data connection management functions
- * $Id: data.c,v 1.58 2003/02/12 19:03:36 castaglia Exp $
+ * $Id: data.c,v 1.70 2003/10/06 03:53:49 castaglia Exp $
  */
 
 #include "conf.h"
@@ -75,13 +75,12 @@ static RETSIGTYPE data_urgent(int sig) {
   signal(SIGURG, data_urgent);
 }
 
-static int _xlate_ascii_read(char *buf, int *bufsize, int *adjlen)
-{
+static int xlate_ascii_read(char *buf, int *bufsize, int *adjlen) {
   char *dest = buf,*src = buf;
   int thislen = *bufsize;
 
   *adjlen = 0;
-  while(thislen--) {
+  while (thislen--) {
     if (*src != '\r')
       *dest++ = *src++;
     else {
@@ -113,17 +112,16 @@ static int _xlate_ascii_read(char *buf, int *bufsize, int *adjlen)
  *           and should be the difference between buflen's original
  *           value and its value when this function returns
  */
-static void _xlate_ascii_write(char **buf, unsigned int *buflen,
-    unsigned int bufsize, unsigned int *expand) {
+static unsigned int xlate_ascii_write(char **buf, unsigned int *buflen,
+    unsigned int bufsize) {
   char *tmpbuf = *buf;
   unsigned int tmplen = *buflen;
   unsigned int lfcount = 0;
+  unsigned int added = 0;
+
   int res = 0;
   register unsigned int i = 0;
 
-  /* Make sure this is zero (could be a holdover from a previous call). */
-  *expand = 0;
-
   /* First, determine how many bare LFs are present. */
   if (tmpbuf[0] == '\n')
     lfcount++;
@@ -132,6 +130,10 @@ static void _xlate_ascii_write(char **buf, unsigned int *buflen,
     if (tmpbuf[i] == '\n' && tmpbuf[i-1] != '\r')
       lfcount++;
 
+  if (lfcount == 0)
+    /* No translation needed. */
+    return 0;
+
   /* Assume that for each LF (including a leading LF), space for another
    * char (a '\r') is needed.  Determine whether there is enough space in
    * the buffer for the adjusted data.  If not, allocate a new buffer that is
@@ -142,27 +144,21 @@ static void _xlate_ascii_write(char **buf, unsigned int *buflen,
    *
    * Note: the res variable is needed in order to force signedness of the
    * resulting difference.  Without it, this condition would never evaluate
-   * to true, as C's promotion rules would insure that the resulting value
+   * to true, as C's promotion rules would ensure that the resulting value
    * would be of the same type as the operands: an unsigned int (which will
    * never be less than zero).
    */
-  if ((res = (bufsize - tmplen - lfcount)) < 0) {
+  if ((res = (bufsize - tmplen - lfcount)) <= 0) {
     pool *copy_pool = make_sub_pool(session.xfer.p);
     char *copy_buf = pcalloc(copy_pool, tmplen);
 
-    memmove(copy_buf, tmpbuf, tmplen);
+    memcpy(copy_buf, tmpbuf, tmplen);
 
     /* Allocate a new session.xfer.buf of the needed size. */
-    session.xfer.bufsize = tmplen + lfcount;
+    session.xfer.bufsize = tmplen + lfcount + 1;
     session.xfer.buf = pcalloc(session.xfer.p, session.xfer.bufsize);
 
-    /* Allow space for a CR to be inserted before an LF if an LF is the
-     * first character in the buffer.
-     */
-    session.xfer.buf++;
-    session.xfer.bufstart = session.xfer.buf;
-
-    memmove(session.xfer.buf, copy_buf, tmplen);
+    memcpy(session.xfer.buf, copy_buf, tmplen);
     destroy_pool(copy_pool);
 
     tmpbuf = session.xfer.buf;
@@ -174,29 +170,29 @@ static void _xlate_ascii_write(char **buf, unsigned int *buflen,
     /* Shift everything in the buffer to the right one character, making
      * space for a '\r'
      */
-    memmove(&(tmpbuf[1]), &(tmpbuf[0]), bufsize);
+    memmove(&(tmpbuf[1]), &(tmpbuf[0]), tmplen);
     tmpbuf[0] = '\r';
 
-    /* Increment the number of "expanded" characters, and decrement the
-     * number of bare LFs.
+    /* Increment the number of added characters, and decrement the number
+     * of bare LFs.
      */
-    (*expand)++;
+    added++;
     lfcount--;
   }
 
   for (i = 1; i < bufsize && (lfcount > 0); i++) {
     if (tmpbuf[i] == '\n' && tmpbuf[i-1] != '\r') {
-      memmove(&(tmpbuf[i+1]), &(tmpbuf[i]), bufsize - i);
+      memmove(&(tmpbuf[i+1]), &(tmpbuf[i]), bufsize - i - 1);
       tmpbuf[i] = '\r';
-      (*expand)++;
+      added++;
       lfcount--;
     }
   }
 
-  /* Always make sure the buffer is NUL-terminated. */
-  tmpbuf[tmplen + (*expand)] = '\0';
   *buf = tmpbuf;
-  *buflen = tmplen + (*expand);
+  *buflen = tmplen + added;
+
+  return added;
 }
 
 static void data_new_xfer(char *filename, int direction) {
@@ -209,9 +205,8 @@ static void data_new_xfer(char *filename, int direction) {
   session.xfer.filename = pstrdup(session.xfer.p,filename);
   session.xfer.direction = direction;
   session.xfer.bufsize = PR_TUNABLE_BUFFER_SIZE;
-  session.xfer.buf = (char *)pcalloc(session.xfer.p, PR_TUNABLE_BUFFER_SIZE + 1);
+  session.xfer.buf = pcalloc(session.xfer.p, PR_TUNABLE_BUFFER_SIZE + 1);
   session.xfer.buf++;	/* leave room for ascii translation */
-  session.xfer.bufstart = session.xfer.buf;
   session.xfer.buflen = 0;
 }
 
@@ -232,34 +227,35 @@ static int data_pasv_open(char *reason, off_t size) {
    * lookups, and then set it to what the configuration wants it to
    * be.
    */
-  rev = inet_reverse_dns(session.xfer.p, ServerUseReverseDNS);
+  rev = pr_netaddr_set_reverse_dns(ServerUseReverseDNS);
 
   /* Protocol and socket options should be set before handshaking. */
 
   if (session.xfer.direction == PR_NETIO_IO_RD) {
-    inet_set_socket_opts(session.d->pool,session.d,
-      (main_server->tcp_rwin_override ?  main_server->tcp_rwin : 0), 0);
-    inet_set_proto_opts(session.pool, session.d, 0, 0, 1, 1);
+    pr_inet_set_socket_opts(session.d->pool, session.d,
+      (main_server->tcp_rcvbuf_override ?  main_server->tcp_rcvbuf_len : 0), 0);
+    pr_inet_set_proto_opts(session.pool, session.d, main_server->tcp_mss_len, 0,
+      0, 1, 1);
     
   } else {
-    inet_set_socket_opts(session.d->pool,session.d, 0,
-      (main_server->tcp_swin_override ?  main_server->tcp_swin : 0));
-    inet_set_proto_opts(session.pool, session.d, 0, 0, 1, 1);
+    pr_inet_set_socket_opts(session.d->pool, session.d,
+      0, (main_server->tcp_sndbuf_override ?  main_server->tcp_sndbuf_len : 0));
+    pr_inet_set_proto_opts(session.pool, session.d, main_server->tcp_mss_len, 0,
+      0, 1, 1);
   }
 
-  c = inet_accept(session.xfer.p, session.d, session.c, -1, -1, TRUE);
-  inet_reverse_dns(session.xfer.p,rev);
+  c = pr_inet_accept(session.xfer.p, session.d, session.c, -1, -1, TRUE);
+  pr_netaddr_set_reverse_dns(rev);
 
   if (c && c->mode != CM_ERROR) {
-    inet_close(session.pool, session.d);
-    inet_setnonblock(session.pool, c);
+    pr_inet_close(session.pool, session.d);
+    pr_inet_set_nonblock(session.pool, c);
     session.d = c;
 
     log_debug(DEBUG4, "passive data connection opened - local  : %s:%d",
-               inet_ntoa(*session.d->local_ipaddr), session.d->local_port);
+      pr_netaddr_get_ipstr(session.d->local_addr), session.d->local_port);
     log_debug(DEBUG4, "passive data connection opened - remote : %s:%d",
-              inet_ntoa(*session.d->remote_ipaddr),
-              session.d->remote_port);
+      pr_netaddr_get_ipstr(session.d->remote_addr), session.d->remote_port);
 
     if (session.xfer.xfer_type != STOR_UNIQUE) {
       if (size)
@@ -315,8 +311,8 @@ static int data_active_open(char *reason, off_t size) {
   if (!reason && session.xfer.filename)
     reason = session.xfer.filename;
 
-  session.d = inet_create_connection(session.pool, NULL, -1,
-    session.c->local_ipaddr, session.c->local_port-1, TRUE);
+  session.d = pr_inet_create_connection(session.pool, NULL, -1,
+    session.c->local_addr, session.c->local_port-1, TRUE);
 
   /* Set the "stalled" timer, if any, to prevent the connection
    * open from taking too long
@@ -324,22 +320,24 @@ static int data_active_open(char *reason, off_t size) {
   if (TimeoutStalled)
     add_timer(TimeoutStalled, TIMER_STALLED, NULL, stalled_timeout_cb);
 
-  rev = inet_reverse_dns(session.pool, ServerUseReverseDNS);
+  rev = pr_netaddr_set_reverse_dns(ServerUseReverseDNS);
 
   /* Protocol and socket options should be set before handshaking. */
 
   if (session.xfer.direction == PR_NETIO_IO_RD) {
-    inet_set_socket_opts(session.d->pool,session.d,
-       (main_server->tcp_rwin_override ?  main_server->tcp_rwin : 0), 0);
-    inet_set_proto_opts(session.pool, session.d, 0, 0, 1, 1);
+    pr_inet_set_socket_opts(session.d->pool, session.d,
+      (main_server->tcp_rcvbuf_override ?  main_server->tcp_rcvbuf_len : 0), 0);
+    pr_inet_set_proto_opts(session.pool, session.d, main_server->tcp_mss_len, 0,
+      0, 1, 1);
     
   } else {
-    inet_set_socket_opts(session.d->pool,session.d, 0,
-      (main_server->tcp_swin_override ?  main_server->tcp_swin : 0));
-    inet_set_proto_opts(session.pool, session.d, 0, 0, 1, 1);
+    pr_inet_set_socket_opts(session.d->pool, session.d,
+      0, (main_server->tcp_sndbuf_override ?  main_server->tcp_sndbuf_len : 0));
+    pr_inet_set_proto_opts(session.pool, session.d, main_server->tcp_mss_len, 0,
+      0, 1, 1);
   }
 
-  if (inet_connect(session.d->pool, session.d, &session.data_addr,
+  if (pr_inet_connect(session.d->pool, session.d, &session.data_addr,
       session.data_port) == -1) {
     pr_response_add_err(R_425, "Unable to build data connection: %s",
       strerror(session.d->xerrno));
@@ -348,17 +346,16 @@ static int data_active_open(char *reason, off_t size) {
     return -1;
   }
 
-  c = inet_openrw(session.pool, session.d, NULL, PR_NETIO_STRM_DATA,
+  c = pr_inet_openrw(session.pool, session.d, NULL, PR_NETIO_STRM_DATA,
     session.d->listen_fd, -1, -1, TRUE);
 
-  inet_reverse_dns(session.pool,rev);
+  pr_netaddr_set_reverse_dns(rev);
 
   if (c) {
     log_debug(DEBUG4, "active data connection opened - local  : %s:%d",
-	      inet_ntoa(*session.d->local_ipaddr), session.d->local_port);
+      pr_netaddr_get_ipstr(session.d->local_addr), session.d->local_port);
     log_debug(DEBUG4, "active data connection opened - remote : %s:%d",
-	      inet_ntoa(*session.d->remote_ipaddr),
-	      session.d->remote_port);
+      pr_netaddr_get_ipstr(session.d->remote_addr), session.d->remote_port);
 
     if (session.xfer.xfer_type != STOR_UNIQUE) {
       if (size)
@@ -392,13 +389,12 @@ static int data_active_open(char *reason, off_t size) {
       pr_response_send(R_150, "FILE: %s", reason);
     }
 
-    inet_close(session.pool,session.d);
-    inet_setnonblock(session.pool,session.d);
+    pr_inet_close(session.pool, session.d);
+    pr_inet_set_nonblock(session.pool, session.d);
     session.d = c;
     return 0;
   }
 
-
   pr_response_add_err(R_425, "Unable to build data connection: %s",
     strerror(session.d->xerrno));
   destroy_pool(session.d->pool);
@@ -410,7 +406,7 @@ void pr_data_reset(void) {
   if (session.d && session.d->pool)
     destroy_pool(session.d->pool);
   session.d = NULL;
-  session.sf_flags &= (SF_ALL^(SF_ABORT|SF_XFER|SF_PASSIVE|SF_ASCII_OVERRIDE));
+  session.sf_flags &= (SF_ALL^(SF_ABORT|SF_XFER|SF_PASSIVE|SF_ASCII_OVERRIDE|SF_EPSV_ALL));
 }
 
 void pr_data_init(char *filename, int direction) {
@@ -445,7 +441,7 @@ int pr_data_open(char *filename, char *reason, int direction, off_t size) {
       end_login(1);
     }
 
-    res = data_pasv_open(reason,size);
+    res = data_pasv_open(reason, size);
 
   /* Active data transfers... */
   } else {
@@ -455,7 +451,7 @@ int pr_data_open(char *filename, char *reason, int direction, off_t size) {
       end_login(1);
     }
 
-    res = data_active_open(reason,size);
+    res = data_active_open(reason, size);
   }
 
   if (res >= 0) {
@@ -481,19 +477,18 @@ int pr_data_open(char *filename, char *reason, int direction, off_t size) {
     if (TimeoutNoXfer)
       reset_timer(TIMER_NOXFER, ANY_MODULE);
 
-    /* allow aborts */
+    /* Allow aborts. */
 
     /* Set the current NetIO stream to allow interrupted syscalls, so our
      * SIGURG handler can interrupt it
      */
     pr_netio_set_poll_interval(nstrm, 1);
 
-    /* PORTABILITY: sigaction is used here to allow us
-     * to indicate (w/ POSIX at least) that we want
-     * SIGURG to interrupt syscalls.  Put in whatever
-     * is necessary for your arch here (probably not necessary
-     * as the only _important_ interrupted syscall is select()),
-     * which on any sensible system is interrupted.
+    /* PORTABILITY: sigaction is used here to allow us to indicate
+     * (w/ POSIX at least) that we want SIGURG to interrupt syscalls.  Put
+     * in whatever is necessary for your arch here; probably not necessary
+     * as the only _important_ interrupted syscall is select(), which on
+     * any sensible system is interrupted.
      */
 
     act.sa_handler = data_urgent;
@@ -504,7 +499,7 @@ int pr_data_open(char *filename, char *reason, int direction, off_t size) {
 #endif
     sigaction(SIGURG, &act, NULL);
 #ifdef HAVE_SIGINTERRUPT
-    /* this is the BSD way of ensuring interruption.
+    /* This is the BSD way of ensuring interruption.
      * Linux uses it too (??)
      */
     siginterrupt(SIGURG, 1);
@@ -519,7 +514,7 @@ void pr_data_close(int quiet) {
   nstrm = NULL;
 
   if (session.d) {
-    inet_lingering_close(session.pool, session.d, PR_TUNABLE_TIMEOUTLINGER);
+    pr_inet_lingering_close(session.pool, session.d, PR_TUNABLE_TIMEOUTLINGER);
     session.d = NULL;
   }
 
@@ -552,7 +547,7 @@ void pr_data_close(int quiet) {
 void pr_data_cleanup(void) {
   /* sanity check */
   if (session.d) {
-    inet_lingering_close(session.pool, session.d, PR_TUNABLE_TIMEOUTLINGER);
+    pr_inet_lingering_close(session.pool, session.d, PR_TUNABLE_TIMEOUTLINGER);
     session.d = NULL;
   }
 
@@ -571,7 +566,7 @@ void pr_data_abort(int err, int quiet) {
   nstrm = NULL;
 
   if (session.d) {
-    inet_lingering_close(session.pool, session.d, PR_TUNABLE_TIMEOUTLINGER);
+    pr_inet_lingering_close(session.pool, session.d, PR_TUNABLE_TIMEOUTLINGER);
     session.d = NULL;
   }
 
@@ -738,7 +733,7 @@ void pr_data_abort(int err, int quiet) {
 	msg = msgbuf;
     }
 
-    pr_response_add_err(respcode, fmt ? fmt : "Transfer aborted.  %s",
+    pr_response_add_err(respcode, fmt ? fmt : "Transfer aborted. %s",
       msg ? msg : "");
   }
 
@@ -754,11 +749,12 @@ void pr_data_abort(int err, int quiet) {
  */
 
 int pr_data_xfer(char *cl_buf, int cl_size) {
-  char *buf = session.xfer.buf;
   int len = 0;
   int total = 0;
 
   if (session.xfer.direction == PR_NETIO_IO_RD) {
+    char *buf = session.xfer.buf;
+
     if (session.d) {
       if (session.sf_flags & (SF_ASCII|SF_ASCII_OVERRIDE)) {
         int adjlen,buflen;
@@ -783,13 +779,13 @@ int pr_data_xfer(char *cl_buf, int cl_size) {
 	     * adjlen is returned as the number of characters unprocessed in
 	     *        the buffer (to be dealt with later)
 	     *
-	     * We skip the call to _xlate_ascii_read() in one case:
+	     * We skip the call to xlate_ascii_read() in one case:
 	     * when we have one character in the buffer and have reached
-	     * end of data, this is so that _xlate_ascii_read() won't sit
+	     * end of data, this is so that xlate_ascii_read() won't sit
 	     * forever waiting for the next character after a final '\r'.
 	     */
 	    if (len > 0 || buflen > 1)
-	      _xlate_ascii_read(buf, &buflen, &adjlen);
+	      xlate_ascii_read(buf, &buflen, &adjlen);
 	
 	    /* now copy everything we can into cl_buf */
 	    if (buflen > cl_size) {
@@ -818,10 +814,10 @@ int pr_data_xfer(char *cl_buf, int cl_size) {
 	
 	  /* Restart if data was returned by pr_netio_read() (len > 0) but
 	   * no data was copied to the client buffer (buflen = 0).
-	   * This indicates that _xlate_ascii_read() needs more data
+	   * This indicates that xlate_ascii_read() needs more data
 	   * in order to translate, so we need to call pr_netio_read() again.
            */
-	} while(len > 0 && buflen == 0);
+	} while (len > 0 && buflen == 0);
 
         /* Return how much data we actually copied into the client buffer.
          */
@@ -840,45 +836,38 @@ int pr_data_xfer(char *cl_buf, int cl_size) {
 
   } else { /* PR_NETIO_IO_WR */
 
-    /* copy client buffer to internal buffer, and
-     * xlate ascii as necessary
-     */
     while (cl_size) {
-      int o_size, size = cl_size;
+      int buflen = cl_size;
+      unsigned int xferbuflen;
 
-      if (size > PR_TUNABLE_BUFFER_SIZE)
-        size = PR_TUNABLE_BUFFER_SIZE;
+      if (buflen > PR_TUNABLE_BUFFER_SIZE)
+        buflen = PR_TUNABLE_BUFFER_SIZE;
 
-      o_size = size;
-      memcpy(buf, cl_buf, size);
+      xferbuflen = buflen;
 
-      while (size) {
-        char *wb = buf;
-        unsigned int wsize = size, adjlen = 0;
+      /* Fill up our internal buffer. */
+      memcpy(session.xfer.buf, cl_buf, buflen);
 
-        if (session.sf_flags & (SF_ASCII|SF_ASCII_OVERRIDE))
-          _xlate_ascii_write(&wb, &wsize, session.xfer.bufsize, &adjlen);
+      if (session.sf_flags & (SF_ASCII|SF_ASCII_OVERRIDE)) {
 
-        if (pr_netio_write(session.d->outstrm, wb, wsize) == -1)
-          return -1;
-
-        if (TimeoutStalled)
-          reset_timer(TIMER_STALLED, ANY_MODULE);
-
-        /* Do not take any added CRs into account for the session sum. */
-        total += (wsize - adjlen);
-        size -= (wsize - adjlen);
-
-        if (size) {
-          /* Advance the output buffer pointer into unsent buffer space. */
-          wb += wsize;
-	  memcpy(buf, wb, size);
-          buf[size] = '\0';
-        }
+        /* Scan the internal buffer, looking for LFs with no preceding CRs.
+         * Add CRs (and expand the internal buffer) as necessary. xferbuflen
+         * will be adjusted so that it contains the length of data in
+         * the internal buffer, including any added CRs.
+         */
+        xlate_ascii_write(&session.xfer.buf, &xferbuflen, session.xfer.bufsize);
       }
 
-      cl_size -= o_size;
-      cl_buf += o_size;
+      if (pr_netio_write(session.d->outstrm, session.xfer.buf,
+          xferbuflen) == -1)
+        return -1;
+
+      if (TimeoutStalled)
+        reset_timer(TIMER_STALLED, ANY_MODULE);
+
+      cl_size -= buflen;
+      cl_buf += buflen;
+      total += buflen;
     }
 
     len = total;
@@ -917,7 +906,7 @@ pr_sendfile_t pr_data_sendfile(int retr_fd, off_t *offset, size_t count) {
       return -1;
 
   for (;;) {
-#if defined(HAVE_LINUX_SENDFILE)
+#if defined(HAVE_LINUX_SENDFILE) || defined(HAVE_SOLARIS_SENDFILE)
     off_t orig_offset = *offset;
 
     /* Linux semantics are fairly straightforward in a glibc 2.x world:
@@ -945,7 +934,9 @@ pr_sendfile_t pr_data_sendfile(int retr_fd, off_t *offset, size_t count) {
       session.total_bytes += len;
       total += len;
 
+      pr_signals_handle();
       continue;
+
     } else if (len == -1) {
       /* Linux updates offset on error, not len like BSD, fix up so
        * BSD-based code works.
