@@ -24,10 +24,8 @@
  * the source code for OpenSSL in the source distribution.
  */
 
-/*
- * ProFTPD logging support.
- *
- * $Id: log.c,v 1.55 2003/02/12 19:03:36 castaglia Exp $
+/* ProFTPD logging support.
+ * $Id: log.c,v 1.61 2003/10/08 18:13:03 castaglia Exp $
  */
 
 #include "conf.h"
@@ -42,7 +40,7 @@ static int logstderr = TRUE;
 static int debug_level = DEBUG0;	/* Default is no debug logging */
 static int facility = LOG_DAEMON;
 static int set_facility = -1;
-static char systemlog_fn[MAX_PATH_LEN] = {'\0'};
+static char systemlog_fn[PR_TUNABLE_PATH_MAX] = {'\0'};
 static char systemlog_host[256] = {'\0'};
 static int systemlog_fd = -1;
 
@@ -98,7 +96,7 @@ int log_open_xfer(const char *path) {
   return xfer_fd;
 }
 
-int log_xfer(long xfertime, char *remhost, off_t fsize, char *fname,
+int log_xfer(long xfertime, const char *remhost, off_t fsize, char *fname,
     char xfertype, char direction, char access_mode, char *user,
     char abort_flag) {
 
@@ -131,7 +129,8 @@ int log_xfer(long xfertime, char *remhost, off_t fsize, char *fname,
  * but I haven't been able to test them.
  */
 
-int log_wtmp(char *line, const char *name, const char *host, p_in_addr_t *ip) {
+int log_wtmp(char *line, const char *name, const char *host,
+    pr_netaddr_t *ip) {
   struct stat buf;
   struct utmp ut;
   int res = 0;
@@ -242,10 +241,10 @@ int log_wtmp(char *line, const char *name, const char *host, p_in_addr_t *ip) {
 int log_openfile(const char *log_file, int *log_fd, mode_t log_mode) {
   pool *tmp_pool = NULL;
   char *tmp = NULL, *lf;
-  unsigned char *allow_log_symlinks = NULL;
+  unsigned char have_stat = FALSE, *allow_log_symlinks = NULL;
   struct stat sbuf;
 
-  /* sanity check */
+  /* Sanity check */
   if (!log_file || !log_fd) {
     errno = EINVAL;
     return -1;
@@ -255,7 +254,8 @@ int log_openfile(const char *log_file, int *log_fd, mode_t log_mode) {
   tmp_pool = make_sub_pool(permanent_pool);
   lf = pstrdup(tmp_pool, log_file);
 
-  if ((tmp = strrchr(lf, '/')) == NULL) {
+  tmp = strrchr(lf, '/');
+  if (tmp == NULL) {
     log_debug(DEBUG0, "inappropriate log file: %s", lf);
     destroy_pool(tmp_pool);
     return -1;
@@ -282,7 +282,7 @@ int log_openfile(const char *log_file, int *log_fd, mode_t log_mode) {
 
   /* Do not log to world-writeable directories */
   if (sbuf.st_mode & S_IWOTH) {
-    log_debug(DEBUG0, "error: %s is a world writeable directory", lf);
+    log_pri(PR_LOG_NOTICE, "error: %s is a world writeable directory", lf);
     destroy_pool(tmp_pool);
     return LOG_WRITEABLE_DIR;
   }
@@ -296,30 +296,107 @@ int log_openfile(const char *log_file, int *log_fd, mode_t log_mode) {
     FALSE);
 
   if (!allow_log_symlinks || *allow_log_symlinks == FALSE) {
+    int flags = O_APPEND|O_CREAT|O_WRONLY;
 
-    /* Prevent a race condition between stat() and open() by opening the
-     * file now, _then_ checking to see if it's a symlink
+#ifdef O_NOFOLLOW
+    /* On systems that support the O_NOFOLLOW flag (e.g. Linux and FreeBSD),
+     * use it so that the path being opened, if it is a symlink, is not
+     * followed.
      */
-    if ((*log_fd = open(lf, O_APPEND|O_CREAT|O_WRONLY,
-          log_mode)) == -1) {
-      destroy_pool(tmp_pool);
-      return -1;
+    flags |= O_NOFOLLOW;
+
+#elif defined(SOLARIS2)
+    /* Solaris doesn't support the O_NOFOLLOW flag.  Instead, in their
+     * wisdom (hah!), Solaris decided that if the given path is a symlink
+     * and the flags O_CREAT and O_EXCL are set, the link is not followed.
+     * Right.  The problem here is the case where the path is not a symlink;
+     * using O_CREAT|O_EXCL will then cause the open() to fail if the
+     * file already exists.
+     */
+    flags |= O_EXCL;
+#endif /* O_NOFOLLOW or SOLARIS2 */
+
+    *log_fd = open(lf, flags, log_mode);
+    if (*log_fd == -1) {
+
+      if (errno != EEXIST) {
+        destroy_pool(tmp_pool);
+
+        /* More portability fun: Linux likes to report ELOOP if O_NOFOLLOW
+         * is used to open a symlink file; FreeBSD likes to return EMLINK.
+         * Both would lead to rather misleading error messages being
+         * logged.  Catch these errnos, and return the value that properly
+         * informs the caller that the given path was an illegal symlink.
+         */
+
+        switch (errno) {
+#ifdef ELOOP
+          case ELOOP:
+            return LOG_SYMLINK;
+#endif /* ELOOP */
+
+#ifdef EMLINK
+          case EMLINK:
+            return LOG_SYMLINK;
+#endif /* EMLINK */
+        }
+
+        return -1;
+
+      } else {
+#if defined(SOLARIS2)
+        /* On Solaris, because of the stupid multiplexing of O_CREAT and
+         * O_EXCL to get open() not to follow a symlink, it's possible that
+         * the path already exists.  Now, we'll try to open() without
+         * O_EXCL, then lstat() the path to see if this pre-existing file is
+         * a symlink or a regular file.
+         *
+         * Note that because this check cannot be done atomically on Solaris,
+         * the possibility of a race condition/symlink attack still exists.
+         * Solaris doesn't provide a good way around this situation.
+         */
+        flags &= ~O_EXCL;
+
+        *log_fd = open(lf, flags, log_mode);
+        if (*log_fd == -1) {
+          destroy_pool(tmp_pool);
+          return -1;
+        }
+
+        /* The race condition on Solaris is here, between the open() call
+         * above and the lstat() call below...
+         */
+
+        if (lstat(lf, &sbuf) != -1)
+          have_stat = TRUE;
+#else
+        destroy_pool(tmp_pool);
+        return -1;
+#endif /* SOLARIS2 */
+      }
     }
 
     /* Stat the file using the descriptor, not the path */
-    if (fstat(*log_fd, &sbuf) != -1 && S_ISLNK(sbuf.st_mode)) {
-      log_debug(DEBUG0, "error: %s is a symbolic link", lf);
+    if (!have_stat && fstat(*log_fd, &sbuf) != -1)
+      have_stat = TRUE;
+
+    if (!have_stat || S_ISLNK(sbuf.st_mode)) {
+      log_debug(DEBUG0, !have_stat ? "error: unable to stat %s" :
+        "error: %s is a symbolic link", lf);
+
       close(*log_fd);
       *log_fd = -1;
       destroy_pool(tmp_pool);
       return LOG_SYMLINK;
     }
 
-  } else
-    if ((*log_fd = open(lf, O_CREAT|O_APPEND|O_WRONLY, log_mode)) == -1) {
+  } else {
+    *log_fd = open(lf, O_CREAT|O_APPEND|O_WRONLY, log_mode);
+    if (*log_fd == -1) {
       destroy_pool(tmp_pool);
       return -1;
     }
+  }
 
   destroy_pool(tmp_pool);
   return 0;
@@ -377,7 +454,7 @@ void log_discard(void) {
   syslog_discard = TRUE;
 }
 
-static void log(int priority, int f, char *s) {
+static void log_write(int priority, int f, char *s) {
   unsigned int *max_priority = NULL;
   char serverinfo[PR_TUNABLE_BUFFER_SIZE] = {'\0'};
 
@@ -390,7 +467,8 @@ static void log(int priority, int f, char *s) {
     if (session.c && session.c->remote_name) {
       snprintf(serverinfo + strlen(serverinfo),
         sizeof(serverinfo) - strlen(serverinfo), " (%s[%s])",
-        session.c->remote_name, inet_ntoa(*session.c->remote_ipaddr));
+        session.c->remote_name,
+        pr_netaddr_get_ipstr(session.c->remote_addr));
       serverinfo[sizeof(serverinfo)-1] = '\0';
     }
   }
@@ -461,7 +539,7 @@ void log_pri(int priority, const char *fmt, ...) {
   /* Always make sure the buffer is NUL-terminated. */
   buf[sizeof(buf) - 1] = '\0';
 
-  log(priority, facility, buf);
+  log_write(priority, facility, buf);
 }
 
 /* Like log_pri(), but sends the log entry in the LOG_AUTHPRIV
@@ -479,7 +557,7 @@ void log_auth(int priority, const char *fmt, ...) {
   /* Always make sure the buffer is NUL-terminated. */
   buf[sizeof(buf) - 1] = '\0';
 
-  log(priority, LOG_AUTHPRIV, buf);
+  log_write(priority, LOG_AUTHPRIV, buf);
 }
 
 /* Disable logging to stderr, should be done right before forking
@@ -547,7 +625,7 @@ void log_debug(int level, const char *fmt, ...) {
   /* Always make sure the buffer is NUL-terminated. */
   buf[sizeof(buf) - 1] = '\0';
 
-  log(PR_LOG_DEBUG, facility, buf);
+  log_write(PR_LOG_DEBUG, facility, buf);
 }
 
 void init_log(void) {
@@ -556,7 +634,8 @@ void init_log(void) {
   if (gethostname(buf, sizeof(buf)) == -1)
     sstrncpy(buf, "localhost", sizeof(buf));
 
-  sstrncpy(systemlog_host, inet_validate(buf), sizeof(systemlog_host));
+  sstrncpy(systemlog_host, (char *) pr_inet_validate(buf),
+    sizeof(systemlog_host));
   memset(systemlog_fn, '\0', sizeof(systemlog_fn));
   log_closesyslog();
 }
