@@ -25,6 +25,7 @@
 
 /*
  * Module handling routines
+ * $Id: modules.c,v 1.30 2003/09/08 00:31:32 castaglia Exp $
  */
 
 #include "conf.h"
@@ -39,8 +40,9 @@ struct stash {
 
   union {
     conftable *sym_conf;
-    cmdtable  *sym_cmd;
+    cmdtable *sym_cmd;
     authtable *sym_auth;
+    cmdtable *sym_hook;
     void *sym_generic;
   } ptr;
 };
@@ -63,11 +65,14 @@ module *curr_module = NULL;			/* Current running module */
 
 extern module *static_modules[];
 
-typedef struct postparse_cb {
-  struct postparse_cb *next, *prev;
+typedef struct mod_cb {
+  struct mod_cb *next, *prev;
 
-  int (*module_postparse_init_cb)(void);
-} postparse_t;
+  int (*module_cb)(void);
+} module_cb_t;
+
+static pool *daemon_cb_pool = NULL;
+static xaset_t *daemon_startups = NULL;
 
 static pool *postparse_init_pool = NULL;
 static xaset_t *postparse_inits = NULL;
@@ -94,7 +99,7 @@ static int sym_cmp(struct stash *s1, struct stash *s2) {
 
   ret = strcmp(s1->sym_name,s2->sym_name);
 
-  /* higher priority modules must go BEFORE lower priority in the
+  /* Higher priority modules must go BEFORE lower priority in the
    * hash tables.
    */
 
@@ -153,6 +158,14 @@ int pr_stash_add_symbol(pr_stash_type_t sym_type, void *data) {
       sym->ptr.sym_auth = data;
       break;
 
+    case PR_SYM_HOOK:
+      sym = sym_alloc();
+      sym->sym_type = PR_SYM_HOOK;
+      sym->sym_name = ((cmdtable *) data)->command;
+      sym->sym_module = ((cmdtable *) data)->m;
+      sym->ptr.sym_hook = data;
+      break;
+
     default:
       errno = EINVAL;
       return -1;
@@ -161,7 +174,7 @@ int pr_stash_add_symbol(pr_stash_type_t sym_type, void *data) {
   idx = symtab_hash(sym->sym_name);
 
   if (!symbol_table[idx])
-    symbol_table[idx] = xaset_create(permanent_pool, (XASET_COMPARE) sym_cmp);
+    symbol_table[idx] = xaset_create(symbol_pool, (XASET_COMPARE) sym_cmp);
 
   xaset_insert_sort(symbol_table[idx], (xasetmember_t *) sym, TRUE);
   return idx;
@@ -228,6 +241,9 @@ void *pr_stash_get_symbol(pr_stash_type_t sym_type, const char *name,
 
     case PR_SYM_AUTH:
       return sym ? sym->ptr.sym_auth : NULL;
+
+    case PR_SYM_HOOK:
+      return sym ? sym->ptr.sym_hook : NULL;
   }
 
   /* In case the compiler complains */
@@ -319,6 +335,24 @@ int pr_stash_remove_symbol(pr_stash_type_t sym_type, const char *sym_name,
       break;
     }
 
+    case PR_SYM_HOOK: {
+      int idx = -1;
+      cmdtable *tab = NULL;
+
+      tab = pr_stash_get_symbol(PR_SYM_HOOK, sym_name, NULL, &idx);
+
+      while (tab) {
+        if (!sym_module || curr_sym->sym_module == sym_module) {
+          xaset_remove(symbol_table[symtab_idx], (xasetmember_t *) curr_sym);
+          destroy_pool(curr_sym->sym_pool);
+        }
+
+        tab = pr_stash_get_symbol(PR_SYM_HOOK, sym_name, NULL, &idx);
+      }
+
+      break;
+    }
+
     default:
       errno = EINVAL;
       return -1;
@@ -372,38 +406,7 @@ privdata_t *mod_privdata_find(cmd_rec *cmd, char *tag, module *m)
   return (i < cmd->privarr->nelts ? *p : NULL);
 }
 
-modret_t *call_module_auth(module *m, modret_t *(*func)(cmd_rec*), cmd_rec *cmd)
-{
-  modret_t *res;
-  module *prev_module = curr_module;
-
-  if (!cmd->tmp_pool)
-    cmd->tmp_pool = make_sub_pool(cmd->pool);
-
-  curr_module = m;
-  res = func(cmd);
-  curr_module = prev_module;
-
-  return res;
-}
-
-modret_t *call_module_cmd(module *m, modret_t *(*func)(cmd_rec*), cmd_rec *cmd)
-{
-  modret_t *res;
-  module *prev_module = curr_module;
-
-  if (!cmd->tmp_pool)
-    cmd->tmp_pool = make_sub_pool(cmd->pool);
-
-  curr_module = m;
-  res = func(cmd);
-  curr_module = prev_module;
-
-  return res;
-}
-
-modret_t *call_module(module *m, modret_t *(*func)(cmd_rec*), cmd_rec *cmd)
-{
+modret_t *call_module(module *m, modret_t *(*func)(cmd_rec *), cmd_rec *cmd) {
   modret_t *res;
   module *prev_module = curr_module;
 
@@ -420,8 +423,7 @@ modret_t *call_module(module *m, modret_t *(*func)(cmd_rec*), cmd_rec *cmd)
   return res;
 }
 
-modret_t *mod_create_data(cmd_rec *cmd,void *d)
-{
+modret_t *mod_create_data(cmd_rec *cmd,void *d) {
   modret_t *ret;
 
   ret = pcalloc(cmd->tmp_pool,sizeof(modret_t));
@@ -430,8 +432,7 @@ modret_t *mod_create_data(cmd_rec *cmd,void *d)
   return ret;
 }
 
-modret_t *mod_create_ret(cmd_rec *cmd,unsigned char err,char *n,char *m)
-{
+modret_t *mod_create_ret(cmd_rec *cmd, unsigned char err, char *n, char *m) {
   modret_t *ret;
 
   ret = pcalloc(cmd->tmp_pool,sizeof(modret_t));
@@ -445,8 +446,7 @@ modret_t *mod_create_ret(cmd_rec *cmd,unsigned char err,char *n,char *m)
   return ret;
 }
 
-modret_t *mod_create_error(cmd_rec *cmd,int mr_errno)
-{
+modret_t *mod_create_error(cmd_rec *cmd, int mr_errno) {
   modret_t *ret;
 
   ret = pcalloc(cmd->tmp_pool,sizeof(modret_t));
@@ -511,6 +511,18 @@ void list_modules(void) {
   }
 }
 
+int module_daemon_startup(void) {
+  module_cb_t *di = NULL;
+
+  if (!daemon_startups)
+    return 0;
+
+  for (di = (module_cb_t *) daemon_startups->xas_list; di; di = di->next)
+    di->module_cb();
+
+  return 0;
+}
+
 int module_preparse_init(void) {
   int numconf = 0,numcmd = 0,numauth = 0;
   module *m;
@@ -519,11 +531,11 @@ int module_preparse_init(void) {
   authtable *auth = NULL;
   register unsigned int i = 0;
 
-  installed_modules = xaset_create(permanent_pool,NULL);
+  installed_modules = xaset_create(permanent_pool, NULL);
 
   for (i = 0; static_modules[i]; i++) {
     m = static_modules[i];
-    m->priority = m->auth_priority = i;
+    m->priority = i;
 
     if (m->api_version < PR_MODULE_API_VERSION) {
       log_pri(PR_LOG_ERR, "Fatal: module '%s' API version (0x%x) is too old "
@@ -580,7 +592,14 @@ int module_preparse_init(void) {
         memcpy(cmdtab, cmd, sizeof(cmdtable));
         cmdtab->m = m;
 
-        pr_stash_add_symbol(PR_SYM_CMD, cmdtab);
+        /* HOOKs and CMDs share the cmdtable type, so check the cmd_type
+         * when adding this symbol to the stash.
+         */
+        if (cmdtab->cmd_type == HOOK)
+          pr_stash_add_symbol(PR_SYM_HOOK, cmdtab);
+
+        else
+          pr_stash_add_symbol(PR_SYM_CMD, cmdtab);
       }
     }
 
@@ -608,19 +627,34 @@ int module_preparse_init(void) {
 }
 
 int module_postparse_init(void) {
-  postparse_t *pp = NULL;
+  module_cb_t *pp = NULL;
 
   if (!postparse_inits)
     return 0;
 
-  for (pp = (postparse_t *) postparse_inits->xas_list; pp; pp = pp->next)
-    pp->module_postparse_init_cb();
+  for (pp = (module_cb_t *) postparse_inits->xas_list; pp; pp = pp->next)
+    pp->module_cb();
 
   return 0;
 }
 
+void pr_register_daemon_startup(int (*cb)(void)) {
+  module_cb_t *di = NULL;
+
+  if (!daemon_cb_pool)
+    daemon_cb_pool = make_sub_pool(permanent_pool);
+
+  if (!daemon_startups)
+    daemon_startups = xaset_create(daemon_cb_pool, NULL);
+
+  di = pcalloc(daemon_cb_pool, sizeof(module_cb_t));
+  di->module_cb = cb;
+
+  xaset_insert(daemon_startups, (xasetmember_t *) di);
+}
+
 void pr_register_postparse_init(int (*cb)(void)) {
-  postparse_t *pp = NULL;
+  module_cb_t *pp = NULL;
 
   if (!postparse_init_pool)
     postparse_init_pool = make_sub_pool(permanent_pool);
@@ -628,12 +662,22 @@ void pr_register_postparse_init(int (*cb)(void)) {
   if (!postparse_inits)
     postparse_inits = xaset_create(postparse_init_pool, NULL);
 
-  pp = pcalloc(postparse_init_pool, sizeof(postparse_t));
-  pp->module_postparse_init_cb = cb;
+  pp = pcalloc(postparse_init_pool, sizeof(module_cb_t));
+  pp->module_cb = cb;
 
   xaset_insert(postparse_inits, (xasetmember_t *) pp);
 }
 
+void module_remove_daemon_startups(void) {
+  if (daemon_startups)
+    daemon_startups = NULL;
+
+  if (daemon_cb_pool) {
+    destroy_pool(daemon_cb_pool);
+    daemon_cb_pool = NULL;
+  }
+}
+
 void module_remove_postparse_inits(void) {
   if (postparse_inits)
     postparse_inits = NULL;
@@ -644,9 +688,11 @@ void module_remove_postparse_inits(void) {
   }
 }
 
-int pr_init_stash(void) {
+int init_stash(void) {
   if (symbol_pool)
-    symbol_pool = make_sub_pool(permanent_pool); 
+    destroy_pool(symbol_pool);
+
+  symbol_pool = make_sub_pool(permanent_pool); 
   memset(symbol_table, '\0', sizeof(symbol_table));
 
   return 0;
