@@ -26,7 +26,7 @@
 
 /*
  * Resource allocation code
- * $Id: pool.c,v 1.31 2003/02/12 08:46:36 castaglia Exp $
+ * $Id: pool.c,v 1.35 2003/09/08 00:29:16 castaglia Exp $
  */
 
 #include "conf.h"
@@ -77,7 +77,7 @@ static void *null_alloc(size_t size) {
   return ret;
 }
 
-static void *xmalloc(size_t size) {
+static void *smalloc(size_t size) {
   void *ret;
 
   ret = malloc(size);
@@ -88,7 +88,7 @@ static void *xmalloc(size_t size) {
 }
 
 #if 0
-void *xcalloc(size_t num, size_t size) {
+void *scalloc(size_t num, size_t size) {
   void *ret;
 
   ret = calloc(num,size);
@@ -97,9 +97,9 @@ void *xcalloc(size_t num, size_t size) {
   return ret;
 }
 
-void *xrealloc(void *p, size_t size) {
+void *srealloc(void *p, size_t size) {
   if (p == 0)
-    return xmalloc(size);
+    return smalloc(size);
   p = realloc(p,size);
   if (p == 0)
     p = null_alloc(size);
@@ -113,7 +113,7 @@ void *xrealloc(void *p, size_t size) {
 
 static union block_hdr *malloc_block(int size) {
   union block_hdr *blok =
-    (union block_hdr *) xmalloc(size + sizeof(union block_hdr));
+    (union block_hdr *) smalloc(size + sizeof(union block_hdr));
 
   blok->h.next = NULL;
   blok->h.first_avail = (char *) (blok + 1);
@@ -200,7 +200,7 @@ static union block_hdr *new_block(int min_size) {
 static unsigned long bytes_in_block_list(union block_hdr *blok) {
   unsigned long size = 0;
 
-  while(blok) {
+  while (blok) {
     size += blok->h.endp - (char *) (blok + 1);
     blok = blok->h.next;
   }
@@ -291,10 +291,6 @@ void debug_walk_pools(void) {
   debug_pool_info();
 }
 
-
-#if 0
-/* NOTE: not used at present */
-
 /* Release the entire free block list */
 static void pool_release_free_block_list(void) {
   union block_hdr *blok,*next;
@@ -310,7 +306,6 @@ static void pool_release_free_block_list(void) {
 
   pr_alarms_unblock();
 }
-#endif
 
 struct pool *make_sub_pool(struct pool *p) {
   union block_hdr *blok;
@@ -344,11 +339,17 @@ struct pool *make_sub_pool(struct pool *p) {
 
 /* Initialize the pool system by creating the base permanent_pool. */
 
-void pr_init_pools(void) {
+void init_pools(void) {
   if (!permanent_pool)
     permanent_pool = make_sub_pool(NULL);
 }
 
+void free_pools(void) {
+  destroy_pool(permanent_pool);
+  permanent_pool = NULL;
+  pool_release_free_block_list();
+}
+
 static void clear_pool(struct pool *p) {
 
   /* Sanity check. */
@@ -376,6 +377,9 @@ static void clear_pool(struct pool *p) {
 }
 
 void destroy_pool(pool *p) {
+  if (p == NULL)
+    return;
+
   pr_alarms_block();
 
   if (p->parent) {
@@ -388,9 +392,9 @@ void destroy_pool(pool *p) {
     if (p->sub_next)
       p->sub_next->sub_prev = p->sub_prev;
   }
-
   clear_pool(p);
   free_blocks(p->first);
+
   pr_alarms_unblock();
 }
 
@@ -405,7 +409,7 @@ static long bytes_in_free_blocks(void) {
 }
 #endif
 
-/* Allocation inteface ...
+/* Allocation interface...
  */
 
 void *palloc(struct pool *p, int reqsize) {
@@ -490,7 +494,7 @@ char *pdircat(pool *p, ...) {
 
   while ((res = va_arg(dummy, char *)) != NULL) {
     /* If the first argument is "", we have to account for a leading /
-     * which must be added.  -jss 3/2/2001
+     * which must be added.
      */
     if (!count++ && !*res)
       len++;
@@ -509,7 +513,7 @@ char *pdircat(pool *p, ...) {
 
   last = 0;
 
-  while((argp = va_arg(dummy, char *)) != NULL) {
+  while ((argp = va_arg(dummy, char *)) != NULL) {
     if (last && last == '/' && *argp == '/')
       argp++;
     else if (last && last != '/' && *argp != '/')
@@ -590,7 +594,7 @@ void array_cat(array_header *dst, const array_header *src)
 
     if (new_size == 0) ++new_size;
 
-    while(dst->nelts + src->nelts > new_size)
+    while (dst->nelts + src->nelts > new_size)
       new_size *= 2;
 
     new_data = pcalloc(dst->pool, elt_size * new_size);
@@ -707,7 +711,7 @@ void run_cleanup(pool *p, void *data, void (*cleanup_cb)(void *)) {
 #endif
 
 static void run_cleanups(cleanup_t *c) {
-  while(c) {
+  while (c) {
     (*c->plain_cleanup_cb)(c->data);
     c = c->next;
   }
