@@ -27,7 +27,7 @@
 /* Various basic support routines for ProFTPD, used by all modules
  * and not specific to one or another.
  *
- * $Id: support.c,v 1.57 2003/01/25 02:38:00 castaglia Exp $
+ * $Id: support.c,v 1.61 2003/08/13 06:07:39 castaglia Exp $
  */
 
 #include "conf.h"
@@ -43,7 +43,7 @@
 #endif
 
 #ifdef AIX3
-#include <sys/statfs.h>
+# include <sys/statfs.h>
 #endif
 
 static pool *exithandler_pool = NULL;
@@ -137,9 +137,8 @@ void run_exit_handlers(void) {
     e->exit_cb();
 }
 
-void schedule(void (*f)(void*,void*,void*,void*),int nloops,
-              void *a1, void *a2, void *a3, void *a4)
-{
+void schedule(void (*f)(void*,void*,void*,void*),int nloops, void *a1,
+    void *a2, void *a3, void *a4) {
   pool *p, *sub_pool;
   sched_t *s;
 
@@ -180,59 +179,53 @@ void run_schedule(void) {
   }
 }
 
-/*
-** Get the maximum size of a file name (pathname component).
-** If a directory file descriptor, e.g. the d_fd DIR structure element,
-** is not available, the second argument should be 0.
-**
-** Note: a POSIX compliant system typically should NOT define NAME_MAX,
-** since the value almost certainly varies across different file system types.
-** Refer to POSIX 1003.1a, Section 2.9.5, Table 2-5.
-** Alas, current (Jul 2000) Linux systems define NAME_MAX anyway.
-** NB: NAME_MAX_GUESS is defined in support.h.
-*/
-int
-get_name_max(char *dirname, int dir_fd)
-{
-	int	name_max = 0;
+/* Get the maximum size of a file name (pathname component).
+ * If a directory file descriptor, e.g. the d_fd DIR structure element,
+ * is not available, the second argument should be 0.
+ *
+ * Note: a POSIX compliant system typically should NOT define NAME_MAX,
+ * since the value almost certainly varies across different file system types.
+ * Refer to POSIX 1003.1a, Section 2.9.5, Table 2-5.
+ * Alas, current (Jul 2000) Linux systems define NAME_MAX anyway.
+ * NB: NAME_MAX_GUESS is defined in support.h.
+ */
+int get_name_max(char *dirname, int dir_fd) {
+  int name_max = 0;
 #if defined(HAVE_FPATHCONF) || defined(HAVE_PATHCONF)
-	char	*msgfmt = "";
+  char *msgfmt = "";
 
 # if defined(HAVE_FPATHCONF)
-	if ( dir_fd > 0 ) {
-		name_max = fpathconf(dir_fd, _PC_NAME_MAX);
-		msgfmt = "fpathconf(%s, _PC_NAME_MAX) = %d, errno = %d";
-	}
-	else
+  if (dir_fd > 0) {
+    name_max = fpathconf(dir_fd, _PC_NAME_MAX);
+    msgfmt = "fpathconf(%s, _PC_NAME_MAX) = %d, errno = %d";
+  } else
 # endif
 # if defined(HAVE_PATHCONF)
-	if ( dirname != NULL ) {
-		name_max = pathconf(dirname, _PC_NAME_MAX);
-		msgfmt = "pathconf(%s, _PC_NAME_MAX) = %d, errno = %d";
-	}
-	else
+  if (dirname != NULL) {
+    name_max = pathconf(dirname, _PC_NAME_MAX);
+    msgfmt = "pathconf(%s, _PC_NAME_MAX) = %d, errno = %d";
+  } else
 # endif
-		/* no data provided to use either pathconf() or fpathconf() */
-		return -1;
-	if ( name_max < 0 ) {
-		/*
-		** NB: errno may not be set if the failure is due
-		** to a limit or option not being supported.
-		*/
-		log_debug(DEBUG1, msgfmt,
-				dirname ? dirname : "(NULL)", name_max, errno);
-	}
-#else
-	name_max = NAME_MAX_GUESS;
-#endif
+  /* No data provided to use either pathconf() or fpathconf() */
+  return -1;
 
-	return name_max;
+  if (name_max < 0) {
+    /* NB: errno may not be set if the failure is due to a limit or option
+     * not being supported.
+     */
+    log_debug(DEBUG1, msgfmt, dirname ? dirname : "(NULL)", name_max, errno);
+  }
+
+#else
+  name_max = NAME_MAX_GUESS;
+#endif /* HAVE_FPATHCONF or HAVE_PATHCONF */
+
+  return name_max;
 }
 
 
 /* Interpolates a pathname, expanding ~ notation if necessary
  */
-
 char *dir_interpolate(pool *p, const char *path) {
   struct passwd *pw;
   char *user,*tmp;
@@ -267,30 +260,36 @@ char *dir_interpolate(pool *p, const char *path) {
 /* dir_best_path() creates the "most" fully canonicalized path possible
  * (i.e. if path components at the end don't exist, they are ignored
  */
-
 char *dir_best_path(pool *p, const char *path) {
-  char workpath[MAXPATHLEN + 1] = {'\0'};
-  char realpath_buf[MAXPATHLEN + 1] = {'\0'};
+  char workpath[PR_TUNABLE_PATH_MAX + 1] = {'\0'};
+  char realpath_buf[PR_TUNABLE_PATH_MAX + 1] = {'\0'};
   char *target = NULL, *ntarget;
   int fini = 0;
 
   if (*path == '~') {
-    if (pr_fs_interpolate(path, workpath, MAXPATHLEN) != 1)
-      pr_fs_dircat(workpath, sizeof(workpath), pr_fs_getcwd(), path);
+    if (pr_fs_interpolate(path, workpath, sizeof(workpath)-1) != 1) {
+      if (pr_fs_dircat(workpath, sizeof(workpath), pr_fs_getcwd(), path) < 0)
+        return NULL;
+    }
 
-  } else
-    pr_fs_dircat(workpath, sizeof(workpath), pr_fs_getcwd(), path);
+  } else {
+    if (pr_fs_dircat(workpath, sizeof(workpath), pr_fs_getcwd(), path) < 0)
+      return NULL;
+  }
 
-  pr_fs_clean_path(pstrdup(p, workpath), workpath, MAXPATHLEN);
+  pr_fs_clean_path(pstrdup(p, workpath), workpath, sizeof(workpath)-1);
 
   while (!fini && *workpath) {
-    if (pr_fs_resolve_path(workpath, realpath_buf, MAXPATHLEN, 0) != -1)
+    if (pr_fs_resolve_path(workpath, realpath_buf,
+        sizeof(realpath_buf)-1, 0) != -1)
       break;
 
     ntarget = strrchr(workpath, '/');
     if (ntarget) {
-      if (target)
-        pr_fs_dircat(workpath, sizeof(workpath), workpath, target);
+      if (target) {
+        if (pr_fs_dircat(workpath, sizeof(workpath), workpath, target) < 0)
+          return NULL;
+      }
 
       target = ntarget;
       *target++ = '\0';
@@ -300,56 +299,66 @@ char *dir_best_path(pool *p, const char *path) {
   }
 
   if (!fini && *workpath) {
-    if (target)
-      pr_fs_dircat(workpath, sizeof(workpath), realpath_buf, target);
+    if (target) {
+      if (pr_fs_dircat(workpath, sizeof(workpath), realpath_buf, target) < 0)
+        return NULL;
 
-    else
+    } else
       sstrncpy(workpath, realpath_buf, sizeof(workpath));
 
-  } else
-    pr_fs_dircat(workpath, sizeof(workpath), "/", target);
+  } else {
+    if (pr_fs_dircat(workpath, sizeof(workpath), "/", target) < 0)
+      return NULL;
+  }
 
   return pstrdup(p, workpath);
 }
 
 char *dir_canonical_path(pool *p, const char *path) {
-  char buf[MAXPATHLEN + 1]  = {'\0'};
-  char work[MAXPATHLEN + 1] = {'\0'};
+  char buf[PR_TUNABLE_PATH_MAX + 1]  = {'\0'};
+  char work[PR_TUNABLE_PATH_MAX + 1] = {'\0'};
 
   if (*path == '~') {
-    if (pr_fs_interpolate(path, work, MAXPATHLEN) != 1)
-      pr_fs_dircat(work, sizeof(work), pr_fs_getcwd(), path);
+    if (pr_fs_interpolate(path, work, sizeof(work)-1) != 1) {
+      if (pr_fs_dircat(work, sizeof(work), pr_fs_getcwd(), path) < 0)
+        return NULL;
+    }
 
-  } else
-    pr_fs_dircat(work, sizeof(work), pr_fs_getcwd(), path);
+  } else {
+    if (pr_fs_dircat(work, sizeof(work), pr_fs_getcwd(), path) < 0)
+      return NULL;
+  }
 
-  pr_fs_clean_path(work, buf, MAXPATHLEN);
+  pr_fs_clean_path(work, buf, sizeof(buf)-1);
   return pstrdup(p, buf);
 }
 
 char *dir_canonical_vpath(pool *p, const char *path) {
-  char buf[MAXPATHLEN + 1]  = {'\0'};
-  char work[MAXPATHLEN + 1] = {'\0'};
+  char buf[PR_TUNABLE_PATH_MAX + 1]  = {'\0'};
+  char work[PR_TUNABLE_PATH_MAX + 1] = {'\0'};
 
   if (*path == '~') {
-    if (pr_fs_interpolate(path, work, MAXPATHLEN) != 1)
-      pr_fs_dircat(work, sizeof(work), pr_fs_getvwd(), path);
+    if (pr_fs_interpolate(path, work, sizeof(work)-1) != 1) {
+      if (pr_fs_dircat(work, sizeof(work), pr_fs_getvwd(), path) < 0)
+        return NULL;
+    }
 
-  } else
-    pr_fs_dircat(work, sizeof(work), pr_fs_getvwd(), path);
+  } else {
+    if (pr_fs_dircat(work, sizeof(work), pr_fs_getvwd(), path) < 0)
+      return NULL;
+  }
 
-  pr_fs_clean_path(work, buf, MAXPATHLEN);
+  pr_fs_clean_path(work, buf, sizeof(buf)-1);
   return pstrdup(p, buf);
 }
 
 /* dir_realpath() is needed to properly dereference symlinks (getcwd() may
  * not work if permissions cause problems somewhere up the tree).
  */
-
 char *dir_realpath(pool *p, const char *path) {
-  char buf[MAXPATHLEN + 1] = {'\0'};
+  char buf[PR_TUNABLE_PATH_MAX + 1] = {'\0'};
 
-  if (pr_fs_resolve_partial(path, buf, MAXPATHLEN, 0) == -1)
+  if (pr_fs_resolve_partial(path, buf, sizeof(buf)-1, 0) == -1)
     return NULL;
 
   return pstrdup(p, buf);
@@ -360,7 +369,6 @@ char *dir_realpath(pool *p, const char *path) {
  * a *full* reference based on the root directory, not upon a chrooted
  * dir.
  */
-
 char *dir_abs_path(pool *p, const char *path, int interpolate) {
   char *res = NULL;
 
@@ -385,12 +393,12 @@ char *dir_abs_path(pool *p, const char *path, int interpolate) {
   return res;
 }
 
-/* Return the mode (including the file type)
-   of the file pointed to by symlink PATH, or 0 if it doesn't exist.
-   Catch symlink loops using LAST_INODE and RCOUNT.  */
-
+/* Return the mode (including the file type) of the file pointed to by symlink
+ * PATH, or 0 if it doesn't exist. Catch symlink loops using LAST_INODE and
+ * RCOUNT.
+ */
 static mode_t _symlink(char *path, ino_t last_inode, int rcount) {
-  char buf[MAXPATHLEN + 1];
+  char buf[PR_TUNABLE_PATH_MAX + 1];
   struct stat sbuf;
   int i;
 
@@ -441,10 +449,10 @@ mode_t file_mode(char *path) {
 }
 
 /* If DIRP == 1, fail unless PATH is an existing directory.
-   If DIRP == 0, fail unless PATH is an existing non-directory.
-   If DIRP == -1, fail unless PATH exists; the caller doesn't care whether
-   PATH is a file or a directory. */
-
+ * If DIRP == 0, fail unless PATH is an existing non-directory.
+ * If DIRP == -1, fail unless PATH exists; the caller doesn't care whether
+ * PATH is a file or a directory.
+ */
 static int _exists(char *path, int dirp) {
   mode_t fmode;
 
@@ -565,7 +573,6 @@ char *strip_end(char *s, char *ch) {
  * the next non-separator in the string.  If the src string is
  * empty or NULL, the next token returned is NULL.
  */
-
 char *get_token(char **s, char *sep) {
   char *res;
 
@@ -574,11 +581,11 @@ char *get_token(char **s, char *sep) {
 
   res = *s;
 
-  while(**s && !strchr(sep,**s)) (*s)++;
+  while (**s && !strchr(sep,**s))
+    (*s)++;
 
-  if (**s) {
+  if (**s)
     *(*s)++ = '\0';
-  }
 
   return res;
 }
@@ -588,24 +595,26 @@ char *get_token(char **s, char *sep) {
  * never returns NULL, only an empty string if no token remains
  * in the source string.
  */
-
 char *safe_token(char **s) {
   char *res = "";
 
   if (!s || !*s)
     return res;
 
-  while (isspace((int) **s) && **s) (*s)++;
+  while (isspace((int) **s) && **s)
+    (*s)++;
 
   if (**s) {
     res = *s;
 
-    while (!isspace((int) **s) && **s) (*s)++;
+    while (!isspace((int) **s) && **s)
+      (*s)++;
 
     if (**s)
       *(*s)++ = '\0';
 
-    while (isspace((int) **s) && **s) (*s)++;
+    while (isspace((int) **s) && **s)
+      (*s)++;
   }
 
   return res;
@@ -615,7 +624,6 @@ char *safe_token(char **s) {
  * filled with the times to deny new connections and disconnect
  * existing ones.
  */
-
 int check_shutmsg(time_t *shut, time_t *deny, time_t *disc, char *msg,
                   size_t msg_size) {
   FILE *fp;
@@ -714,20 +722,20 @@ char *sreplace(pool *p, char *s, ...) {
   char *m,*r,*src = s,*cp;
   char **mptr,**rptr;
   char *marr[33],*rarr[33];
-  char buf[2048] = {'\0'}, *pbuf = NULL;
-  int  mlen = 0, rlen = 0;
-  int  blen, dyn=1;
+  char buf[PR_TUNABLE_PATH_MAX] = {'\0'}, *pbuf = NULL;
+  int mlen = 0, rlen = 0;
+  int blen, dyn = 1;
 
   cp = buf;
   *cp = '\0';
 
-  memset(marr,'\0',sizeof(marr));
-  memset(rarr,'\0',sizeof(rarr));
-  blen=strlen(src)+1;
+  memset(marr, '\0', sizeof(marr));
+  memset(rarr, '\0', sizeof(rarr));
+  blen = strlen(src) + 1;
 
-  va_start(args,s);
+  va_start(args, s);
 
-  while((m = va_arg(args, char *)) != NULL && mlen < 32) {
+  while ((m = va_arg(args, char *)) != NULL && mlen < 32) {
     if ((r = va_arg(args, char *)) == NULL)
       break;
     blen += (strlen(r) - strlen(m));
@@ -737,37 +745,37 @@ char *sreplace(pool *p, char *s, ...) {
 
   va_end(args);
 
-  /* Try to handle large buffer situations (i.e. escaping of MAXPATHLEN
+  /* Try to handle large buffer situations (i.e. escaping of PR_TUNABLE_PATH_MAX
    * (>2048) correctly, but do not allow very big buffer sizes, that may
    * be dangerous (BUFSIZ may be defined in stdio.h) in some library
    * functions.
    */
 #ifndef BUFSIZ
-#define BUFSIZ 8192
+# define BUFSIZ 8192
 #endif
 
-  if (blen < BUFSIZ) {
+  if (blen < BUFSIZ)
     cp = pbuf = (char *) pcalloc(p, ++blen);
-  }
 
   if (!pbuf) {
-    cp   = pbuf = buf;
-    dyn  = 0;
+    cp = pbuf = buf;
+    dyn = 0;
     blen = sizeof(buf);
   }
 
-  while(*src) {
+  while (*src) {
     for (mptr = marr, rptr = rarr; *mptr; mptr++, rptr++) {
       mlen = strlen(*mptr);
       rlen = strlen(*rptr);
 
-      if (strncmp(src,*mptr,mlen) == 0) {
-        sstrncpy(cp,*rptr, blen - strlen(pbuf));
+      if (strncmp(src, *mptr, mlen) == 0) {
+        sstrncpy(cp, *rptr, blen - strlen(pbuf));
 	if (((cp + rlen) - pbuf + 1) > blen) {
 	  log_pri(PR_LOG_ERR,
-		  "Warning, attempt to overflow internal ProFTPD buffers.");
+		  "WARNING: attempt to overflow internal ProFTPD buffers");
 	  cp = pbuf + blen - 1;
 	  goto done;
+
 	} else {
 	  cp += rlen;
 	}
@@ -780,7 +788,7 @@ char *sreplace(pool *p, char *s, ...) {
     if (!*mptr) {
       if ((cp - pbuf + 1) > blen) {
 	log_pri(PR_LOG_ERR,
-		"Warning, attempt to overflow internal ProFTPD buffers.");
+		"WARNING: attempt to overflow internal ProFTPD buffers");
 	cp = pbuf + blen - 1;
       }
       *cp++ = *src++;
@@ -792,7 +800,8 @@ char *sreplace(pool *p, char *s, ...) {
 
   if (dyn)
     return pbuf;
-  return pstrdup(p,buf);
+
+  return pstrdup(p, buf);
 }
 
 /* "safe" memset() (code borrowed from OpenSSL).  This function should be
@@ -820,34 +829,14 @@ void pr_memscrub(void *ptr, size_t ptrlen) {
 /* "safe" strcat, saves room for \0 at end of dest, and refuses to copy
  * more than "n" bytes.
  */
-
 char *sstrcat(char *dest, const char *src, size_t n) {
   register char *d;
 
   for (d = dest; *d && n > 1; d++, n--) ;
 
-  while(n-- > 1 && *src)
+  while (n-- > 1 && *src)
     *d++ = *src++;
 
   *d = 0;
   return dest;
 }
-
-/* "safe" strncpy, saves room for \0 at end of dest, and refuses to copy
- * more than "n" bytes.
- */
-char *sstrncpy(char *dest, const char *src, size_t n) {
-  register char *d = dest;
-
-  if (!dest)
-    return NULL;
-
-  if (src && *src) {
-    for (; *src && n > 1; n--)
-      *d++ = *src++;
-  }
-
-  *d = '\0';
-
-  return dest;
-}
