@@ -1,7 +1,7 @@
 /*
  * ProFTPD - FTP server daemon
  * Copyright (c) 1997, 1998 Public Flood Software
- * Copyright (c) 2001, 2002 The ProFTPD Project team
+ * Copyright (c) 2001, 2002, 2003 The ProFTPD Project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,15 +23,15 @@
  * the source code for OpenSSL in the source distribution.
  */
 
-/* 
+/*
  * Timer system, based on alarm() and SIGALRM
- * $Id: timers.c,v 1.12 2002/07/09 22:20:10 castaglia Exp $
+ * $Id: timers.c,v 1.18 2003/01/07 17:31:08 castaglia Exp $
  */
 
-#include <signal.h>
-
 #include "conf.h"
 
+#include <signal.h>
+
 /* From src/main.c */
 volatile extern unsigned int recvd_signal_flags;
 
@@ -77,7 +77,7 @@ static int process_timers(int elapsed) {
     return 0;
 
   _indispatch++;
-  block_alarms();
+  pr_alarms_block();
 
   if (elapsed) {
     for (t = (timer_t *) timers->xas_list; t; t=next) {
@@ -119,7 +119,7 @@ static int process_timers(int elapsed) {
     xaset_insert_sort(timers, (xasetmember_t *) t, TRUE);
   }
 
-  unblock_alarms();
+  pr_alarms_unblock();
   _indispatch--;
 
   /* If no active timers remain in the list, there is no reason to set the
@@ -215,7 +215,7 @@ int reset_timer(int timerno, module *mod) {
   if (_indispatch)
     return -1;
 
-  block_alarms();
+  pr_alarms_block();
 
   if (!recycled)
     recycled = xaset_create(NULL, NULL);
@@ -235,7 +235,7 @@ int reset_timer(int timerno, module *mod) {
       break;
     }
 
-  unblock_alarms();
+  pr_alarms_unblock();
 
   return (t ? t->timerno : 0);
 }
@@ -247,7 +247,7 @@ int remove_timer(int timerno, module *mod) {
   if (!timers)
     return 0;
 
-  block_alarms();
+  pr_alarms_block();
 
   for (t = (timer_t *) timers->xas_list; t; t = t->next)
     if (t->timerno == timerno && (t->mod == mod || mod == ANY_MODULE)) {
@@ -255,8 +255,8 @@ int remove_timer(int timerno, module *mod) {
         t->remove++;
 
       } else {
-        xaset_remove(timers,(xasetmember_t*)t);
-        xaset_insert(free_timers,(xasetmember_t*)t);
+        xaset_remove(timers, (xasetmember_t*)t);
+        xaset_insert(free_timers, (xasetmember_t*)t);
 	nalarms++;
 
         /* The handle_alarm() function also readjusts the timers lists
@@ -264,11 +264,11 @@ int remove_timer(int timerno, module *mod) {
          * is removed.
          */
         handle_alarm();
-      }      
+      }
       break;
     }
 
-  unblock_alarms();
+  pr_alarms_unblock();
 
   return (t ? t->timerno : 0);
 }
@@ -283,14 +283,14 @@ int add_timer(int seconds, int timerno, module *mod, callback_t cb) {
     free_timers = xaset_create(NULL, NULL);
 
   /* Try to use an old timer first */
-  block_alarms();
+  pr_alarms_block();
   if ((t = (timer_t*)free_timers->xas_list) != NULL)
     xaset_remove(free_timers, (xasetmember_t*)t);
   else
     /* Must allocate a new one */
     t = palloc(permanent_pool, sizeof(timer_t));
 
-  if (timerno == -1) { 
+  if (timerno == -1) {
     /* Dynamic timer */
     if (dynamic_timerno < 1024)
       dynamic_timerno = 1024;
@@ -324,7 +324,7 @@ int add_timer(int seconds, int timerno, module *mod, callback_t cb) {
     handle_alarm();
   }
 
-  unblock_alarms();
+  pr_alarms_unblock();
 
   return timerno;
 }
@@ -335,11 +335,11 @@ int add_timer(int seconds, int timerno, module *mod, callback_t cb) {
  * block/unblock functions.
  */
 
-void block_alarms(void) {
+void pr_alarms_block(void) {
   ++alarms_blocked;
 }
 
-void unblock_alarms(void) {
+void pr_alarms_unblock(void) {
   --alarms_blocked;
   if (alarms_blocked == 0 && alarm_pending) {
     alarm_pending = 0;
@@ -370,6 +370,6 @@ int timer_sleep(int seconds) {
     sigsuspend(&oset);
     handle_alarm();
   }
-  
-  return 0;  
+
+  return 0;
 }
